!***********************************************************************************************************************************
!  Calculation of Rotor Rotational, Thickness, and Broadband Noise from CAMRADII Output
!
!  Wayne Johnson
!  Johnson Aeronautics
!  November 2019
!
!  Copyright 2019 Wayne Johnson
!***********************************************************************************************************************************
!  Theory and Input: "Rotor Noise from CAMRAD II" (file CII-noise.pdf)
!
!  Blade loading, motion, velocity obtained from sound sensor.
!  Best results with high azimuthal resolution, using post-trim analysis (but can also use standard resolution, without post-trim).
!  For BVI airloads using lifting-line theory and a small azimuthal step size, a large vortex core radius is recommended.
!  Typical input --
!  NLJOB: OUTRES=0 (standard) or OUTRES=1 (enhanced)
!  TRIM: MPSI=24,OPPOST=1,MPSIH=240, ! post trim analysis
!  TRIM ROTOR: MPSEN=1,MPTIME=240, ! sound sensor
!  ROTOR STRUCTURE: OPSND=1, ! sound sensor
!  ROTOR WAKE: CORE=0.8,COREWG=0.8, ! vortex core size
!
!  Must use same aircraft (NROTOR,NBLADE,NPANEL) and same OPSND for all cases in the CAMRADII job.
!
!  Brooks broadband noise model requires CAMRADII aerodynamic sensors: U,alpha,cl (all three, in that order).
!  Typical input --
!  ROTOR AERODYNAMICS: NSEN=3,QUANT=24,25,61,IDENT=0,0,1,OPSCL=0,1,0, ! aero sensors
!  TRIM ROTOR: MASEN=1,MATIME=240 ! aero sensors
!
!  CIIsound.f90 is a Fortran program; should be compiled with double precision
!  Intel compiler: ifort -O1 -fpe0 -traceback -static-intel -r8 -i8 CIIsound.f90 -o CIIsound
!***********************************************************************************************************************************
!  CIIsound:  Calculation of Rotor Rotational, Thickness, and Broadband Noise from CAMRADII Output
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      module CIIsound_structures
!
      integer, parameter:: ncasemax=10         ! number of cases
      integer, parameter:: nobsmax=100         ! number of observers
!-------------------------------------------------------------------------------------------
!                          CIIsound              CAMRADII
      integer, parameter:: nrtrmax=20          ! NROTOR=40     maximum number of rotors
      integer, parameter:: nbldmax=10          ! CWKWNG=40     wake and inflow, number of wings
      integer, parameter:: npropsmax=201       ! SRWSNX=201    rotor structure, number of radial stations
      integer, parameter:: npropamax=201       ! CLLWNX=200    lifting line wing, number of span stations (and aero panels)
      integer, parameter:: npnlmax=200         ! CLLWNX=200    lifting line wing, number of span stations (and aero panels)
      integer, parameter:: npnlemax=201        ! CLLWNE=201    lifting line wing, number of panel edges (CLLWNX+1)
!-------------------------------------------------------------------------------------------
      integer, parameter:: mpsimax=360         ! number of azimuth steps
      integer, parameter:: ntimemax=2048       ! number of time steps
      integer, parameter:: ntimemax2=ntimemax/2
      integer, parameter:: lfftwork=ntimemax+20
!
      TYPE InputParam
         SEQUENCE
         integer KIND_obs(ncasemax)      ! observer definition (0 trajectory, 1 input (airframe), 2 input (inertial))
                                         ! trajectory (KIND_obs=0)
         integer KIND_traj(ncasemax)     !    trajectory definition (0 general (xref,zref), 1 takeoff, 2 flyover, 3 approach)
         integer KIND_mic(ncasemax)      !    microphone definition (0 general (xmic), 1 center, 2 starboard, 3 port)
         integer Ntraj                   !    number of trajectory steps
         real xbegin(ncasemax)           !    beginning aircraft position, negative for microphone ahead (m)
         real xend(ncasemax)             !    ending aircraft position (m)
         real xref(ncasemax)             !    trajectory reference position (m)
         real zref(ncasemax)             !    trajectory altitude above ground level, at xref (m)
         real xmic(3,ncasemax)           !    microphone position, inertial axes (m)
                                         ! input observers (KIND_obs=1 or 2)
         integer nObserver(ncasemax)     !    number of observers
         real position_obs(3,nobsmax,ncasemax) 
                                         !    observer position, relative airframe origin, airframe or inertial axes (m)
                                         ! analysis
         integer Ntime                   !    number of time steps
         integer Nspan                   !    number of radial stations, thickness noise (maximum npropamax; root cutout to tip)
         integer interp                  !    loading interpolation (1 linear, 2 harmonic, 3 linear-harmonic)
         integer NfreqBVI(2)             !    blade passage frequency range for BVI SPL
                                         ! model
         integer nprop                   !    number radial stations
         real rprop(npropamax,nrtrmax)   !    radial stations (r/R)
         real thick(npropamax,nrtrmax)   !    thickness ratio (Axs=0.685*thick*chord**2)
         integer MODEL_broadband         !    broadband noise model (1 Pegg, 2 Brooks)
         integer OMIT_broadband          !    broadband sound (1 to omit from total sound)
                                         ! Brooks broadband noise model
         integer READ_aero               !    read aerodynamic sensors (0 not); required for Brooks broadband model
         integer OMIT_TBLTE              !    turbulent boundary-layer, trailing edge noise (1 to omit)
         integer OMIT_LBLVS              !    laminar boundary-layer, vortex shedding noise (1 to omit)
         integer OMIT_BLUNT              !    trailing-edge bluntness noise (1 to omit)
         integer OMIT_TIP                !    tip vortex formation noise (1 to omit)
         integer MODEL_direct            !    directivity (0 Brooks, 1 ROTONET)
         integer MODEL_alpha             !    alpha_star (1 from angle-of-attack, 2 from lift coefficient)
         integer MODEL_trip              !    boundary layer (0 untripped (natural transition), 1 tripped, 2 lightly tripped)
         integer Nbbave                  !    number of time steps in average
         real alpha_zl(npropamax,nrtrmax)!    zero-lift angle-of-attack (deg) (for MODEL_alpha=1)
         real hTE(npropamax,nrtrmax)     !    trailing edge thickness ratio h/c (BLUNT noise)
         real psiTE(npropamax,nrtrmax)   !    trailing edge angle (deg) (BLUNT noise)
         integer isRound(nrtrmax)        !    blade tip cap shape (1 rounded) (TIP noise)
         real ftipalpha(nrtrmax)         !    blade tip angle-of-attack correction factor (TIP noise)
                                         ! output
         integer OUT_perf                !    write performance file (logical name fileperf); 0 no write
         integer OUT_sensor              !    write sound sensor file (logical name filesen); 0 no write, 1 English, 2 SI
         integer OUT_source              !    write sound source file (logical name filesrc); 0 no write
         integer OUT_sound               !    write sound file (logical name filesnd); 0 no write,
                                         !          1 basic, 2 third-octave spectra, 3 narrow band and time history, 4 blades
         integer OUT_soundfiles          !    write sound file (0 single file, 1 separate files)
      ENDTYPE InputParam
!
!        Sound File Output:
!        aircraft: total, broadband, rotational, loading, thickness
!        summed rotors: rotational, loading, thickness
!        rotor n: total, broadband, rotational, loading, thickness
!        rotor n blade m: loading, thickness
!
!
      TYPE SoundValue ! sound calculated (from time domain)
         SEQUENCE
         integer Ntime                         ! number of time steps
         real Omega                            ! rotational speed (rad/sec)
         real period                           ! period=2*pi/Omega (sec)
         integer nblade                        ! number of blades
         real press(0:ntimemax)                ! sound pressure (time)
         real msq_press                        ! mean square pressure (from time)
         real OASPL_press                      ! overall sound pressure level (dB)
         real harm(ntimemax)                   ! sound pressure (harmonic analysis)
         real narrow(ntimemax2)                ! narrow band spectrum (pressure)
         real SPL_narrow(ntimemax2)            ! narrow band spectrum (dB)
         real msq_narrow                       ! mean square pressure (from harm)
         real OASPL_narrow                     ! overall sound pressure level (dB)
         real msq_BVI                          ! BVI mean square pressure (BVI blade passage frequency range)
         real BVISPL                           ! BVI sound pressure level (dB)
         real spectrum_press(0:40)             ! third-octave spectrum (mean square pressure)
         real spectrum(0:40)                   ! third-octave spectrum (dB)
         real OASPL                            ! overall sound pressure level (dB)
         real OASPLA                           ! A-weighted overall sound pressure level (dBA)
      ENDTYPE SoundValue
      TYPE SoundValueF ! sound calculated (spectrum only)
         SEQUENCE
         real spectrum_press(0:40)             ! third-octave spectrum (mean square pressure)
         real spectrum(0:40)                   ! third-octave spectrum (dB)
         real OASPL                            ! overall sound pressure level (dB)
         real OASPLA                           ! A-weighted overall sound pressure level (dBA)
         real PNL                              ! perceived noise level (PNdB)
         real PNLT                             ! perceived noise level, tone corrected (PNdB)
      ENDTYPE SoundValueF
!
      TYPE SoundBlade ! sound blade
         SEQUENCE
         integer Ntime                         ! number of time steps
         real Omega                            ! rotational speed (rad/sec)
         real period                           ! period=2*pi/Omega (sec)
         integer freq2band(ntimemax2)          ! third-octave band number from frequency
         type (SoundValue) sound_loading       ! loading sound
         type (SoundValue) sound_thickness     ! thickness sound
      ENDTYPE SoundBlade
      TYPE SoundRotor ! sound rotor
         SEQUENCE
         integer opsnd                         ! sound sensors (0 for none)
         integer nblade                        ! number of blades
         integer npanel                        ! number of aerodynamic panels
         real distance                         ! hub distance (m)
         real elevation                        ! elevation angle (deg)
         real azimuth                          ! azimuth angle (deg)
         type (SoundValue) sound_loading       ! loading sound (sum all blades)
         type (SoundValue) sound_thickness     ! thickness sound (sum all blades)
         type (SoundValue) sound_rotational    ! total sound (sum loading and thickness)
         type (SoundValueF) sound_broadband    ! broadband sound
         type (SoundValueF) sound_bb_TBLTE     ! broadband sound, TBLTE
         type (SoundValueF) sound_bb_LBLVS     ! broadband sound, LBLVS
         type (SoundValueF) sound_bb_BLUNT     ! broadband sound, BLUNT
         type (SoundValueF) sound_bb_TIP       ! broadband sound, TIP
         type (SoundValueF) sound_total        ! total sound (sum loading, thickness, and broadband)
         type (SoundBlade) Blade(nbldmax)
      ENDTYPE SoundRotor
      TYPE SoundData ! sound
         SEQUENCE
         real density            ! density (kg/m3)
         real csound             ! speed of sound (m/sec)
         real kinvisc            ! kinematic viscosity (m2/sec)
         real speed              ! aircraft or wind speed (m/sec)
         integer nrotor          ! number of rotors
         real position_obs(3)    ! observer position
         real position_traj(3)   ! trajectory position
         real time_traj          ! trajectory time
         real position_mic(3)    ! microphone position
         integer primary(nrtrmax)! rotational sound (0 primary, n parent rotor)
         real SEL                ! sound exposure level of trajectory (dB)
         real EPNL               ! effective perceived noise level (dB)
         type (SoundValue) sound_loading(nrtrmax)      ! loading sound (sum all rotors with same rotational speed)
         type (SoundValue) sound_thickness(nrtrmax)    ! thickness sound (sum all rotors with same rotational speed)
         type (SoundValue) sound_rotational(nrtrmax)   ! total sound (sum loading and thickness)
         type (SoundValueF) sound_load_total           ! loading sound (sum all rotors)
         type (SoundValueF) sound_thick_total          ! thickness sound (sum all rotors)
         type (SoundValueF) sound_rot_total            ! rotational sound (sum all rotors)
         type (SoundValueF) sound_broadband            ! broadband sound (sum all rotors)
         type (SoundValueF) sound_total                ! total sound (sum all rotors)
         type (SoundRotor) Rotor(nrtrmax)
      ENDTYPE SoundData
!
      TYPE SourceBlade ! source blade
         SEQUENCE
         integer mpsi                          ! number of azimuth steps
         real Omega                            ! rotational speed (rad/sec)
         real period                           ! period=2*pi/Omega (sec)
         real psi(0:mpsimax)                   ! psi=jpsi*(360/mpsi) (deg)
         real time(0:mpsimax)                  ! time (force, disp, vel)
         real force(0:mpsimax,3,npnlmax)       ! sum of forces, QC+3QC (N)
         real disp(0:mpsimax,3,npnlmax)        ! position relative airframe origin (m)
         real vel(0:mpsimax,3,npnlmax)         ! velocity relative inertial frame (m/sec)
         real disp3(0:mpsimax,3,npnlmax)       ! 3QC position relative airframe origin (m)
         real Umag(0:mpsimax,npnlmax)          ! velocity (ft/sec)
         real alpha(0:mpsimax,npnlmax)         ! angle of attack (deg)
         real cl(0:mpsimax,npnlmax)            ! lift coefficient
         real time_deriv(0:mpsimax)            ! time (force_deriv)
         real force_deriv(0:mpsimax,3,npnlmax) ! d(force)/dt (from finite difference)
         real force_harm(mpsimax,3,npnlmax)    ! harmonics of forces
         real disp_harm(mpsimax,3,npnlmax)     ! harmonics of position
         real vel_harm(mpsimax,3,npnlmax)      ! harmonics of velocity
         real echord(0:mpsimax,3,npnlmax)      ! panel axes, chord
         real etangent(0:mpsimax,3,npnlmax)    ! panel axes, tangent
         real enormal(0:mpsimax,3,npnlmax)     ! panel axes, normal
      ENDTYPE SourceBlade
      TYPE SourceRotor ! source rotor
         SEQUENCE
         integer opsnd           ! sound sensors (0 for none)
         integer nblade          ! number of blades
         integer npanel          ! number of aerodynamic panels
         integer rotate          ! direction of rotation
         real radius             ! radius (m)
         real Vtip               ! tip speed (m/sec)
         real area_rotor         ! rotor area
         real area_blade         ! total blade area = sigma*area_rotor
         real hub_force(3)       ! hub force (N), airframe axes; summed radially and over blades, averaged over azimuth
         real hub_disp(3)        ! hub position (m), airframe axes; root panel, averaged over blades and azimuth
         real rroot              ! root r/R
         real rtip               ! tip r/R
         real dr_thick           ! span station increment
         real Axs(npropamax)     ! cross-section area, Axs=0.685*thick*chord**2 (m2); Nspan stations
         real chord(npnlmax)     ! chord (m)
         real alpha_zl(npnlmax)  ! zero-lift angle-of-attack (deg)
         real hTE(npnlmax)       ! trailing edge thickness h (m)
         real psiTE(npnlmax)     ! trailing edge angle (deg)
         type (SourceBlade) Blade(nbldmax)
      ENDTYPE SourceRotor
      TYPE SourceData ! source
         SEQUENCE
         real density            ! density (kg/m3)
         real csound             ! speed of sound (m/sec)
         real kinvisc            ! kinematic viscosity (m2/sec)
         real speed              ! aircraft or wind speed (m/sec)
         integer nrotor          ! number of rotors
         type (SourceRotor) Rotor(nrtrmax)
      ENDTYPE SourceData
!
      TYPE Blade ! CAMRADII output blade
         SEQUENCE
!        blade sound sensor = loading, displacement, velocity, 3QC displacement; airframe axes (periodic)
!        aerodynamic collocation point (QC), Rotor m Blade m Panel k; jpsi=0,mpsi
!        solve one blade (oppart=1): copy Nblade to m=1,Nblade-1, azref=azref(Nblade)+m*360/Nblade
         integer mpsi                    ! number of azimuth steps
         real azref                      ! azimuth for zero time (deg)
         real Omega                      ! rotational speed (rad/sec)
         real period                     ! period=2*pi/Omega (sec)
         real psi(0:mpsimax)             ! psi=jpsi*(360/mpsi) (deg)
         real time(0:mpsimax)            ! time=radians(psi-azref)/Omega=((psi-azref)/360)*period
         real force(3,npnlmax,0:mpsimax) ! sum of forces, QC+3QC (lb or N)
         real disp(3,npnlmax,0:mpsimax)  ! position relative airframe origin (ft or m)
         real vel(3,npnlmax,0:mpsimax)   ! velocity relative inertial frame (ft/sec or m/sec)
         real disp3(3,npnlmax,0:mpsimax) ! 3QC position relative airframe origin (ft or m)
!        aerodynamic sensors: U,alpha,cl (all three, in that order)
         real Umag(npnlmax,0:mpsimax)    ! velocity (ft/sec or m/sec)
         real alpha(npnlmax,0:mpsimax)   ! angle of attack (deg)
         real cl(npnlmax,0:mpsimax)      ! lift coefficient
      ENDTYPE Blade
      TYPE Rotor ! CAMRADII output rotor
         SEQUENCE
         integer opsnd           ! sound sensors (0 for none)
         integer oppart          ! part definition (1 reference blade, 2-3 all blades)
         integer oprpm           ! transmission configuration (0 primary, n parent rotor number)
         real rgear              ! transmission gear ratio
         real azref              ! rotor azimuth at time=0 (deg)
         integer nblade          ! number of blades
         integer rotate          ! direction of rotation
         real radius             ! radius (ft or m)
         real sigma              ! solidity
         real area_rotor         ! rotor area
         real area_blade         ! total blade area = sigma*area_rotor
         integer npanel          ! number of aerodynamic panels
         real ra(npnlmax)        ! aerodynamic panel mid point (r/R)
         real dra(npnlmax)       ! aerodynamic panel width (Dr/R)
         integer nprop           ! number of aerodynamic properties
         real rprop(npropamax)   ! radial stations (r/R)
         real chord(npropamax)   ! chord (ft or m)
         real hub_force(3)       ! hub force (lb or N), airframe axes; summed radially and over blades, averaged over azimuth
         real hub_disp(3)        ! hub position (ft or m), airframe axes; root panel, averaged over blades and azimuth
         type (Blade) Blade(nbldmax)
      ENDTYPE Rotor
      TYPE Condition ! operating condition
         SEQUENCE
         real density            ! density (slug/ft3 or kg/m3)
         real csound             ! speed of sound (ft/sec or m/sec)
         real kinvisc            ! kinematic viscosity (ft2/sec or m2/sec)
         real Vkts               ! aircraft or wind speed (knots)
         real speed              ! aircraft or wind speed (ft/sec or m/sec)
         real roll               ! airframe orientation relative inertial axes (deg), roll phi_F
         real pitch              ! airframe orientation relative inertial axes (deg), pitch theta_F
         real yaw                ! airframe orientation relative inertial axes (deg), yaw psi_F
         real climb              ! velocity orientation relative inertial axes (deg), climb theta_V
         real side               ! velocity orientation relative inertial axes (deg), side psi_V
         real Vtip(nrtrmax)      ! tip speed (ft/sec or m/sec)
         real Omega(nrtrmax)     ! rotational speed (rad/sec)
         real mu(nrtrmax)        ! advance ratio
         real muz(nrtrmax)       ! axial advance ratio
         real Mtip(nrtrmax)      ! tip Mach number
         real Mat(nrtrmax)       ! advancing tip Mach number
         real CTs(nrtrmax)       ! CT/s
         real CFI(3,3)           ! transformation from inertial to airframe axes, CFI=X(phiF)*Y(thetaF)*Z(psiF)
      ENDTYPE Condition
      TYPE Output ! CAMRADII output for sound
         SEQUENCE
         integer opunit          ! unit system (1 English, 2 SI)
         integer nrotor          ! number of rotors
         integer opfree          ! constraint (0 wind tunnel, 1 free flight)
         type (Condition) Condition
         type (Rotor) Rotor(nrtrmax)
      ENDTYPE Output
!
!   mathematical constants
      real, parameter:: pi=3.141592653589793238462643383280
      real, parameter:: degrad=180./pi
      real, parameter:: raddeg=pi/180.
!   physical constants
      real, parameter:: g=9.80665  ! 32.17405
      real, parameter:: rho=1.225  ! .00237689
      real, parameter:: cs=340.294 ! 1116.45
      real, parameter:: m_ft=.3048
      real, parameter:: m_nm=1852.
      real, parameter:: kg_lbm=.45359237
      real, parameter:: mps_kt=m_nm/3600.
      real, parameter:: N_lb=kg_lbm*g ! 4.4482216152605
!   sound constants
      real, parameter:: pref=0.00002
      real, parameter:: prefsq=pref**2
!   sound arrays
      real, parameter :: SPegg_octave(-17:23) = [-101.,-101.,-29.0,-101.,-101.,-24.5,-101.,-101.,-19.5,-101., &
                                                 -101.,-15.3,-101.,-101.,-11.7,-101.,-101.,-7.5,-101.,-101., &
                                                 -11.5,-101.,-101.,-12.1,-101.,-101.,-16.5,-101.,-101.,-17.0, &
                                                 -101.,-101.,-21.8,-101.,-101.,-26.4,-101.,-101.,-30.0,-101., &
                                                 -101.]
      real, parameter :: SPegg_third(-17:23) = [-38.54,-35.53,-33.77,-31.71,-30.32,-29.27,-26.91,-25.39,-24.27,-22.39, &
                                                -21.08,-20.07,-18.52,-17.37,-16.47,-14.59,-13.28,-12.27,-13.24,-14.50, &
                                                -16.27,-16.46,-16.66,-16.87,-17.91,-19.27,-21.27,-21.43,-21.60,-21.77, &
                                                -22.87,-24.34,-26.57,-27.64,-29.05,-31.17,-32.07,-33.22,-34.77,-36.53, &
                                                -39.54]
      real, parameter :: f3center(0:40) = [1.,1.250,1.600,2.000,2.500,3.150,4.000,5.000,6.300,8.000,10., &
                                              12.50,16.00,20.00,25.00,31.50,40.00,50.00,63.00,80.00,100., &
                                              125.0,160.0,200.0,250.0,315.0,400.0,500.0,630.0,800.0,1000., &
                                              1250.,1600.,2000.,2500.,3150.,4000.,5000.,6300.,8000.,10000.]
      real, parameter :: f3lower(0:40) = [0.9,1.120,1.400,1.800,2.240,2.800,3.550,4.500,5.600,7.100,9., &
                                              11.20,14.00,18.00,22.40,28.00,35.50,45.00,56.00,71.00,90., &
                                              112.0,140.0,180.0,224.0,280.0,355.0,450.0,560.0,710.0,900., &
                                              1120.,1400.,1800.,2240.,2800.,3550.,4500.,5600.,7100.,9000.]
      real, parameter :: f3upper(0:40) = [1.12,1.400,1.800,2.240,2.800,3.550,4.500,5.600,7.100,9.000,11.2, &
                                               14.00,18.00,22.40,28.00,35.50,45.00,56.00,71.00,90.00,112., &
                                               140.0,180.0,224.0,280.0,355.0,450.0,560.0,710.0,900.0,1120., &
                                               1400.,1800.,2240.,2800.,3550.,4500.,5600.,7100.,9000.,11200.]
      real, parameter :: dBAcorr(0:40) = [-148.58,-140.83,-132.28,-124.55,-116.85,-108.89,-100.72,-93.14,-85.40,-77.55,-70.43, &
                                                   -63.58, -56.42, -50.39, -44.82, -39.52, -34.54,-30.27,-26.22,-22.40,-19.14, &
                                                   -16.19, -13.24, -10.85,  -8.67,  -6.64,  -4.77, -3.25, -1.91, -0.79,  0.00, &
                                                     0.58,   0.99,   1.20,   1.27,   1.20,   0.96,  0.55, -0.12, -1.15, -2.49]
!   work array
      real fftwork(lfftwork)
!
!   COMMON
      integer ncase
      type (InputParam) theInput
      type (Output) CIIoutput(ncasemax)
      type (SourceData) Source(ncasemax)
      type (SoundData) Sound(ncasemax,nobsmax)
      COMMON /job/theInput,CIIoutput,Source,Sound,fftwork,ncase
!
      endmodule CIIsound_structures
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      program CIIsound
      use CIIsound_structures
!
!   read input and initialize
      call GetParam
!   extract data from CAMRADII output
      call ExtractData
!   write extracted data
      call WriteExtractedData
!   construct source data
      call SetSource
!   write source data
      call WriteSourceData
!     call WriteInterpolatedData
!   calculate sound
      call GetSound
      call WriteSound
!
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine GetParam
      use CIIsound_structures
!
      integer KIND_obs(ncasemax),KIND_traj(ncasemax),KIND_mic(ncasemax)
      integer Ntraj
      real xbegin(ncasemax),xend(ncasemax),xref(ncasemax),zref(ncasemax),xmic(3,ncasemax)
      integer nObserver(ncasemax)
      real position_obs(3,nobsmax,ncasemax)
      integer Ntime,Nspan,nterp,NfreqBVI(2)
      integer nprop
      real rprop(npropamax,nrtrmax),thick(npropamax,nrtrmax)
      integer MODEL_broadband,OMIT_broadband
      integer READ_aero,OMIT_TBLTE,OMIT_LBLVS,OMIT_BLUNT,OMIT_TIP
      integer MODEL_direct,MODEL_alpha,MODEL_trip,Nbbave
      real alpha_zl(npropamax,nrtrmax),hTE(npropamax,nrtrmax),psiTE(npropamax,nrtrmax)
      integer isRound(nrtrmax)
      real ftipalpha(nrtrmax)
      integer OUT_perf,OUT_sensor,OUT_source,OUT_sound,OUT_soundfiles
!
      namelist /CIIsound_input/KIND_obs,KIND_traj,KIND_mic,Ntraj,xbegin,xend,xref,zref,xmic,nObserver,position_obs, &
         Ntime,Nspan,interp,NfreqBVI,nprop,rprop,thick,MODEL_broadband,OMIT_broadband, &
         READ_aero,OMIT_TBLTE,OMIT_LBLVS,OMIT_BLUNT,OMIT_TIP,MODEL_direct,MODEL_alpha,MODEL_trip,Nbbave, &
         alpha_zl,hTE,psiTE,isRound,ftipalpha,OUT_perf,OUT_sensor,OUT_source,OUT_sound,OUT_soundfiles
!
!   initialize
      KIND_obs=1; KIND_traj=0; KIND_mic=0 ! input (airframe)
      Ntraj=51; xbegin=-500.; xend=500.; xref=0.; zref=0.; xmic=0.
      nObserver=1; position_obs=0.
      Ntime=1024; Nspan=20
      interp=3 ! linear-harmonic
      NfreqBVI(1)=10; NfreqBVI(2)=50
      nprop=2; rprop=0.
      do i=1,nrtrmax; rprop(1,i)=0.; rprop(2,i)=1.; enddo
      thick=0.12 ! NACA 0012
      MODEL_broadband=1; OMIT_broadband=0 ! Pegg
      READ_aero=0; OMIT_TBLTE=0; OMIT_LBLVS=0; OMIT_BLUNT=0; OMIT_TIP=0
      MODEL_direct=1; MODEL_alpha=1; MODEL_trip=1 ! ROTONET; angle-of-attack; tripped
      Nbbave=24
      alpha_zl=0.; hTE=0.00252; psiTE=15.974 ! NACA 0012
      isRound=0; ftipalpha=1.
      OUT_perf=0; OUT_sensor=0; OUT_source=0; OUT_sound=1; OUT_soundfiles=0
!
!   read
      read (5,CIIsound_input)
!
!   check
      Ntraj=max(3,min(nobsmax,Ntraj))
      do nc=1,ncasemax
           nObserver(nc)=max(1,min(nobsmax,nObserver(nc))); if (KIND_obs(nc) .le. 0) nObserver(nc)=Ntraj
           KIND_obs(nc)=max(0,min(2,KIND_obs(nc)))
           KIND_traj(nc)=max(0,min(3,KIND_traj(nc)))
           KIND_mic(nc)=max(0,min(3,KIND_mic(nc)))
      enddo
      Ntime=max(1,min(ntimemax,Ntime))
      Nspan=max(1,min(npropamax,Nspan))
      nprop=max(1,min(npropamax,nprop))
      Nbbave=max(1,Nbbave)
      if (MODEL_broadband .ge. 2) READ_aero=1
!
!   save
      theInput%KIND_obs       =KIND_obs
      theInput%KIND_traj      =KIND_traj
      theInput%KIND_mic       =KIND_mic
      theInput%Ntraj          =Ntraj
      theInput%xbegin         =xbegin
      theInput%xend           =xend
      theInput%xref           =xref
      theInput%zref           =zref
      theInput%xmic           =xmic
      theInput%nObserver      =nObserver
      theInput%position_obs   =position_obs
      theInput%Ntime          =Ntime
      theInput%Nspan          =Nspan
      theInput%interp         =interp
      theInput%NfreqBVI       =NfreqBVI
      theInput%nprop          =nprop
      theInput%rprop          =rprop
      theInput%thick          =thick
      theInput%MODEL_broadband=MODEL_broadband
      theInput%OMIT_broadband =OMIT_broadband
      theInput%OMIT_TBLTE     =OMIT_TBLTE
      theInput%OMIT_LBLVS     =OMIT_LBLVS
      theInput%OMIT_BLUNT     =OMIT_BLUNT
      theInput%OMIT_TIP       =OMIT_TIP
      theInput%MODEL_direct   =MODEL_direct
      theInput%MODEL_alpha    =MODEL_alpha
      theInput%MODEL_trip     =MODEL_trip
      theInput%Nbbave         =Nbbave
      theInput%alpha_zl       =alpha_zl
      theInput%hTE            =hTE
      theInput%psiTE          =psiTE
      theInput%isRound        =isRound
      theInput%ftipalpha      =ftipalpha
      theInput%READ_aero      =READ_aero
      theInput%OUT_perf       =OUT_perf
      theInput%OUT_sensor     =OUT_sensor
      theInput%OUT_source     =OUT_source
      theInput%OUT_sound      =OUT_sound
      theInput%OUT_soundfiles =OUT_soundfiles
!
      call rfft1i (theInput%Ntime,fftwork,lfftwork,ierror )
      if (ierror .ne. 0) write (6,*) '  initialize fft error',ierror
!
      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
!***********************************************************************************************************************************
! Extract Data from CAMRADII Output File
!***********************************************************************************************************************************
      subroutine ExtractData
      use CIIsound_structures
!
      character title*80
      integer opunit,level,iter(3),mpsi,nwprnt,config,nrotor,opfree,opaero,outres
      integer oppart(nrtrmax),oprpm(nrtrmax),opsnd(nrtrmax),nblade(nrtrmax),npanel(nrtrmax),rotate(nrtrmax)
      real azref(nrtrmax),rgear(nrtrmax),radius(nrtrmax),sigma(nrtrmax)
      real ra(npnlmax),dra(npnlmax),rprop(npropamax),chord(npropamax)
      real Vtip(nrtrmax,ncasemax),Omega(nrtrmax,ncasemax),CFI(3,3)
      real weight(ncasemax)
      real eviron(9,ncasemax),opcond(29,ncasemax),perf(50,ncasemax)
      real perfr(117,ncasemax,nrtrmax)
      real sensor(12,0:mpsimax),azrefs,Omegas
      integer mpsis,mraero,mpsiaero
      real raero(npnlmax),Umag(npnlmax,0:mpsimax),alpha(npnlmax,0:mpsimax),cl(npnlmax,0:mpsimax)
!
      character line*132,tname*100
   99 format (a132)
   90 format (103x,i10)
!
!   read CAMRADII output file
      nfile=11
      call getenv('filein',tname)
      open (nfile,file=tname,status='old',readonly)
!   output resolution
    9 read (nfile,99,err=10,end=10) line
      if (index(line,'OUTRES') .eq. 0) go to 9
      read (line,90) outres
!
!   case
      ncase=0
    1 ncase=ncase+1
   11 read (nfile,99,err=10,end=10) line
      if (index(line,'CAMRAD II') .eq. 0) go to 11
!   case parameters
      call read_environment(nfile,eviron(1,ncase),ierror)
      if (ierror .ne. 0) call exit_on_error(nfile,'reading environment')
      CIIoutput(ncase)%Condition%density=eviron(1,ncase)
      CIIoutput(ncase)%Condition%csound =eviron(5,ncase)
      CIIoutput(ncase)%Condition%kinvisc=eviron(8,ncase)
      call read_param(nfile,title,opunit,level,iter,mpsi,oppart,nwprnt,ierror)
      if (ierror .ne. 0) call exit_on_error(nfile,'reading parameters')
      call read_aparam(nfile,config,nrotor,opfree,opaero,oprpm,rgear,azref,weight(ncase),ierror)
      if (ierror .ne. 0) call exit_on_error(nfile,'reading aircraft parameters')
      CIIoutput(ncase)%opunit=opunit
      CIIoutput(ncase)%nrotor=nrotor
      CIIoutput(ncase)%opfree=opfree
      do krtr=1,nrotor
      CIIoutput(ncase)%Rotor(krtr)%oppart=oppart(krtr)
      CIIoutput(ncase)%Rotor(krtr)%oprpm=oprpm(krtr)
      CIIoutput(ncase)%Rotor(krtr)%rgear=rgear(krtr)
      CIIoutput(ncase)%Rotor(krtr)%azref=azref(krtr)
      enddo
      do krtr=1,nrotor
      call read_rsparam(nfile,rotate(krtr),radius(krtr),nblade(krtr),sigma(krtr),opsnd(krtr),ierror)
      if (ierror .ne. 0) call exit_on_error(nfile,'reading rotor parameters')
      if (opsnd(krtr) .ne. 0) opsnd(krtr)=1
      CIIoutput(ncase)%Rotor(krtr)%opsnd=opsnd(krtr)
      CIIoutput(ncase)%Rotor(krtr)%nblade=nblade(krtr)
      CIIoutput(ncase)%Rotor(krtr)%rotate=rotate(krtr)
      CIIoutput(ncase)%Rotor(krtr)%radius=radius(krtr)
      CIIoutput(ncase)%Rotor(krtr)%sigma=sigma(krtr)
      CIIoutput(ncase)%Rotor(krtr)%area_rotor=pi*radius(krtr)**2
      CIIoutput(ncase)%Rotor(krtr)%area_blade=sigma(krtr)*(pi*radius(krtr)**2)
      enddo
      do krtr=1,nrotor
      call read_raparam(nfile,npnlmax,npropamax,mra,ra,dra,nprop,rprop,chord,ierror)
      if (ierror .ne. 0) call exit_on_error(nfile,'reading rotor aero parameters')
      npanel(krtr)=mra
      CIIoutput(ncase)%Rotor(krtr)%npanel=mra
      CIIoutput(ncase)%Rotor(krtr)%ra=ra
      CIIoutput(ncase)%Rotor(krtr)%dra=dra
      CIIoutput(ncase)%Rotor(krtr)%nprop=nprop
      CIIoutput(ncase)%Rotor(krtr)%rprop=rprop
      CIIoutput(ncase)%Rotor(krtr)%chord=chord
      enddo
!
      niter=iter(1)
      if (level .ge. 2) niter=niter+iter(2)
      if (level .ge. 3) niter=niter+iter(3)
      if (nwprnt .eq. 0) niter=1
!   last iteration
   12 read (nfile,99,err=10,end=10) line
      if (index(line,'TRIM LOOP ITERATION') .eq. 0) go to 12
      do 2 kiter=1,niter
   13 read (nfile,99,err=10,end=10) line
      if (index(line,'TRIM CONV') .eq. 0) go to 13
      if (kiter .ne. niter) go to 2
!   trim solution
      call read_opcond(nfile,opfree,nrotor,opcond(1,ncase),Vtip(1,ncase),Omega(1,ncase),CFI,ierror)
      if (ierror .ne. 0) call exit_on_error(nfile,'reading operating conditions')
      CIIoutput(ncase)%Condition%Vkts=opcond(1,ncase)
      CIIoutput(ncase)%Condition%speed=opcond(4,ncase)
      CIIoutput(ncase)%Condition%roll =opcond(11,ncase)
      CIIoutput(ncase)%Condition%pitch=opcond(10,ncase)
      CIIoutput(ncase)%Condition%yaw  =opcond(9,ncase)
      CIIoutput(ncase)%Condition%climb=opcond(7,ncase)
      CIIoutput(ncase)%Condition%side =opcond(8,ncase)
      do krtr=1,nrotor
      CIIoutput(ncase)%Condition%Vtip(krtr) =Vtip(krtr,ncase)
      CIIoutput(ncase)%Condition%Omega(krtr)=Omega(krtr,ncase)
      enddo
      CIIoutput(ncase)%Condition%CFI=CFI
      call read_rotorcraft_perf(nfile,perf(1,ncase),ierror)
      if (ierror .ne. 0) call exit_on_error(nfile,'reading rotorcraft performance')
      do krtr=1,nrotor
      call read_rotor_perf(nfile,perfr(1,ncase,krtr),ierror)
      if (ierror .ne. 0) call exit_on_error(nfile,'reading rotor performance')
      CIIoutput(ncase)%Condition%mu(krtr)=perfr(4,ncase,krtr)
      CIIoutput(ncase)%Condition%muz(krtr)=perfr(3,ncase,krtr)
      CIIoutput(ncase)%Condition%Mtip(krtr)=perfr(6,ncase,krtr)
      CIIoutput(ncase)%Condition%Mat(krtr)=perfr(7,ncase,krtr)
      CIIoutput(ncase)%Condition%CTs(krtr)=perfr(17,ncase,krtr)
      enddo
!   sound sensors
      do nrtr=1,nrotor
      if (opsnd(nrtr) .ne. 0) then
         do mbld=1,nblade(nrtr)
         if ((oppart(nrtr) .ge. 2) .or. (mbld .eq. nblade(nrtr))) then
            do kpnl=1,npanel(nrtr)
               call read_resp_sound(nfile,outres,nrtr,mbld,kpnl,mpsimax,azrefs,Omegas,mpsis,sensor,ierror)
               if (ierror .ne. 0) call exit_on_error(nfile,'reading sound sensor')
               period=2.*pi/Omegas; dpsi=360./float(mpsis)
               do jpsi=0,mpsis
                  do i=1,3
                  CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%force(i,kpnl,jpsi)=sensor(i,jpsi)
                  CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%disp(i,kpnl,jpsi)=sensor(i+3,jpsi)
                  CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%vel(i,kpnl,jpsi)=sensor(i+6,jpsi)
                  if (opfree .eq. 0) then
                  CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%vel(i,kpnl,jpsi)=sensor(i+6,jpsi)+CFI(i,1)*opcond(4,ncase)
                  endif
                  CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%disp3(i,kpnl,jpsi)=sensor(i+9,jpsi)
                  enddo
                  CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%psi(jpsi)=float(jpsi)*dpsi
                  CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%time(jpsi)=((float(jpsi)*dpsi-azrefs)/360.)*period
               enddo
            enddo
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%mpsi=mpsis
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%azref=azrefs
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%Omega=Omegas
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%period=period
         endif
         enddo
         if ((oppart(nrtr) .le. 1) .and. (nblade(nrtr) .ge. 2)) then
         nbld=nblade(nrtr)
         do mbld=1,nbld-1
            mpsis =CIIoutput(ncase)%Rotor(nrtr)%Blade(nbld)%mpsi
            Omegas=CIIoutput(ncase)%Rotor(nrtr)%Blade(nbld)%Omega
            azrefs=CIIoutput(ncase)%Rotor(nrtr)%Blade(nbld)%azref+float(mbld)*(360./float(nbld))
            period=2.*pi/Omegas; dpsi=360./float(mpsis)
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%mpsi=mpsis
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%azref=azrefs
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%Omega=Omegas
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%period=period
            do jpsi=0,mpsis
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%psi(jpsi)=float(jpsi)*dpsi
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%time(jpsi)=((float(jpsi)*dpsi-azrefs)/360.)*period
            do kpnl=1,npanel(nrtr)
            do i=1,3
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%force(i,kpnl,jpsi)=CIIoutput(ncase)%Rotor(nrtr)%Blade(nbld)%force(i,kpnl,jpsi)
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%disp(i,kpnl,jpsi) =CIIoutput(ncase)%Rotor(nrtr)%Blade(nbld)%disp(i,kpnl,jpsi)
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%vel(i,kpnl,jpsi)  =CIIoutput(ncase)%Rotor(nrtr)%Blade(nbld)%vel(i,kpnl,jpsi)
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%disp3(i,kpnl,jpsi)=CIIoutput(ncase)%Rotor(nrtr)%Blade(nbld)%disp3(i,kpnl,jpsi)
            enddo
            enddo
            enddo
         enddo
         endif
         do i=1,3
            force=0.; disp=0.
            do mbld=1,nblade(nrtr)
            mpsis=CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%mpsi
            do jpsi=1,mpsis
               do kpnl=1,npanel(nrtr)
               force=force+CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%force(i,kpnl,jpsi)
               enddo
               disp=disp+CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%disp(i,1,jpsi)
            enddo
            enddo
            CIIoutput(ncase)%Rotor(nrtr)%hub_force(i)=force/float(mpsis)
            CIIoutput(ncase)%Rotor(nrtr)%hub_disp(i)=disp/(float(mpsis*nblade(nrtr)))
         enddo
!   aerodynamic sensors
      if (theInput%READ_aero .ne. 0) then
         do mbld=1,nblade(nrtr)
         if ((oppart(nrtr) .ge. 2) .or. (mbld .eq. nblade(nrtr))) then
            call read_aero_trim(nfile,'RESULTANT SECTION VELOCITY U',outres,npnlmax,mpsimax,mraero,mpsiaero,raero,Umag,ierror)
            if (ierror .ne. 0) call exit_on_error(nfile,'reading section velocity')
            call read_aero_trim(nfile,'SECTION ANGLE OF ATTACK ALPHA',outres,npnlmax,mpsimax,mraero,mpsiaero,raero,alpha,ierror)
            if (ierror .ne. 0) call exit_on_error(nfile,'reading angle of attack')
            call read_aero_trim(nfile,'TOTAL LIFT COEFFICIENT CL',outres,npnlmax,mpsimax,mraero,mpsiaero,raero,cl,ierror)
            if (ierror .ne. 0) call exit_on_error(nfile,'reading lift coefficient')
            do kpnl=1,npanel(nrtr); do jpsi=0,mpsiaero
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%Umag(kpnl,jpsi) =Umag(kpnl,jpsi)
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%alpha(kpnl,jpsi)=alpha(kpnl,jpsi)
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%cl(kpnl,jpsi)   =cl(kpnl,jpsi)
            enddo; enddo
         endif
         enddo
         if ((oppart(nrtr) .le. 1) .and. (nblade(nrtr) .ge. 2)) then
         nbld=nblade(nrtr)
         do mbld=1,nbld-1
            do kpnl=1,npanel(nrtr); do jpsi=0,mpsiaero
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%Umag(kpnl,jpsi) =CIIoutput(ncase)%Rotor(nrtr)%Blade(nbld)%Umag(kpnl,jpsi)
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%alpha(kpnl,jpsi)=CIIoutput(ncase)%Rotor(nrtr)%Blade(nbld)%alpha(kpnl,jpsi)
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%cl(kpnl,jpsi)   =CIIoutput(ncase)%Rotor(nrtr)%Blade(nbld)%cl(kpnl,jpsi)
            enddo; enddo
         enddo
         endif
      else
         do mbld=1,nblade(nrtr)
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%Umag=0.
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%alpha=0.
            CIIoutput(ncase)%Rotor(nrtr)%Blade(mbld)%cl=0.
         enddo
      endif
      endif
      enddo
!   end case
    2 continue
      go to 1
   10 close(nfile)
      ncase=ncase-1
!
!   export data
      if (theInput%OUT_perf .ne. 0) then
      nfile=12
      call getenv('fileperf',tname)
      open (nfile,file=tname,status='replace',recordsize=2048)
!
      call write_header(nfile,ncase,title,level,config,nrotor,opfree)
      call write_environment(nfile,ncase,eviron)
      call write_opcond(nfile,ncase,opcond,nrotor,Vtip,Omega,nrtrmax)
      call write_rotorcraft_perf(nfile,ncase,opunit,weight,perf)
      do krtr=1,nrotor
      call write_rotor_perf(nfile,ncase,opunit,krtr,perfr(1,1,krtr))
      enddo
!
      close(nfile)
      endif
!
      return
      end
!***********************************************************************************************************************************
      subroutine exit_on_error(nfile,error)
      integer nfile
      character*(*) error
!
      close(nfile)
   99 format (' Extracting data from CAMRADII output file; exit on error, ',80a)
      write (6,99) error
      stop
!
      return
      end
!***********************************************************************************************************************************
! Read/Write Subroutines to Extract Data from CAMRADII Output File
!***********************************************************************************************************************************
      subroutine read_environment(nfile,eviron,ierror)
      integer nfile,ierror
      real eviron(9)
!
!   read environment
!
!   subroutine input and output
!   input
!      nfile       integer   CAMRADII output file
!   output
!      eviron(9)   real      environment data
!      ierror      integer   error if not zero
!
      character line*132
      real e(9)
      equivalence (e(1),density),(e(2),temp   ),(e(3),altmsl )
      equivalence (e(4),dratio ),(e(5),csound ),(e(6),altpress)
      equivalence (e(7),visc   ),(e(8),visckin),(e(9),altdens )
!
   99 format (a132)
   89 format (32x,f10.0,34x,f10.0,34x,f10.0)
!
   11 read (nfile,99,err=100,end=100) line
      if (index(line,'ENVIRONMENT') .eq. 0) go to 11
      read (nfile,99,err=100,end=100) line
      read (line,89) density,temp,altmsl
      read (nfile,99,err=100,end=100) line
      read (line,89) dratio,csound,altpress
      read (nfile,99,err=100,end=100) line
      read (line,89) visc,visckin,altdens
!
      do i=1,9
      eviron(i)=e(i)
      enddo
!
      ierror=0
      return
!
  100 ierror=1
      return
      end
!***********************************************************************************************************************************
      subroutine read_param(nfile,title,opunit,level,iter,mpsi,oppart,nwprnt,ierror)
      character*80 title
      integer nfile,opunit,level,iter(3),mpsi,oppart(*),nwprnt
      integer ierror
!
!   read rotorcraft parameters
!
!   subroutine input and output
!   input
!      nfile     integer    CAMRADII output file
!   output
!      title     char*80    title
!      opunit    integer    unit system
!      level     integer    wake level
!      iter(3)   integer    wake iterations (u,p,f)
!      mpsi      integer    number of azimuth steps
!      oppart(*) integer    rotor part definition
!      nwprnt    integer    print wake iterations
!      ierror    integer    error if not zero
!
      character line*132
      integer lvl(10)
!
   99 format (a132)
   89 format (58x,10i4)
   88 format (78x,i4)
   87 format (57x,i5)
   86 format (82x,i4)
   79 format (18x,a80)
   78 format (68x,i3)
   77 format (i5)
!
   31 read (nfile,99,err=100,end=100) line
      if (index(line,'CASE TITLE') .eq. 0) go to 31
      read (line,79) title
   32 read (nfile,99,err=100,end=100) line
      if (index(line,'UNIT SYSTEM') .eq. 0) go to 32
      read (line,78) opunit
!
   11 read (nfile,99,err=100,end=100) line
      if (index(line,'ROTOR WAKE ANALYSIS LEVEL') .eq. 0) go to 11
      read (line,89) (lvl(i),i=1,10)
   12 read (nfile,99,err=100,end=100) line
      if (index(line,'NUMBER OF ITERATIONS') .eq. 0) go to 12
      read (nfile,99,err=100,end=100) line
      read (line,88) iter(1)
      read (nfile,99,err=100,end=100) line
      read (line,88) iter(2)
      read (nfile,99,err=100,end=100) line
      read (line,88) iter(3)
   13 read (nfile,99,err=100,end=100) line
      if (index(line,'MPSI') .eq. 0) go to 13
      read (line,87) mpsi
   15 read (nfile,99,err=100,end=100) line
      if (index(line,'PART DEFINITION') .eq. 0) go to 15
      irtr=0
   16 if (line(95:99)   .ne. '     ') then; irtr=irtr+1; read (line(95:99),  77) oppart(irtr); endif
      if (line(100:104) .ne. '     ') then; irtr=irtr+1; read (line(100:104),77) oppart(irtr); endif
      if (line(105:109) .ne. '     ') then; irtr=irtr+1; read (line(105:109),77) oppart(irtr); endif
      if (line(110:114) .ne. '     ') then; irtr=irtr+1; read (line(110:114),77) oppart(irtr); endif
      if (line(115:119) .ne. '     ') then; irtr=irtr+1; read (line(115:119),77) oppart(irtr); endif
      read (nfile,99,err=100,end=100) line
      if (line(21:25) .eq. '     ') go to 16
   14 read (nfile,99,err=100,end=100) line
      if (index(line,'NWPRNT') .eq. 0) go to 14
      read (line,86) nwprnt
!
      level=1
      do irtr=1,10
      level=max(level,lvl(irtr))
      enddo
!
      ierror=0
      return
!
  100 ierror=1
      return
      end
!***********************************************************************************************************************************
      subroutine read_aparam(nfile,config,nrotor,opfree,opaero,oprpm,rgear,azref,weight,ierror)
      integer nfile,config,nrotor,opfree,opaero,oprpm(*)
      real rgear(*),azref(*),weight
      integer ierror
!
!   read rotorcraft parameters
!
!   subroutine input and output
!   input
!      nfile     integer    CAMRADII output file
!   output
!      config    integer    rotorcraft configuration
!      nrotor    integer    number of rotors
!      opfree    integer    wind tunnel (0) or free flight (1)
!      opaero    integer    airframe aerodynamics
!      oprpm(*)  integer    transmission configuration
!      rgear(*)  real       gear ratio
!      azref(*)  real       rotor azimuth at time=0
!      weight    real       gross weight
!      ierror    integer    error if not zero
!
      character line*132
!
   99 format (a132)
   85 format (40x,i4)
   84 format (45x,i3)
   83 format (70x,i4)
   82 format (50x,i4)
   81 format (44x,f16.0)
   76 format (97x,6i4)
   75 format (98x,2f14.0)
   74 format (58x,f12.3,4f14.3)
!
   21 read (nfile,99,err=100,end=100) line
      if (index(line,'ROTORCRAFT CONFIGURATION') .eq. 0) go to 21
      read (line,85) config
   22 read (nfile,99,err=100,end=100) line
      if (index(line,'NROTOR') .eq. 0) go to 22
      read (line,84) nrotor
      if (config .le. 0) nrotor=1
      if ((config .gt. 0) .and. (nrotor .le. 2)) nrotor=2
   23 read (nfile,99,err=100,end=100) line
      if (index(line,'OPFREE') .eq. 0) go to 23
      read (line,83) opfree
   24 read (nfile,99,err=100,end=100) line
      if (index(line,'OPAERO') .eq. 0) go to 24
      read (line,82) opaero
      oprpm(1)=0; rgear(1)=1.
      if (nrotor .ge. 2) then
   26 read (nfile,99,err=100,end=100) line
      if (index(line,'OPRPM') .eq. 0) go to 26
      nr2=0
  126 nr1=nr2+1; nr2=min(nr1+5,nrotor)
      read (line,76) (oprpm(irtr),irtr=nr1,nr2)
      if (nr2 .lt. nrotor) then; read (nfile,99,err=100,end=100) line; go to 126; endif
   27 read (nfile,99,err=100,end=100) line
      if (index(line,'RGEAR') .eq. 0) go to 27
      nr2=0
  127 nr1=nr2+1; nr2=min(nr1+1,nrotor)
      read (line,75) (rgear(irtr),irtr=nr1,nr2)
      if (nr2 .lt. nrotor) then; read (nfile,99,err=100,end=100) line; go to 127; endif
      endif
   28 read (nfile,99,err=100,end=100) line
      if (index(line,'ROTOR AZIMUTH') .eq. 0) go to 28
      nr2=0
  128 nr1=nr2+1; nr2=min(nr1+4,nrotor)
      read (line,74) (azref(irtr),irtr=nr1,nr2)
      if (nr2 .lt. nrotor) then; read (nfile,99,err=100,end=100) line; go to 128; endif
   25 read (nfile,99,err=100,end=100) line
      if (index(line,'GROSS WEIGHT') .eq. 0) go to 25
      read (line,81) weight
!
      ierror=0
      return
!
  100 ierror=1
      return
      end
!***********************************************************************************************************************************
      subroutine read_rsparam(nfile,rotate,radius,nblade,sigma,opsnd,ierror)
      integer nfile,rotate,nblade,opsnd,ierror
      real radius,sigma
!
!   read rotor structure parameters
!
!   subroutine input and output
!   input
!      nfile     integer    CAMRADII output file
!   output
!      rotate    integer    direction of rotation
!      radius    real       blade radius
!      nblade    integer    number of blades
!      sigma     real       solidity
!      opsnd     integer    sound sensor
!      ierror    integer    error if not zero
!
      character line*132
!
   99 format (a132)
   89 format (28x,f10.0)
   88 format (46x,i4)
   87 format (29x,f12.0)
   86 format (51x,i4)
   85 format (73x,i4)
!
   11 read (nfile,99,err=100,end=100) line
      if (index(line,'BLADE RADIUS') .eq. 0) go to 11
      read (line,89) radius
      read (nfile,99,err=100,end=100) line
      read (line,88) nblade
      read (nfile,99,err=100,end=100) line
      read (line,85) rotate
      read (nfile,99,err=100,end=100) line
      read (line,87) sigma
   12 read (nfile,99,err=100,end=100) line
      if (index(line,'SOUND SENSORS') .eq. 0) go to 12
      read (nfile,99,err=100,end=100) line
      read (line,86) opsnd
!
      ierror=0
      return
!
  100 ierror=1
      return
      end
!***********************************************************************************************************************************
      subroutine read_raparam(nfile,npnlmax,npropamax,mra,ra,dra,nprop,rprop,chord,ierror)
      integer nfile,npnlmax,npropamax,mra,nprop,ierror
      real ra(*),dra(*),rprop(*),chord(*)
!
!   read rotor aerodynamic parameters
!
!   subroutine input and output
!   input
!      nfile         integer    CAMRADII output file
!      npnlmax       integer    size of ra,dra arrays
!      npropamax     integer    size of chord array
!   output
!      mra           integer    number of aerodynamic panels
!      ra(mra)       real       panel mid points
!      dra(mra)      real       panel width
!      nprop         integer    number of span stations
!      rprop(nprop)  real       span stations
!      chord(nprop)  real       blade chord
!      ierror        integer    error if not zero
!
      character line*132
!
   99 format (a132)
   89 format (63x,i5)
   88 format (50x,2f16.0)
   87 format (69x,i5)
   86 format (17x,f9.0,f19.0)
!
   11 read (nfile,99,err=100,end=100) line
      if (index(line,'NUMBER OF PANELS') .eq. 0) go to 11
      read (line,89) mra
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      n=min(mra,npnlmax)
      do i=1,n
      read (nfile,88) ra(i),dra(i)
      enddo
!
   12 read (nfile,99,err=100,end=100) line
      if (index(line,'NPROP') .eq. 0) go to 12
      read (line,87) nprop
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      n=min(nprop,npropamax)
      do i=1,n
      read (nfile,86) rprop(i),chord(i)
      enddo
!
      ierror=0
      return
!
  100 ierror=1
      return
      end
!***********************************************************************************************************************************
      subroutine read_opcond(nfile,opfree,nrotor,opcond,vtipr,omegar,CFI,ierror)
      integer nfile,opfree,nrotor,ierror
      real opcond(29),vtipr(*),omegar(*),CFI(3,3)
!
!   read operating condition
!
!   subroutine input and output
!   input
!      nfile          integer   CAMRADII output file
!      opfree         integer   wind tunnel (0) or free flight (1)
!      nrotor         integer   number of rotors
!   output
!      opcond(29)     real      operating condition data
!      vtipr(*)       real      tip speed
!      omegar(*)      real      rotational speed
!      CFI(3,3)       real      transformation from inertial to airframe axes
!
      character line*132
      real mach,mtip,latcyc,lngcyc
      real o(29),c(3)
      equivalence (o( 1),vkts    ),(o( 2),vel     ),(o(3),mach   )
      equivalence (o( 4),speed   ),(o( 5),que     ),(o(6),renum  )
      equivalence (o( 7),climb   ),(o( 8),side    )
      equivalence (o( 9),yaw     ),(o(10),pitch   ),(o(11),roll  )
      equivalence (o(12),vtip    ),(o(13),mtip    ),(o(14),rpm   )
      equivalence (o(15),turn    )
      equivalence (o(16),coll    ),(o(17),latcyc  ),(o(18),lngcyc)
      equivalence (o(19),pedal   ),(o(20),throttle),(o(21),gov   )
      equivalence (o(22),ugust   ),(o(23),vgust   ),(o(24),wgust )
      equivalence (o(25),flaperon),(o(26),aileron ),(o(27),engine)
      equivalence (o(28),elevator),(o(29),rudder  )
      equivalence (c( 1),theta0  ),(c( 2),thetac  ),(c( 3),thetas)
!
   99 format (a132)
   89 format (32x,f10.0,34x,f10.0,34x,f10.0)
   88 format (33x,f10.0,23x,f10.0,29x,f10.0)
   87 format (33x,f10.0,23x,f10.0,29x,i4,15x,i4)
   86 format (29x,f10.0,54x,f10.0)
!
   11 read (nfile,99,err=100,end=100) line
      if (index(line,'VELOCITY (KNOTS)') .eq. 0) go to 11
!     opfree=1
!     if (index(line,'YAW ANGLE') .ne. 0) opfree=0
!
!   free flight
      if (opfree .ne. 0) then
      read (line,89) vkts
      read (nfile,99,err=100,end=100) line
      read (line,89) vel
      read (nfile,99,err=100,end=100) line
      read (line,89) mach
      read (nfile,99,err=100,end=100) line
      read (line,89) speed
      read (nfile,99,err=100,end=100) line
      read (line,89) que
      read (nfile,99,err=100,end=100) line
      read (line,89) renum
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,89) climb
      read (nfile,99,err=100,end=100) line
      read (line,89) side
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,89) x,yaw,vtip
      read (nfile,99,err=100,end=100) line
      read (line,89) x,pitch,mtip
      read (nfile,99,err=100,end=100) line
      read (line,89) x,roll,rpm
      read (nfile,99,err=100,end=100) line
      read (line,89) x,turn,omega
      if (nrotor .gt. 1) then
   22 read (nfile,99,err=100,end=100) line
      if (index(line,'ROTOR SPEED') .eq. 0) go to 22
      do irtr=1,nrotor
      read (nfile,99,err=100,end=100) line
      read (line,86) vtipr(irtr),omegar(irtr)
      enddo
      else
      vtipr(1)=vtip; omegar(1)=omega
      endif
   12 read (nfile,99,err=100,end=100) line
      if (index(line,'COLLECTIVE') .eq. 0) go to 12
      read (line,88) coll,pedal!,ugust
      read (nfile,99,err=100,end=100) line
      read (line,88) latcyc!,throttle,vgust
      read (nfile,99,err=100,end=100) line
      read (line,88) lngcyc!,gov,wgust
     !read (nfile,99,err=100,end=100) line
     !read (nfile,99,err=100,end=100) line
     !read (nfile,99,err=100,end=100) line
     !read (line,88) flaperon,aileron,engine
     !read (nfile,99,err=100,end=100) line
     !read (line,88) elevator,rudder
!
!   wind tunnel
      else
      read (line,89) vkts,yaw,vtip
      read (nfile,99,err=100,end=100) line
      read (line,89) vel,pitch,mtip
      read (nfile,99,err=100,end=100) line
      read (line,89) mach,roll,rpm
      read (nfile,99,err=100,end=100) line
      read (line,89) speed,x,omega
      read (nfile,99,err=100,end=100) line
      read (line,89) que
      read (nfile,99,err=100,end=100) line
      read (line,89) renum
      climb=0.
      side=0.
      turn=0.
      if (nrotor .gt. 1) then
   23 read (nfile,99,err=100,end=100) line
      if (index(line,'ROTOR SPEED') .eq. 0) go to 23
      do irtr=1,nrotor
      read (nfile,99,err=100,end=100) line
      read (line,86) vtipr(irtr),omegar(irtr)
      enddo
      else
      vtipr(1)=vtip; omegar(1)=omega
      endif
   13 read (nfile,99,err=100,end=100) line
      if (index(line,'COLLECTIVE') .eq. 0) go to 13
      read (line,88) coll,pedal!,ugust
      read (nfile,99,err=100,end=100) line
      read (line,88) latcyc!,throttle,vgust
      read (nfile,99,err=100,end=100) line
      read (line,88) lngcyc!,gov,wgust
     !read (nfile,99,err=100,end=100) line
     !read (nfile,99,err=100,end=100) line
     !read (nfile,99,err=100,end=100) line
     !read (line,88) flaperon,aileron,engine
     !read (nfile,99,err=100,end=100) line
     !read (line,88) elevator,rudder
      endif
!
      do i=1,29
      opcond(i)=o(i)
      enddo
!
      pi=3.141592653589793238462643383280; raddeg=pi/180.
      phi  =roll *raddeg; cs1=cos(phi);   sn1=sin(phi)
      theta=pitch*raddeg; cs2=cos(theta); sn2=sin(theta)
      psi  =yaw  *raddeg; cs3=cos(psi);   sn3=sin(psi)
      CFI(1,1)=    cs2*cs3
      CFI(2,1)=sn1*sn2*cs3-cs1*sn3
      CFI(3,1)=cs1*sn2*cs3+sn1*sn3
      CFI(1,2)=    cs2*sn3
      CFI(2,2)=sn1*sn2*sn3+cs1*cs3
      CFI(3,2)=cs1*sn2*sn3-sn1*cs3
      CFI(1,3)=   -sn2
      CFI(2,3)=sn1*cs2
      CFI(3,3)=cs1*cs2
!
      ierror=0
      return
!
  100 ierror=1
      return
      end
!***********************************************************************************************************************************
      subroutine read_rotorcraft_perf(nfile,perf,ierror)
      integer nfile,ierror
      real perf(50)
!
!   read rotorcraft performance
!
!   subroutine input and output
!   input
!      nfile       integer   CAMRADII output file
!   output
!      perf(50)    real      performance data
!      ierror      integer   error if not zero
!
      character line*132
      real q(50),lod
      equivalence (q( 1),cp      ),(q( 2),cps      ),(q( 3),hp      )
      equivalence (q( 5),cp_cp   ),(q( 6),cps_cp   ),(q( 7),hp_cp   )
      equivalence (q( 9),cp_oi   ),(q(10),cps_oi   ),(q(11),hp_oi   )
      equivalence (q(13),cp_ind  ),(q(14),cps_ind  ),(q(15),hp_ind  )
      equivalence (q(17),cp_int  ),(q(18),cps_int  ),(q(19),hp_int  )
      equivalence (q(21),cp_o    ),(q(22),cps_o    ),(q(23),hp_o    )
      equivalence (q(25),cp_p    ),(q(26),cps_p    ),(q(27),hp_p    )
      equivalence (q(29),cp_c    ),(q(30),cps_c    ),(q(31),hp_c    )
      equivalence (q(33),cp_osxn ),(q(34),cps_osxn ),(q(35),hp_osxn )
      equivalence (q(37),cp_m    ),(q(38),cps_m    ),(q(39),hp_m    )
      equivalence (q(41),cp_ideal),(q(42),cps_ideal),(q(43),hp_ideal)
      equivalence (q(45),cp_n    ),(q(46),cps_n    ),(q(47),hp_n    )
      equivalence (q( 4),power      )
      equivalence (q( 8),power_cp   )
      equivalence (q(12),power_oi   )
      equivalence (q(16),power_ind  )
      equivalence (q(20),power_int  )
      equivalence (q(24),power_o    )
      equivalence (q(28),power_p    )
      equivalence (q(32),power_c    )
      equivalence (q(36),power_osxn )
      equivalence (q(40),power_m    )
      equivalence (q(44),power_ideal)
      equivalence (q(48),power_n    )
      equivalence (q(49),lode       )
      equivalence (q(50),lod        )
!
   99 format (a132)
   89 format (37x,f12.0,16x,f11.0,11x,f12.0,7x,f13.0)
   88 format (64x,f14.0)
   87 format (64x,f11.0)
!
   11 read (nfile,99,err=100,end=100) line
      if (index(line,'ROTORCRAFT POWER') .eq. 0) go to 11
      read (nfile,99,err=100,end=100) line
      read (line,89) cp,cps,hp,power
      read (nfile,99,err=100,end=100) line
      read (line,89) cp_cp,cps_cp,hp_cp,power_cp
      read (nfile,99,err=100,end=100) line
      read (line,89) cp_oi,cps_oi,hp_oi,power_oi
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,89) cp_ind,cps_ind,hp_ind,power_ind
      read (nfile,99,err=100,end=100) line
      read (line,89) cp_int,cps_int,hp_int,power_int
      read (nfile,99,err=100,end=100) line
      read (line,89) cp_o,cps_o,hp_o,power_o
      read (nfile,99,err=100,end=100) line
      read (line,89) cp_p,cps_p,hp_p,power_p
      read (nfile,99,err=100,end=100) line
      read (line,89) cp_c,cps_p,hp_c,power_c
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,89) cp_osxn,cps_osxn,hp_osxn,power_osxn
      read (nfile,99,err=100,end=100) line
      read (line,89) cp_m,cps_m,hp_m,power_m
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,89) cp_ideal,cps_ideal,hp_ideal,power_ideal
      read (nfile,99,err=100,end=100) line
      read (line,89) cp_n,cps_n,hp_n,power_n
      do i=1,5
      read (nfile,99,err=100,end=100) line
      enddo
      read (line,87) lode
      do i=1,3
      read (nfile,99,err=100,end=100) line
      enddo
      read (line,87) lod
!
      do i=1,50
      perf(i)=q(i)
      enddo
!
      ierror=0
      return
!
  100 ierror=1
      return
      end
!***********************************************************************************************************************************
      subroutine read_rotor_perf(nfile,perf,ierror)
      integer nfile,ierror
      real perf(117)
!
!   read rotor performance
!
!   subroutine input and output
!   input
!      nfile       integer   CAMRADII output file
!   output
!      perf(117)   real      performance data
!      ierror      integer   error if not zero
!
      character line*132
      real mux,muy,muz,mu,mtip,mach,mat
      real lift,kappa,kappa_int,loq
      real p(117)
      equivalence (p( 1),mux   ),(p( 2),muy   ),(p( 3),muz   )
      equivalence (p( 4),mu    ),(p( 5),mach  ),(p( 6),mtip  )
      equivalence (p( 7),mat   )
      equivalence (p( 8),alpha ),(p( 9),psi   )
      equivalence (p(10),betac ),(p(11),betas ),(p(12),beta0 )
      equivalence (p(13),thetas),(p(14),thetac),(p(15),theta0)
      equivalence (p(16),ct ),(p(17),cts ),(p(18),thrust  )
      equivalence (p(19),ch ),(p(20),chs ),(p(21),hforce  )
      equivalence (p(22),cy ),(p(23),cys ),(p(24),yforce  )
      equivalence (p(25),cmx),(p(26),cmxs),(p(27),rollmom )
      equivalence (p(28),cmy),(p(29),cmys),(p(30),pitchmom)
      equivalence (p(31),cmz),(p(32),cmzs),(p(33),yawmom  )
      equivalence (p(34),cl ),(p(35),cls ),(p(36),lift    )
      equivalence (p(37),cx ),(p(38),cxs ),(p(39),xforce  )
      equivalence (p(40),cq ),(p(41),cqs ),(p(42),torque  )
      equivalence (p(43),cp      ),(p(44),cps      ),(p(45),hp      )
      equivalence (p(47),cp_cp   ),(p(48),cps_cp   ),(p(49),hp_cp   )
      equivalence (p(51),cp_oi   ),(p(52),cps_oi   ),(p(53),hp_oi   )
      equivalence (p(55),cp_ind  ),(p(56),cps_ind  ),(p(57),hp_ind  )
      equivalence (p(59),cp_int  ),(p(60),cps_int  ),(p(61),hp_int  )
      equivalence (p(63),cp_o    ),(p(64),cps_o    ),(p(65),hp_o    )
      equivalence (p(67),cp_osxn ),(p(68),cps_osxn ),(p(69),hp_osxn )
      equivalence (p(71),cp_m    ),(p(72),cps_m    ),(p(73),hp_m    )
      equivalence (p(75),cp_ideal),(p(76),cps_ideal),(p(77),hp_ideal)
      equivalence (p(79),cp_n    ),(p(80),cps_n    ),(p(81),hp_n    )
      equivalence (p(46),power      )
      equivalence (p(50),power_cp   )
      equivalence (p(54),power_oi   )
      equivalence (p(58),power_ind  )
      equivalence (p(62),power_int  )
      equivalence (p(66),power_o    )
      equivalence (p(70),power_osxn )
      equivalence (p(74),power_m    )
      equivalence (p(78),power_ideal)
      equivalence (p(82),power_n    )
      equivalence (p(83),kappa),(p(84),kappa_int)
      equivalence (p(85),cdo  )
      equivalence (p(86),fm      ),(p(87),powercoeff )
      equivalence (p(88),fm2     ),(p(89),windturbeff)
      equivalence (p(90),propeff ),(p(91),winge      )
      equivalence (p(92),rotorlod),(p(93),winglox    )
      equivalence (p( 94),drg  ),(p( 95),doq  )
      equivalence (p( 98),drg_o),(p( 99),doq_o)
      equivalence (p(102),drg_i),(p(103),doq_i)
      equivalence (p(106),drg_t),(p(107),doq_t)
      equivalence (p(110),frcx ),(p(111),xoq  )
      equivalence (p(114),frcl ),(p(115),loq  )
      equivalence (p( 96),cd   ),(p( 97),cdw  )
      equivalence (p(100),cd_o ),(p(101),cdw_o)
      equivalence (p(104),cd_i ),(p(105),cdw_i)
      equivalence (p(108),cd_t ),(p(109),cdw_t)
      equivalence (p(112),cfrcx),(p(113),cxw  )
      equivalence (p(116),cfrcl),(p(117),clw  )
!
   99 format (a132)
   89 format (24x,3f9.0,f12.0,f8.0,f14.0,f10.0,f17.0,f11.0)
   88 format (25x,f7.0,37x,f7.0,28x,f8.0)
   87 format (33x,f12.0,14x,f11.0,11x,f16.0)
   86 format (37x,f12.0,16x,f11.0,11x,f12.0,7x,f13.0)
   85 format (68x,f10.4)
   84 format (86x,f10.4)
   83 format (48x,f9.0,48x,f9.0)
   82 format (42x,f14.0,13x,f11.0,12x,f10.0,13x,f10.0)
!
   11 read (nfile,99,err=100,end=100) line
      if (index(line,'BC (LONG)') .eq. 0) go to 11
      read (nfile,99,err=100,end=100) line
      read (line,89) mux,muy,muz,alpha,psi,betac,betas,thetas,thetac
      do i=1,5
      read (nfile,99,err=100,end=100) line
      enddo
      read (line,88) mu,mtip,beta0
      read (nfile,99,err=100,end=100) line
      read (line,88) mach,mat,theta0
   12 read (nfile,99,err=100,end=100) line
      if (index(line,'ROTOR FORCES') .eq. 0) go to 12
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,87) ct,cts,thrust
      read (nfile,99,err=100,end=100) line
      read (line,87) ch,chs,hforce
      read (nfile,99,err=100,end=100) line
      read (line,87) cy,cys,yforce
      read (nfile,99,err=100,end=100) line
      read (line,87) cmx,cmxs,rollmom
      read (nfile,99,err=100,end=100) line
      read (line,87) cmy,cmys,pitchmom
      read (nfile,99,err=100,end=100) line
      read (line,87) cmz,cmzs,yawmom
      do i=1,9
      read (nfile,99,err=100,end=100) line
      enddo
      read (line,87) cl,cls,lift
      read (nfile,99,err=100,end=100) line
      read (line,87) cx,cxs,xforce
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,87) cq,cqs,torque
   13 read (nfile,99,err=100,end=100) line
      if (index(line,'ROTOR POWER') .eq. 0) go to 13
      read (nfile,99,err=100,end=100) line
      read (line,86) cp,cps,hp,power
      read (nfile,99,err=100,end=100) line
      read (line,86) cp_cp,cps_cp,hp_cp,power_cp
      read (nfile,99,err=100,end=100) line
      read (line,86) cp_oi,cps_oi,hp_oi,power_oi
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,86) cp_ind,cps_ind,hp_ind,power_ind
      read (nfile,99,err=100,end=100) line
      read (line,86) cp_int,cps_int,hp_int,power_int
      read (nfile,99,err=100,end=100) line
      read (line,86) cp_o,cps_o,hp_o,power_o
      read (nfile,99,err=100,end=100) line
      read (line,86) cp_osxn,cps_osxn,hp_osxn,power_osxn
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,86) cp_m,cps_m,hp_m,power_m
      read (nfile,99,err=100,end=100) line
      read (line,86) cp_ideal,cps_ideal,hp_ideal,power_ideal
      read (nfile,99,err=100,end=100) line
      read (line,86) cp_n,cps_n,hp_n,power_n
   14 read (nfile,99,err=100,end=100) line
      if (index(line,'P/PM') .eq. 0) go to 14
      read (nfile,99,err=100,end=100) line
      read (line,85) kappa
      read (nfile,99,err=100,end=100) line
      read (line,85) kappa_int
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,84) cdo
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,83) fm!,powercoeff
      read (nfile,99,err=100,end=100) line
      read (line,83) fm2!,windturbeff
      read (nfile,99,err=100,end=100) line
      read (line,83) propeff!,winge
      read (nfile,99,err=100,end=100) line
      read (line,83) rotorlod!,winglox
     !read (nfile,99,err=100,end=100) line
     !read (nfile,99,err=100,end=100) line
     !read (nfile,99,err=100,end=100) line
     !read (line,82) drg,doq,cd,cdw
     !read (nfile,99,err=100,end=100) line
     !read (line,82) drg_o,doq_o,cd_o,cdw_o
     !read (nfile,99,err=100,end=100) line
     !read (line,82) drg_i,doq_i,cd_i,cdw_i
     !read (nfile,99,err=100,end=100) line
     !read (line,82) drg_t,doq_t,cd_t,cdw_t
     !read (nfile,99,err=100,end=100) line
     !read (line,82) frcx,xoq,cfrcx,cxw
     !read (nfile,99,err=100,end=100) line
     !read (line,82) frcl,loq,cfrcl,clw
!
      do i=1,117
      perf(i)=p(i)
      enddo
!
      ierror=0
      return
!
  100 ierror=1
      return
      end
!***********************************************************************************************************************************
      subroutine read_resp_sound(nfile,outres,nrotor,mblade,kpanel,mpsimax,azref,omega,mpsi,sensor,ierror)
      integer nfile,outres,nrotor,mblade,kpanel,mpsimax,mpsi,ierror
      real azref,omega,sensor(12,0:mpsimax)
!
!   read sound sensor
!
!   subroutine input and output
!   input
!      nfile               integer   CAMRADII output file
!      outres              integer   output resolution
!      nrotor              integer   rotor number
!      mblade              integer   blade number
!      kpanel              integer   panel number
!      mpsimax             integer   size of hist array
!   output
!      azref               real      azimuth for zero time, time=(az-azref)/omega
!      omega               real      rotational speed
!      mpsi                integer   number of azimuth steps
!      sensor(12,0:mpsi)   real      time history data (jpsi=1,mpsi, psi=jpsi*(360/mpsi)
!      ierror              integer   error if not zero
!
      character line*132,longline*256,quantity*32
   99 format (a132)
  199 format (a256)
   89 format (17x,f8.0,4x,6e16.6)
  189 format (17x,f8.0,4x,6e23.12)
   88 format (37x,f8.0)
   87 format (25x,f10.0)
   86 format (48x,i5)
   81 format ('ROTOR',i2,' BLADE',i2,' SOUND, PANEL',i2,2x)
   82 format ('ROTOR',i2,' BLADE',i2,' SOUND, PANEL',i3,1x)
   83 format ('ROTOR',i2,' BLADE',i2,' SOUND, PANEL',i4)
!
!   quantity='ROTOR n BLADE m SOUND, PANEL k'
!   quantity='ROTOR n BLADE m SOUND, PANEL kk'
!   quantity='ROTOR n BLADE m SOUND, PANEL kkk'
      if      (kpanel .le.   9) then; write (quantity,81) nrotor,mblade,kpanel
      else if (kpanel .le.  99) then; write (quantity,82) nrotor,mblade,kpanel
      else if (kpanel .le. 999) then; write (quantity,83) nrotor,mblade,kpanel
      endif
!
   10 read (nfile,99,err=100,end=100) line
      if (index(line,quantity) .eq. 0) go to 10
   11 read (nfile,99,err=100,end=100) line
      if (index(line,'SOLUTION KIND') .eq. 0) go to 11
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,88) azref
      read (nfile,99,err=100,end=100) line
      read (line,87) omega
      read (nfile,99,err=100,end=100) line
      read (line,86) mpsi
      npsi=min(mpsi,mpsimax)
!
      if (outres .eq. 0) then
      ierr=0
   20 read (nfile,99,err=100,end=100) line
      if (index(line,'MOTION VALUE') .eq. 0) go to 20
   21 read (nfile,99,err=100,end=100) line
      if (index(line,'PSI =') .eq. 0) go to 21
      read (line,89) psi,(sensor(k,0),k=1,6)
      do jpsi=1,npsi
      read (nfile,99,err=100,end=100) line
      if (index(line,'PSI =') .eq. 0) then; ierr=1; exit; endif
      read (line,89) psi,(sensor(k,jpsi),k=1,6)
      enddo
   22 read (nfile,99,err=100,end=100) line
      if (index(line,'MOTION VALUE') .eq. 0) go to 22
   23 read (nfile,99,err=100,end=100) line
      if (index(line,'PSI =') .eq. 0) go to 23
      read (line,89) psi,(sensor(k,0),k=7,12)
      do jpsi=1,npsi
      read (nfile,99,err=100,end=100) line
      if (index(line,'PSI =') .eq. 0) then; ierr=1; exit; endif
      read (line,89) psi,(sensor(k,jpsi),k=7,12)
      enddo
!
      else
      ierr=0
  120 read (nfile,199,err=100,end=100) longline
      if (index(longline,'MOTION VALUE') .eq. 0) go to 120
  121 read (nfile,199,err=100,end=100) longline
      if (index(longline,'PSI =') .eq. 0) go to 121
      read (longline,189) psi,(sensor(k,0),k=1,6)
      do jpsi=1,npsi
      read (nfile,199,err=100,end=100) longline
      if (index(longline,'PSI =') .eq. 0) then; ierr=1; exit; endif
      read (longline,189) psi,(sensor(k,jpsi),k=1,6)
      enddo
  122 read (nfile,199,err=100,end=100) longline
      if (index(longline,'MOTION VALUE') .eq. 0) go to 122
  123 read (nfile,199,err=100,end=100) longline
      if (index(longline,'PSI =') .eq. 0) go to 123
      read (longline,189) psi,(sensor(k,0),k=7,12)
      do jpsi=1,npsi
      read (nfile,199,err=100,end=100) longline
      if (index(longline,'PSI =') .eq. 0) then; ierr=1; exit; endif
      read (longline,189) psi,(sensor(k,jpsi),k=7,12)
      enddo
      endif
!
      if (ierr .ne. 0) then
      write (6,999) jpsi-1,psi,mpsi
      ierror=1
      return
      endif
  999 format (' sound sensor error: last read jpsi=',i5,' and psi=',f7.1'; must use MPTIME=',i5,' (MPSI or MPSIH)')
!
      ierror=0
      return
!
  100 ierror=1
      return
      end
!***********************************************************************************************************************************
      subroutine read_aero_trim(nfile,quantity,outres,mrmax,mpsimax,mr,mpsi,r,aero,ierror)
      integer nfile,outres,mrmax,mpsimax,mr,mpsi,ierror
      character*(*) quantity
      real r(*),aero(mrmax,0:mpsimax)
!
!   read aerodynamic sensor
!
!   subroutine input and output
!   input
!      nfile          integer   CAMRADII output file
!      quantity       char      string identifying aerodynamic quantity
!      outres         integer   output resolution
!      mrmax          integer   size of aero array
!      mpsimax        integer   size of aero array
!   output
!      mr             integer   number of radial stations
!      mpsi           integer   number of azimuth steps
!      r(mr)          real      radial stations
!      aero(mr,mpsi)  real      aerodynamic data
!      ierror         integer   error if not zero
!
      character line*132,longline*256
   99 format (a132)
  199 format (a256)
   89 format (35x,i4)
   88 format (29x,6e16.6)
  188 format (29x,6e23.12)
!
   10 read (nfile,99,err=100,end=100) line
      if (index(line,quantity) .eq. 0) go to 10
   11 read (nfile,99,err=100,end=100) line
      if (index(line,'NUMBER OF SPAN STATIONS') .eq. 0) go to 11
      read (line,89) mr
      if (mr .gt. mrmax) go to 100
!
      if (outres .eq. 0) then
      n1=0
   21 n0=n1+1
      n1=min(n0+5,mr)
   22 read (nfile,99,err=100,end=100) line
      if (index(line,'SPAN STATION') .eq. 0) go to 22
      read (line,88) (r(i),i=n0,n1)
      read (nfile,99,err=100,end=100) line
      read (nfile,99,err=100,end=100) line
      read (line,88) (aero(i,0),i=n0,n1)
      jpsi=0
   23 read (nfile,99,err=100,end=100) line
      if (index(line,'PSI =') .ne. 0) then
           jpsi=jpsi+1
           if (jpsi .gt. mpsimax) go to 100
           read (line,88) (aero(i,jpsi),i=n0,n1)
           go to 23
      endif
      if (n1 .lt. mr) go to 21
      mpsi=jpsi
!
      else
      n1=0
  121 n0=n1+1
      n1=min(n0+5,mr)
  122 read (nfile,199,err=100,end=100) longline
      if (index(longline,'SPAN STATION') .eq. 0) go to 122
      read (longline,188) (r(i),i=n0,n1)
      read (nfile,199,err=100,end=100) longline
      read (nfile,199,err=100,end=100) longline
      read (longline,188) (aero(i,0),i=n0,n1)
      jpsi=0
  123 read (nfile,199,err=100,end=100) longline
      if (index(longline,'PSI =') .ne. 0) then
           jpsi=jpsi+1
           if (jpsi .gt. mpsimax) go to 100
           read (longline,188) (aero(i,jpsi),i=n0,n1)
           go to 123
      endif
      if (n1 .lt. mr) go to 121
      mpsi=jpsi
      endif
!
      ierror=0
      return
!
  100 ierror=1
      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine write_header(nf,nc,title,level,config,nrotor,opfree)
      integer nf,nc,level,config,nrotor,opfree
      character*80 title
!
!   write header
!
!   subroutine input and output
!   input
!      nf        integer    export file
!      nc        integer    number of cases
!      level     integer    wake level
!      config    integer    rotorcraft configuration
!      nrotor    integer    number of rotors
!      opfree    integer    wind tunnel (0) or free flight (1)
!
      character t*1
   99 format ('CAMRAD II Calculations'//a80)
   98 format (a44)
   97 format ('number of rotors =',i3)
   96 format (/'case',a1,100(a1,i4))
      t=char(9)
!
      write (nf,99) title
      if (level .le. 1) then
      write (nf,98) 'uniform inflow                              '
      else if (level .eq. 2) then
      write (nf,98) 'nonuniform inflow, prescribed wake geometry '
      else
      write (nf,98) 'nonuniform inflow, free wake geometry       '
      endif
      if (config .le. 0) then
      write (nf,98) 'one rotor                                   '
      else if (config .eq. 1) then
      write (nf,98) 'main rotor and tail rotor helicopter        '
      else if (config .eq. 2) then
      write (nf,98) 'tandem rotor helicopter                     '
      else if (config .eq. 3) then
      write (nf,98) 'coaxial rotor helicopter                    '
      else if (config .eq. 4) then
      write (nf,98) 'tilting proprotor aircraft                  '
      else
      write (nf,98) 'multirotor aircraft                         '
      endif
      write (nf,97) nrotor
      if (opfree .eq. 0) then
      write (nf,98) 'wind tunnel                                 '
      else
      write (nf,98) 'free flight                                 '
      endif
!
      if (nc .gt. 0) then
      write (nf,96) t,(t,n,n=1,nc)
      endif
!
      return
      end
!***********************************************************************************************************************************
      subroutine write_environment(nf,nc,e)
      integer nf,nc
      real e(9,*)
!
!   write environment
!
!   subroutine input and output
!   input
!      nf        integer    export file
!      nc        integer    number of cases
!      e(9,nc)   real       environment data
!
      character t*1
   99 format (/'Environment')
   98 format (' ')
   90 format (a24,a1,100(a1,f14.0))
   91 format (a24,a1,100(a1,f14.1))
   92 format (a24,a1,100(a1,f14.2))
   93 format (a24,a1,100(a1,f14.3))
   94 format (a24,a1,100(a1,f14.4))
   95 format (a24,a1,100(a1,f14.5))
   96 format (a24,a1,100(a1,f14.6))
   86 format (a24,a1,100(a1,g14.6))
      t=char(9)
!
      write (nf,99)
      write (nf,90) 'altitude (above msl)    ',t,(t,e(3,n),n=1,nc)
      write (nf,96) 'density                 ',t,(t,e(1,n),n=1,nc)
      write (nf,92) 'temperature             ',t,(t,e(2,n),n=1,nc)
      write (nf,92) 'speed of sound          ',t,(t,e(5,n),n=1,nc)
      write (nf,86) 'kinematic viscosity     ',t,(t,e(8,n),n=1,nc)
!
      return
      end
!***********************************************************************************************************************************
      subroutine write_opcond(nf,nc,o,nrotor,vtip,omega,nrtrmax)
      integer nf,nc,nrotor,nrtrmax
      real o(29,*),vtip(nrtrmax,*),omega(nrtrmax,*)
!
!   write operating condition
!
!   subroutine input and output
!   input
!      nf            integer    export file
!      nc            integer    number of cases
!      o(29,nc)      real       operating condition data
!      nrotor        integer    number of rotors
!      vtip(nr,nc)   real       tip speed
!      omega(nr,nc)  real       rotational speed
!
      character t*1
   99 format (/'Operating Condition')
   98 format (' ')
   90 format (a24,a1,100(a1,f14.0))
   91 format (a24,a1,100(a1,f14.1))
   92 format (a24,a1,100(a1,f14.2))
   93 format (a24,a1,100(a1,f14.3))
   94 format (a24,a1,100(a1,f14.4))
   95 format (a24,a1,100(a1,f14.5))
   96 format (a24,a1,100(a1,f14.6))
  192 format ('rotor',i2,1x,a16,a1,100(a1,f14.2))
      t=char(9)
!
      write (nf,99)
      write (nf,94) 'V/Vtip                  ',t,(t,o(2,n),n=1,nc)
      write (nf,92) 'flight speed (knots)    ',t,(t,o(1,n),n=1,nc)
      write (nf,94) 'tip Mach number         ',t,(t,o(13,n),n=1,nc)
      if (nrotor .le. 1) then
      write (nf,92) 'rotor tip speed         ',t,(t,o(12,n),n=1,nc)
      write (nf,92) 'rotor speed (rpm)       ',t,(t,o(14,n),n=1,nc)
      else
      do krtr=1,nrotor
      write (nf,192) krtr,'tip speed       ',t,(t,vtip(krtr,n),n=1,nc)
      enddo
      do krtr=1,nrotor
      write (nf,192) krtr,'speed (rpm)     ',t,(t,omega(krtr,n)*9.5492966,n=1,nc)
      enddo
      endif
!     write (nf,98)
      write (nf,92) 'climb angle (deg)       ',t,(t,o(7,n),n=1,nc)
      write (nf,92) 'sideslip angle (deg)    ',t,(t,o(8,n),n=1,nc)
!     write (nf,92) 'turn rate (deg/sec)     ',t,(t,o(15,n),n=1,nc)
!     write (nf,98)
      write (nf,92) 'pitch angle (deg)       ',t,(t,o(10,n),n=1,nc)
      write (nf,92) 'roll angle (deg)        ',t,(t,o(11,n),n=1,nc)
      write (nf,92) 'yaw angle (deg)         ',t,(t,o(9,n),n=1,nc)
!     write (nf,98)
      write (nf,92) 'collective              ',t,(t,o(16,n),n=1,nc)
      write (nf,92) 'lateral cyclic          ',t,(t,o(17,n),n=1,nc)
      write (nf,92) 'longitudinal cyclic     ',t,(t,o(18,n),n=1,nc)
      write (nf,92) 'pedal                   ',t,(t,o(19,n),n=1,nc)
!     write (nf,92) 'throttle                ',t,(t,o(20,n),n=1,nc)
!     write (nf,92) 'governor                ',t,(t,o(21,n),n=1,nc)
!     write (nf,98)
!
      return
      end
!***********************************************************************************************************************************
      subroutine write_rotorcraft_perf(nf,nc,opunit,w,q)
      integer nf,nc,opunit
      real w(*),q(50,*)
!
!   write operating condition
!
!   subroutine input and output
!   input
!      nf         integer    export file
!      nc         integer    number of cases
!      opunit     integer    unit system
!      w(nc)      real       gross weight
!      q(50,nc)   real       performance data
!
      character t*1
   99 format (/'Rotorcraft Performance')
   98 format (' ')
   90 format (a24,a1,100(a1,f14.0))
   91 format (a24,a1,100(a1,f14.1))
   92 format (a24,a1,100(a1,f14.2))
   93 format (a24,a1,100(a1,f14.3))
   94 format (a24,a1,100(a1,f14.4))
   95 format (a24,a1,100(a1,f14.5))
   96 format (a24,a1,100(a1,f14.6))
      t=char(9)
!
      write (nf,99)
      write (nf,92) 'gross weight            ',t,(t,w(n),n=1,nc)
!   English units
      if (opunit .le. 1) then
      write (nf,91) 'total power (hp)        ',t,(t,q(3,n),n=1,nc)
      write (nf,91) '  propulsive power (hp) ',t,(t,q(27,n),n=1,nc)
      write (nf,91) '  climb power (hp)      ',t,(t,q(31,n),n=1,nc)
      write (nf,91) '  ind+int power (hp)    ',t,(t,q(15,n)+q(19,n),n=1,nc)
      write (nf,91) '  profile power (hp)    ',t,(t,q(23,n),n=1,nc)
      write (nf,91) '  Po+Pi (hp)            ',t,(t,q(15,n)+q(19,n)+q(23,n),n=1,nc)
!   SI units
      else
      write (nf,91) 'total power (kW)        ',t,(t,q(4,n)*.001,n=1,nc)
      write (nf,91) '  propulsive power (kW) ',t,(t,q(28,n)*.001,n=1,nc)
      write (nf,91) '  climb power (kW)      ',t,(t,q(32,n)*.001,n=1,nc)
      write (nf,91) '  ind+int power (kW)    ',t,(t,(q(16,n)+q(20,n))*.001,n=1,nc)
      write (nf,91) '  profile power (kW)    ',t,(t,q(24,n)*.001,n=1,nc)
      write (nf,91) '  Po+Pi (kW)            ',t,(t,(q(16,n)+q(20,n)+q(24,n))*.001,n=1,nc)
      endif
!   indices
      write (nf,94) 'rotor L/D=WV/(Po+Pi)    ',t,(t,q(49,n),n=1,nc)
      write (nf,94) 'aircraft L/D=WV/P       ',t,(t,q(50,n),n=1,nc)
!
      return
      end
!***********************************************************************************************************************************
      subroutine write_rotor_perf(nf,nc,opunit,krtr,p)
      integer nf,nc,opunit,krtr
      real p(117,*)
!
!   write operating condition
!
!   subroutine input and output
!   input
!      nf          integer    export file
!      nc          integer    number of cases
!      opunit      integer    unit system
!      krtr        integer    rotor number
!      p(117,nc)   real       performance data
!
      character t*1
   99 format (/'Rotor',i2,' Performance')
   98 format (' ')
   90 format (a24,a1,100(a1,f14.0))
   91 format (a24,a1,100(a1,f14.1))
   92 format (a24,a1,100(a1,f14.2))
   93 format (a24,a1,100(a1,f14.3))
   94 format (a24,a1,100(a1,f14.4))
   95 format (a24,a1,100(a1,f14.5))
   96 format (a24,a1,100(a1,f14.6))
      t=char(9)
!
      write (nf,99) krtr
      write (nf,94) 'advance ratio mu        ',t,(t,p(4,n),n=1,nc)
      write (nf,94) 'mux                     ',t,(t,p(1,n),n=1,nc)
      write (nf,94) 'muy                     ',t,(t,p(2,n),n=1,nc)
      write (nf,94) 'muz                     ',t,(t,p(3,n),n=1,nc)
      write (nf,94) 'alpha (deg)             ',t,(t,p(8,n),n=1,nc)
      write (nf,94) 'tip Mach number Mtip    ',t,(t,p(6,n),n=1,nc)
      write (nf,94) 'advancing tip Mat       ',t,(t,p(7,n),n=1,nc)
!     write (nf,98)
      write (nf,92) 'collective (deg)        ',t,(t,p(15,n),n=1,nc)
      write (nf,92) 'thetas (deg)            ',t,(t,p(13,n),n=1,nc)
      write (nf,92) 'thetac (deg)            ',t,(t,p(14,n),n=1,nc)
      write (nf,92) 'coning (deg)            ',t,(t,p(12,n),n=1,nc)
      write (nf,92) 'betac (deg)             ',t,(t,p(10,n),n=1,nc)
      write (nf,92) 'betas (deg)             ',t,(t,p(11,n),n=1,nc)
!     write (nf,98)
      write (nf,95) 'CT/s                    ',t,(t,p(17,n),n=1,nc)
      write (nf,95) 'CH/s                    ',t,(t,p(20,n),n=1,nc)
      write (nf,96) 'CMx/s                   ',t,(t,p(26,n),n=1,nc)
      write (nf,96) 'CMy/s                   ',t,(t,p(29,n),n=1,nc)
      write (nf,95) 'CL/s                    ',t,(t,p(35,n),n=1,nc)
      write (nf,95) 'CX/s                    ',t,(t,p(38,n),n=1,nc)
      write (nf,96) 'CQ/s                    ',t,(t,p(41,n),n=1,nc)
      write (nf,91) 'thrust                  ',t,(t,p(18,n),n=1,nc)
      write (nf,91) 'Hforce                  ',t,(t,p(21,n),n=1,nc)
      write (nf,91) 'Yforce                  ',t,(t,p(24,n),n=1,nc)
      write (nf,91) 'lift                    ',t,(t,p(36,n),n=1,nc)
      write (nf,91) 'drag                    ',t,(t,p(39,n),n=1,nc)
!     write (nf,98)
      write (nf,96) 'CP/s                    ',t,(t,p(44,n),n=1,nc)
      write (nf,96) '  CPi/s                 ',t,(t,p(56,n),n=1,nc)
      write (nf,96) '  CPint/s               ',t,(t,p(60,n),n=1,nc)
      write (nf,96) '  CPo/s                 ',t,(t,p(64,n),n=1,nc)
!   English units
      if (opunit .le. 1) then
      write (nf,91) 'power (hp)              ',t,(t,p(45,n),n=1,nc)
      write (nf,91) '  induced power (hp)    ',t,(t,p(57,n),n=1,nc)
      write (nf,91) '  interference pwr (hp) ',t,(t,p(61,n),n=1,nc)
      write (nf,91) '  ind+int power (hp)    ',t,(t,p(57,n)+p(61,n),n=1,nc)
      write (nf,91) '  profile power (hp)    ',t,(t,p(65,n),n=1,nc)
      write (nf,91) '  prop+climb power (hp) ',t,(t,p(49,n),n=1,nc)
!   SI units
      else
      write (nf,91) 'power (kW)              ',t,(t,p(46,n)*.001,n=1,nc)
      write (nf,91) '  induced power (kW)    ',t,(t,p(58,n)*.001,n=1,nc)
      write (nf,91) '  interference pwr (kW) ',t,(t,p(62,n)*.001,n=1,nc)
      write (nf,91) '  ind+int power (kW)    ',t,(t,(p(58,n)+p(62,n))*.001,n=1,nc)
      write (nf,91) '  profile power (kW)    ',t,(t,p(66,n)*.001,n=1,nc)
      write (nf,91) '  prop+climb power (kW) ',t,(t,p(50,n)*.001,n=1,nc)
      endif
!     write (nf,98)
      write (nf,94) 'kappa = P/Pm            ',t,(t,p(83,n),n=1,nc)
      write (nf,96) 'cdo = 8CPo/s            ',t,(t,p(85,n),n=1,nc)
!     write (nf,93) 'figure of merit         ',t,(t,p(86,n),n=1,nc)
!     write (nf,93) 'propulsive efficiency   ',t,(t,p(90,n),n=1,nc)
!     write (nf,93) 'rotor lift/drag         ',t,(t,p(92,n),n=1,nc)
!
      return
      end
!***********************************************************************************************************************************

!***********************************************************************************************************************************
! Construct Acoustic Source Data from Extracted CAMRADII Data
!***********************************************************************************************************************************
      subroutine SetSource
      use CIIsound_structures
!
      parameter (lsave=mpsimax+20)
      real wsave(lsave),work(mpsimax),harm(mpsimax)
      integer opsnd
      real cc(3),tt(3),nn(3),aa(3),cmag,tmag,nmag
!
      do n=1,ncase
      nrtr=CIIoutput(n)%nrotor
!   English units to SI
      if (CIIoutput(n)%opunit .le. 1) then
      Source(n)%density=CIIoutput(n)%Condition%density*(N_lb/m_ft**4)
      Source(n)%csound =CIIoutput(n)%Condition%csound *m_ft
      Source(n)%kinvisc=CIIoutput(n)%Condition%kinvisc*m_ft**2
      Source(n)%speed  =CIIoutput(n)%Condition%speed  *m_ft
      Source(n)%nrotor=nrtr
      do i=1,nrtr
      opsnd=CIIoutput(n)%Rotor(i)%opsnd
      nbld=CIIoutput(n)%Rotor(i)%nblade
      npnl=CIIoutput(n)%Rotor(i)%npanel
      Source(n)%Rotor(i)%nblade=nbld
      Source(n)%Rotor(i)%npanel=npnl
      Source(n)%Rotor(i)%opsnd=opsnd
      Source(n)%Rotor(i)%rotate    =CIIoutput(n)%Rotor(i)%rotate
      Source(n)%Rotor(i)%radius    =CIIoutput(n)%Rotor(i)%radius    *m_ft
      Source(n)%Rotor(i)%Vtip      =CIIoutput(n)%Condition%Vtip(i)  *m_ft
      Source(n)%Rotor(i)%area_rotor=CIIoutput(n)%Rotor(i)%area_rotor*m_ft**2
      Source(n)%Rotor(i)%area_blade=CIIoutput(n)%Rotor(i)%area_blade*m_ft**2
      if (opsnd .ne. 0) then
      Source(n)%Rotor(i)%hub_force =CIIoutput(n)%Rotor(i)%hub_force *N_lb
      Source(n)%Rotor(i)%hub_disp  =CIIoutput(n)%Rotor(i)%hub_disp  *m_ft
      do m=1,nbld
      mpsi=CIIoutput(n)%Rotor(i)%Blade(m)%mpsi
      Source(n)%Rotor(i)%Blade(m)%mpsi=mpsi
      Source(n)%Rotor(i)%Blade(m)%Omega =CIIoutput(n)%Rotor(i)%Blade(m)%Omega
      Source(n)%Rotor(i)%Blade(m)%period=CIIoutput(n)%Rotor(i)%Blade(m)%period
      Source(n)%Rotor(i)%Blade(m)%psi   =CIIoutput(n)%Rotor(i)%Blade(m)%psi
      Source(n)%Rotor(i)%Blade(m)%time  =CIIoutput(n)%Rotor(i)%Blade(m)%time
      do k=1,npnl; do j=0,mpsi; do l=1,3
      Source(n)%Rotor(i)%Blade(m)%force(j,l,k)=CIIoutput(n)%Rotor(i)%Blade(m)%force(l,k,j)*N_lb
      Source(n)%Rotor(i)%Blade(m)%disp(j,l,k) =CIIoutput(n)%Rotor(i)%Blade(m)%disp(l,k,j) *m_ft
      Source(n)%Rotor(i)%Blade(m)%vel(j,l,k)  =CIIoutput(n)%Rotor(i)%Blade(m)%vel(l,k,j)  *m_ft
      Source(n)%Rotor(i)%Blade(m)%disp3(j,l,k)=CIIoutput(n)%Rotor(i)%Blade(m)%disp3(l,k,j)*m_ft
      enddo; enddo; enddo
      do k=1,npnl; do j=0,mpsi
      Source(n)%Rotor(i)%Blade(m)%Umag(j,k) =CIIoutput(n)%Rotor(i)%Blade(m)%Umag(k,j)*m_ft
      Source(n)%Rotor(i)%Blade(m)%alpha(j,k)=CIIoutput(n)%Rotor(i)%Blade(m)%alpha(k,j)
      Source(n)%Rotor(i)%Blade(m)%cl(j,k)   =CIIoutput(n)%Rotor(i)%Blade(m)%cl(k,j)
      enddo; enddo
      enddo; endif
      enddo
!   SI units
      else
      Source%density=CIIoutput(n)%Condition%density
      Source%csound =CIIoutput(n)%Condition%csound
      Source%kinvisc=CIIoutput(n)%Condition%kinvisc
      Source%speed  =CIIoutput(n)%Condition%speed
      Source%nrotor=nrtr
      do i=1,nrtr
      opsnd=CIIoutput(n)%Rotor(i)%opsnd
      nbld=CIIoutput(n)%Rotor(i)%nblade
      npnl=CIIoutput(n)%Rotor(i)%npanel
      Source(n)%Rotor(i)%nblade=nbld
      Source(n)%Rotor(i)%npanel=npnl
      Source(n)%Rotor(i)%opsnd=opsnd
      Source(n)%Rotor(i)%rotate    =CIIoutput(n)%Rotor(i)%rotate
      Source(n)%Rotor(i)%radius    =CIIoutput(n)%Rotor(i)%radius
      Source(n)%Rotor(i)%Vtip      =CIIoutput(n)%Condition%Vtip(i)
      Source(n)%Rotor(i)%area_rotor=CIIoutput(n)%Rotor(i)%area_rotor
      Source(n)%Rotor(i)%area_blade=CIIoutput(n)%Rotor(i)%area_blade
      if (opsnd .ne. 0) then
      Source(n)%Rotor(i)%hub_force =CIIoutput(n)%Rotor(i)%hub_force
      Source(n)%Rotor(i)%hub_disp  =CIIoutput(n)%Rotor(i)%hub_disp
      do m=1,nbld
      mpsi=CIIoutput(n)%Rotor(i)%Blade(m)%mpsi
      Source(n)%Rotor(i)%Blade(m)%mpsi=mpsi
      Source(n)%Rotor(i)%Blade(m)%Omega =CIIoutput(n)%Rotor(i)%Blade(m)%Omega
      Source(n)%Rotor(i)%Blade(m)%period=CIIoutput(n)%Rotor(i)%Blade(m)%period
      Source(n)%Rotor(i)%Blade(m)%psi   =CIIoutput(n)%Rotor(i)%Blade(m)%psi
      Source(n)%Rotor(i)%Blade(m)%time  =CIIoutput(n)%Rotor(i)%Blade(m)%time
      do k=1,npnl; do j=0,mpsi; do l=1,3
      Source(n)%Rotor(i)%Blade(m)%force(j,l,k)=CIIoutput(n)%Rotor(i)%Blade(m)%force(l,k,j)
      Source(n)%Rotor(i)%Blade(m)%disp(j,l,k) =CIIoutput(n)%Rotor(i)%Blade(m)%disp(l,k,j)
      Source(n)%Rotor(i)%Blade(m)%vel(j,l,k)  =CIIoutput(n)%Rotor(i)%Blade(m)%vel(l,k,j)
      Source(n)%Rotor(i)%Blade(m)%disp3(j,l,k)=CIIoutput(n)%Rotor(i)%Blade(m)%disp3(l,k,j)
      enddo; enddo; enddo
      do k=1,npnl; do j=0,mpsi
      Source(n)%Rotor(i)%Blade(m)%Umag(j,k) =CIIoutput(n)%Rotor(i)%Blade(m)%Umag(k,j)
      Source(n)%Rotor(i)%Blade(m)%alpha(j,k)=CIIoutput(n)%Rotor(i)%Blade(m)%alpha(k,j)
      Source(n)%Rotor(i)%Blade(m)%cl(j,k)   =CIIoutput(n)%Rotor(i)%Blade(m)%cl(k,j)
      enddo; enddo
      enddo; endif
      enddo
      endif
!
      do i=1,nrtr
      if (Source(n)%Rotor(i)%opsnd .ne. 0) then
      nbld=CIIoutput(n)%Rotor(i)%nblade
      npnl=CIIoutput(n)%Rotor(i)%npanel
      do m=1,nbld; do k=1,npnl; do l=1,3
!   force derivative
      do j=0,mpsi-1
      t2=Source(n)%Rotor(i)%Blade(m)%time(j+1); t1=Source(n)%Rotor(i)%Blade(m)%time(j)
      t=0.5*(t2+t1); dt=t2-t1
      df=Source(n)%Rotor(i)%Blade(m)%force(j+1,l,k)-Source(n)%Rotor(i)%Blade(m)%force(j,l,k)
      Source(n)%Rotor(i)%Blade(m)%time_deriv(j)=t
      Source(n)%Rotor(i)%Blade(m)%force_deriv(j,l,k)=df/dt
      enddo
      period=Source(n)%Rotor(i)%Blade(m)%period
      Source(n)%Rotor(i)%Blade(m)%time_deriv(mpsi)=Source(n)%Rotor(i)%Blade(m)%time_deriv(0)+period
      Source(n)%Rotor(i)%Blade(m)%force_deriv(mpsi,l,k)=Source(n)%Rotor(i)%Blade(m)%force_deriv(0,l,k)
!   harmonics
      call rfft1i (mpsi,wsave,lsave,ierror )
         if (ierror .ne. 0) write (6,*) '  initialize fft error',ierror
!   force
      do j=1,mpsi; harm(j)=Source(n)%Rotor(i)%Blade(m)%force(j-1,l,k); enddo
      call rfft1f (mpsi,1,harm,mpsimax,wsave,lsave,work,mpsimax,ierror )
         if (ierror .ne. 0) write (6,*) '  force: forward fft error',ierror
      do j=1,mpsi; Source(n)%Rotor(i)%Blade(m)%force_harm(j,l,k)=harm(j); enddo
!   disp
      do j=1,mpsi; harm(j)=Source(n)%Rotor(i)%Blade(m)%disp(j-1,l,k); enddo
      call rfft1f (mpsi,1,harm,mpsimax,wsave,lsave,work,mpsimax,ierror )
         if (ierror .ne. 0) write (6,*) '  displacement: forward fft error',ierror
      do j=1,mpsi; Source(n)%Rotor(i)%Blade(m)%disp_harm(j,l,k)=harm(j); enddo
!   vel
      do j=1,mpsi; harm(j)=Source(n)%Rotor(i)%Blade(m)%vel(j-1,l,k); enddo
      call rfft1f (mpsi,1,harm,mpsimax,wsave,lsave,work,mpsimax,ierror )
         if (ierror .ne. 0) write (6,*) '  velocity: forward fft error',ierror
      do j=1,mpsi; Source(n)%Rotor(i)%Blade(m)%vel_harm(j,l,k)=harm(j); enddo
      enddo; enddo; enddo
!   blade cross-section area
      nspan=theInput%Nspan
      rroot=CIIoutput(n)%Rotor(i)%ra(1)   -0.5*CIIoutput(n)%Rotor(i)%dra(1); Source(n)%Rotor(i)%rroot=rroot
      rtip =CIIoutput(n)%Rotor(i)%ra(npnl)+0.5*CIIoutput(n)%Rotor(i)%dra(npnl); Source(n)%Rotor(i)%rtip=rtip
      dr=(rtip-rroot)/float(nspan); Source(n)%Rotor(i)%dr_thick=dr
      do ispan=1,nspan
      r=rroot+float(ispan-1)*dr+0.5*dr
      call linear_interp(r,theInput%nprop,theInput%rprop(1,i),theInput%thick(1,i),0.,thick)
      call linear_interp(r,CIIoutput(n)%Rotor(i)%nprop,CIIoutput(n)%Rotor(i)%rprop,CIIoutput(n)%Rotor(i)%chord,0.,chord)
      if (CIIoutput(n)%opunit .le. 1) chord=chord*m_ft
      Source(n)%Rotor(i)%Axs(ispan)=0.685*thick*(chord**2)
      enddo
!   broadband noise
      if (theInput%MODEL_broadband .ge. 2) then
      do k=1,npnl
      r=CIIoutput(n)%Rotor(i)%ra(k)
      call linear_interp(r,CIIoutput(n)%Rotor(i)%nprop,CIIoutput(n)%Rotor(i)%rprop,CIIoutput(n)%Rotor(i)%chord,0.,chord)
      call linear_interp(r,theInput%nprop,theInput%rprop(1,i),theInput%alpha_zl(1,i),0.,alpha_zl)
      call linear_interp(r,theInput%nprop,theInput%rprop(1,i),theInput%hTE(1,i),0.,hTE)
      call linear_interp(r,theInput%nprop,theInput%rprop(1,i),theInput%psiTE(1,i),0.,psiTE)
      Source(n)%Rotor(i)%chord(k)=chord
      Source(n)%Rotor(i)%alpha_zl(k)=alpha_zl
      Source(n)%Rotor(i)%hTE(k)=hTE*chord
      Source(n)%Rotor(i)%psiTE(k)=psiTE
      enddo; endif
      endif; enddo
!
!   local airfoil segment axes
      do i=1,nrtr
      opsnd=CIIoutput(n)%Rotor(i)%opsnd
      nbld=CIIoutput(n)%Rotor(i)%nblade
      npnl=CIIoutput(n)%Rotor(i)%npanel
      if (opsnd .ne. 0) then
      do m=1,nbld
      mpsi=CIIoutput(n)%Rotor(i)%Blade(m)%mpsi
      do k=1,npnl; do j=0,mpsi
        do l=1,3; cc(l)=Source(n)%Rotor(i)%Blade(m)%disp3(j,l,k)-Source(n)%Rotor(i)%Blade(m)%disp(j,l,k); enddo ! c=x3qc(k)-xqc(k)
        k1=max(1,k-1); k2=min(npnl,k+1)
        do l=1,3; aa(l)=Source(n)%Rotor(i)%Blade(m)%disp(j,l,k2)-Source(n)%Rotor(i)%Blade(m)%disp(j,l,k1); enddo ! a=x(k+1)-x(k-1)
        if (Source(n)%Rotor(i)%rotate .lt. 0) aa=-aa ! CW rotation - change sign a
        nn(1)=cc(2)*aa(3)-cc(3)*aa(2); nn(2)=cc(3)*aa(1)-cc(1)*aa(3); nn(3)=cc(1)*aa(2)-cc(2)*aa(1) ! n=cxa
        tt(1)=nn(2)*cc(3)-nn(3)*cc(2); tt(2)=nn(3)*cc(1)-nn(1)*cc(3); tt(3)=nn(1)*cc(2)-nn(2)*cc(1)! t=nxc
        cmag=cc(1)**2+cc(2)**2+cc(3)**2; if (cmag .ne. 0.) cmag=1./sqrt(cmag); cc=cc*cmag
        tmag=tt(1)**2+tt(2)**2+tt(3)**2; if (tmag .ne. 0.) tmag=1./sqrt(tmag); tt=tt*tmag
        nmag=nn(1)**2+nn(2)**2+nn(3)**2; if (nmag .ne. 0.) nmag=1./sqrt(nmag); nn=nn*nmag
        if (abs(Source(n)%Rotor(i)%Blade(m)%alpha(j,k)) .gt. 90.) then; cc=-cc; nn=-nn; endif ! reverse flow - change sign c,n
        if (Source(n)%Rotor(i)%Blade(m)%cl(j,k) .lt. 0.) then; nn=-nn; tt=-tt; endif ! negative lift - change sign n,t
        do l=1,3
        Source(n)%Rotor(i)%Blade(m)%echord(j,l,k)=cc(l)
        Source(n)%Rotor(i)%Blade(m)%etangent(j,l,k)=tt(l)
        Source(n)%Rotor(i)%Blade(m)%enormal(j,l,k)=nn(l)
        enddo
      enddo; enddo
      enddo
      endif; enddo
!
      enddo
!
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
!***********************************************************************************************************************************
! Calculate Sound
!***********************************************************************************************************************************
      subroutine GetSound
      use CIIsound_structures
!
      integer opsnd
      real xtraj(3),xmic(3),CFI(3,3)
      real x(3),f(3),df(3),y(3),v(3),dv(3),r(3),e(3),MM(3)
      real rmag,Mr,dMr,Msq,Mtip,kinvisc
      real tobs(0:ntimemax),press(0:ntimemax)
      real ptblte(0:40),plblvs(0:40),pblunt(0:40),ptip(0:40),pbb(0:40)
      real dptblte(0:40),dplblvs(0:40),dpblunt(0:40),dptip(0:40)
!
      Ntime=theInput%Ntime
      Nspan=theInput%Nspan
      Nbbave=theInput%Nbbave
!
      do nc=1,ncase
      nrtr=Source(nc)%nrotor
      density=Source(nc)%density
      csound=Source(nc)%csound
      kinvisc=Source(nc)%kinvisc
      speed=Source(nc)%speed
      climb=CIIoutput(nc)%Condition%climb
      CFI=CIIoutput(nc)%Condition%CFI
      nobs=theInput%nObserver(nc)
      do io=1,nobs
!   observer
      xtraj=0.; ttraj=0.; xmic=0.
      if (theInput%KIND_obs(nc) .le. 0) then ! trajectory
           dx=0.; if (theInput%Ntraj .gt. 1) dx=(theInput%xend(nc)-theInput%xbegin(nc))/float(theInput%Ntraj-1); 
           xx=theInput%xbegin(nc)+float(io-1)*dx
           if (theInput%KIND_traj(nc) .le. 0) then; xref=theInput%xref(nc); zref=theInput%zref(nc) ! general
           else if (theInput%KIND_traj(nc) .eq. 1) then; xref=-500.; zref=  0. ! takeoff
           else if (theInput%KIND_traj(nc) .eq. 2) then; xref=   0.; zref=150. ! flyover
           else if (theInput%KIND_traj(nc) .ge. 3) then; xref=   0.; zref=120. ! approach
           endif
           theInput%xref(nc)=xref; theInput%zref(nc)=zref
           zz=-zref-tan(raddeg*climb)*(xx-xref); xtraj(1)=xx; xtraj(2)=0.; xtraj(3)=zz
           ttraj=0.; if (speed .ne. 0.) ttraj=xx/(speed*cos(raddeg*climb))
           if (theInput%KIND_mic(nc) .le. 0) then; do ii=1,3; xmic(ii)=theInput%xmic(ii,nc); enddo  ! general
           else if (theInput%KIND_mic(nc) .eq. 1) then; xmic(2)=   0. ! center
           else if (theInput%KIND_mic(nc) .eq. 2) then; xmic(2)= 150. ! starboard
           else if (theInput%KIND_mic(nc) .ge. 3) then; xmic(2)=-150. ! port
           endif
           theInput%xmic(2,nc)=xmic(2)
           do ii=1,3; x(ii)=0.; do jj=1,3; x(ii)=x(ii)+CFI(ii,jj)*(xmic(jj)-xtraj(jj)); enddo; enddo
      else if (theInput%KIND_obs(nc) .eq. 1) then ! input (airframe)
           do ii=1,3; x(ii)=theInput%position_obs(ii,io,nc); enddo
      else if (theInput%KIND_obs(nc) .ge. 2) then ! input (inertial)
           do ii=1,3; x(ii)=0.; do jj=1,3; x(ii)=x(ii)+CFI(ii,jj)*theInput%position_obs(jj,io,nc); enddo; enddo
      endif
      Sound(nc,io)%density=density
      Sound(nc,io)%csound=csound
      Sound(nc,io)%kinvisc=kinvisc
      Sound(nc,io)%speed=speed
      Sound(nc,io)%nrotor=nrtr
      Sound(nc,io)%position_obs=x
      Sound(nc,io)%position_traj=xtraj
      Sound(nc,io)%time_traj=ttraj
      Sound(nc,io)%position_mic=xmic
!   identify rotors with same period
      Sound(nc,io)%primary=0
      do i=1,nrtr; if (Source(nc)%Rotor(i)%opsnd .ne. 0) then
      if (CIIoutput(nc)%Rotor(i)%oprpm .ne. 0) then
           if (CIIoutput(nc)%Rotor(i)%rgear .eq. 1.) then
                k=CIIoutput(nc)%Rotor(i)%oprpm
                if (Source(nc)%Rotor(k)%opsnd .ne. 0) Sound(nc,io)%primary(i)=k
           endif
      else
           if (i .gt. 1) then; do k=1,i-1; if (Source(nc)%Rotor(k)%opsnd .ne. 0) then
                if (CIIoutput(nc)%Rotor(k)%Blade(1)%Omega .eq. CIIoutput(nc)%Rotor(i)%Blade(1)%Omega) then
                     Sound(nc,io)%primary(i)=k; go to 1
                endif
           endif; enddo; endif
    1      continue
      endif
      endif; enddo
!
      do i=1,nrtr
      opsnd=Source(nc)%Rotor(i)%opsnd
      nbld =Source(nc)%Rotor(i)%nblade
      npnl =Source(nc)%Rotor(i)%npanel
      Sound(nc,io)%Rotor(i)%opsnd=opsnd
      Sound(nc,io)%Rotor(i)%nblade=nbld
      Sound(nc,io)%Rotor(i)%npanel=npnl
      if (opsnd .ne. 0) then
      radius  =Source(nc)%Rotor(i)%radius
      rroot   =Source(nc)%Rotor(i)%rroot
      dr_thick=Source(nc)%Rotor(i)%dr_thick
      Omega=Source(nc)%Rotor(i)%Blade(1)%Omega
      period=Source(nc)%Rotor(i)%Blade(1)%period
      call InitSoundValue(Ntime,Omega,period,nbld,Sound(nc,io)%sound_loading(i))
      call InitSoundValue(Ntime,Omega,period,nbld,Sound(nc,io)%sound_thickness(i))
      call InitSoundValue(Ntime,Omega,period,nbld,Sound(nc,io)%sound_rotational(i))
      call InitSoundValue(Ntime,Omega,period,nbld,Sound(nc,io)%Rotor(i)%sound_loading)
      call InitSoundValue(Ntime,Omega,period,nbld,Sound(nc,io)%Rotor(i)%sound_thickness)
      call InitSoundValue(Ntime,Omega,period,nbld,Sound(nc,io)%Rotor(i)%sound_rotational)
      do m=1,nbld
      Omega=Source(nc)%Rotor(i)%Blade(m)%Omega
      period=Source(nc)%Rotor(i)%Blade(m)%period
      dt=period/float(Ntime)
      Sound(nc,io)%Rotor(i)%Blade(m)%Ntime=Ntime
      Sound(nc,io)%Rotor(i)%Blade(m)%Omega=Omega
      Sound(nc,io)%Rotor(i)%Blade(m)%period=period
      call GetFreq2Band(Ntime,period,Sound(nc,io)%Rotor(i)%Blade(m)%freq2band)
      call InitSoundValue(Ntime,Omega,period,nbld,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading)
      call InitSoundValue(Ntime,Omega,period,nbld,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness)
!
!   loading noise
      do k=1,npnl
!   source time
      do j=0,Ntime
         time=float(j)*dt
         call GetLoading(nc,i,m,k,time,f,df)
         call GetMotion_panel(nc,i,m,k,time,y,v,dv)
            r=x-y; rmag=sqrt(r(1)**2+r(2)**2+r(3)**2); e=r/rmag; MM=v/csound; Msq=MM(1)**2+MM(2)**2+MM(3)**2
            Mr=e(1)*MM(1)+e(2)*MM(2)+e(3)*MM(3); dMr=(e(1)*dv(1)+e(2)*dv(2)+e(3)*dv(3))/csound
            fr=e(1)*f(1)+e(2)*f(2)+e(3)*f(3); dfr=e(1)*df(1)+e(2)*df(2)+e(3)*df(3); fM=f(1)*MM(1)+f(2)*MM(2)+f(3)*MM(3)
            rinv=1./rmag; dop=1./(1.-Mr)
         dp=rinv*(dop**2)*(dfr/csound  + rinv*(fr-fM) + rinv*dop*fr*(rmag*dMr/csound+Mr-Msq))/(4.*pi)
         tobs(j)=time+rmag/csound
         press(j)=-dp !
      enddo
!     do j=1,Ntime
!     if (tobs(j) .le. tobs(j-1)) write (6,*) '  error, observer time not sequential'
!     enddo
!   observer time
      do j=0,Ntime
         time=float(j)*dt
         call linear_interp(time,Ntime,tobs,press,period,dp)
         Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%press(j)=Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%press(j)+dp
      enddo
      enddo
!
!   thickness noise
      do k=1,Nspan
      span=rroot+float(k-1)*dr_thick+0.5*dr_thick
      AxsDs=Source(nc)%Rotor(i)%Axs(k)*(dr_thick*radius)
!   source time
      do j=0,Ntime
         time=float(j)*dt
         call GetMotion_span(nc,i,m,span,time+dt,y,v,dv)
            r=x-y; rmag=sqrt(r(1)**2+r(2)**2+r(3)**2); e=r/rmag; MM=v/csound; Msq=MM(1)**2+MM(2)**2+MM(3)**2
            Mr=e(1)*MM(1)+e(2)*MM(2)+e(3)*MM(3); dMr=(e(1)*dv(1)+e(2)*dv(2)+e(3)*dv(3))/csound
            rinv=1./rmag; dop=1./(1.-Mr)
         x2=(rinv**2)*(dop**3)*(rmag*dMr+Mr*csound-Msq*csound)/(4.*pi)
         call GetMotion_span(nc,i,m,span,time-dt,y,v,dv)
            r=x-y; rmag=sqrt(r(1)**2+r(2)**2+r(3)**2); e=r/rmag; MM=v/csound; Msq=MM(1)**2+MM(2)**2+MM(3)**2
            Mr=e(1)*MM(1)+e(2)*MM(2)+e(3)*MM(3); dMr=(e(1)*dv(1)+e(2)*dv(2)+e(3)*dv(3))/csound
            rinv=1./rmag; dop=1./(1.-Mr)
         x1=(rinv**2)*(dop**3)*(rmag*dMr+Mr*csound-Msq*csound)/(4.*pi)
         call GetMotion_span(nc,i,m,span,time,y,v,dv)
            r=x-y; rmag=sqrt(r(1)**2+r(2)**2+r(3)**2); e=r/rmag; MM=v/csound
            Mr=e(1)*MM(1)+e(2)*MM(2)+e(3)*MM(3)
            dop=1./(1.-Mr)
         dp=density*AxsDs*dop*(x2-x1)/(2.*dt)
         tobs(j)=time+rmag/csound
         press(j)=dp
      enddo
!     do j=1,Ntime
!     if (tobs(j) .le. tobs(j-1)) write (6,*) '  error, observer time not sequential'
!     enddo
!   observer time
      do j=0,Ntime
         time=float(j)*dt
         call linear_interp(time,Ntime,tobs,press,period,dp)
         Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%press(j)=Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%press(j)+dp
      enddo
      enddo
!
!   sum blades (pressure)
      do j=0,Ntime
      dpload =Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%press(j)
      dpthick=Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%press(j)
      Sound(nc,io)%Rotor(i)%sound_loading%press(j)   =Sound(nc,io)%Rotor(i)%sound_loading%press(j)   +dpload
      Sound(nc,io)%Rotor(i)%sound_thickness%press(j) =Sound(nc,io)%Rotor(i)%sound_thickness%press(j) +dpthick
      Sound(nc,io)%Rotor(i)%sound_rotational%press(j)=Sound(nc,io)%Rotor(i)%sound_rotational%press(j)+dpload+dpthick
      enddo
!   end blade
      enddo
!
!   broadband noise - Pegg
      if (theInput%MODEL_broadband .le. 1) then
      f=Source(nc)%Rotor(i)%hub_force
      y=Source(nc)%Rotor(i)%hub_disp
         r=x-y; rmag=sqrt(r(1)**2+r(2)**2+r(3)**2); rinv=1./rmag
         thrust=sqrt(f(1)**2+f(2)**2+f(3)**2)
         rTinv=0.; if ((rmag .ne. 0.) .and. (thrust .ne. 0.)) rTinv=1./(rmag*thrust)
         costheta=(f(1)*r(1)+f(2)*r(2)+f(3)*r(3))*rTinv; costheta=max(-1.,min(1.,costheta))
      CTs=CIIoutput(nc)%Condition%CTs(i)
      Vtip=abs(Source(nc)%Rotor(i)%Vtip); Mtip=Vtip/csound
      area_blade=Source(nc)%Rotor(i)%area_blade
         clmean=6.*abs(CTs); fcl=0.
         if ((clmean .gt. 0.) .and. (clmean .le. 0.48)) fcl=10.*log10(clmean/0.4)
         if (clmean .gt.0.48) fcl=0.7918+80.*log10(clmean/0.48)
      specP=60.*log10(Mtip)+10.*log10(area_blade*(costheta**2+0.1)*rinv**2)+fcl+130.
      freqP=-240.*log10(thrust)+2.448*Vtip+942.
      call GetThirdOctaveBand(freqP,kbandP)
      do kband=0,40
      spec=-101.
      kPegg=kband-kbandP; if ((kPegg .ge. -17) .and. (kPegg .le. 23)) spec=specP+SPegg_third(kPegg)
      Sound(nc,io)%Rotor(i)%sound_broadband%spectrum(kband)=spec
      enddo
      Sound(nc,io)%Rotor(i)%sound_bb_TBLTE%spectrum_press=0.
      Sound(nc,io)%Rotor(i)%sound_bb_LBLVS%spectrum_press=0.
      Sound(nc,io)%Rotor(i)%sound_bb_BLUNT%spectrum_press=0.
      Sound(nc,io)%Rotor(i)%sound_bb_TIP%spectrum_press=0.
!   broadband noise - Brooks
      else
      ptblte=0.; plblvs=0.; pblunt=0.; ptip=0.
      do m=1,nbld
      do k=1,npnl
      period=Source(nc)%Rotor(i)%Blade(m)%period
      dt=period/float(Nbbave)
!   source time
      do j=1,Nbbave
         time=float(j)*dt
         call GetMotion_panel(nc,i,m,k,time,y,v,dv)
            r=x-y; rmag=sqrt(r(1)**2+r(2)**2+r(3)**2); e=r/rmag
            Mr=e(1)*MM(1)+e(2)*MM(2)+e(3)*MM(3)
         call GetBroadband(nc,i,m,k,time,r,Mr,dptblte,dplblvs,dpblunt,dptip)
         factor=(1.-Mr)/float(Nbbave)
         ptblte=ptblte+dptblte*factor; plblvs=plblvs+dplblvs*factor; pblunt=pblunt+dpblunt*factor; ptip=ptip+dptip*factor
      enddo
      enddo
      enddo
      pbb=0.;
      if (theInput%OMIT_TBLTE .eq. 0) pbb=pbb+ptblte
      if (theInput%OMIT_LBLVS .eq. 0) pbb=pbb+plblvs
      if (theInput%OMIT_BLUNT .eq. 0) pbb=pbb+pblunt
      if (theInput%OMIT_TIP   .eq. 0) pbb=pbb+ptip
      Sound(nc,io)%Rotor(i)%sound_broadband%spectrum_press=pbb
      Sound(nc,io)%Rotor(i)%sound_bb_TBLTE%spectrum_press=ptblte
      Sound(nc,io)%Rotor(i)%sound_bb_LBLVS%spectrum_press=plblvs
      Sound(nc,io)%Rotor(i)%sound_bb_BLUNT%spectrum_press=pblunt
      Sound(nc,io)%Rotor(i)%sound_bb_TIP%spectrum_press=ptip
      endif
!
!   geometry
      y=Source(nc)%Rotor(i)%hub_disp; r=x-y; rmag=sqrt(r(1)**2+r(2)**2+r(3)**2); e=r/rmag
      Sound(nc,io)%Rotor(i)%distance=rmag
      Sound(nc,io)%Rotor(i)%elevation=-degrad*asin(max(-1.,min(1.,e(3))))
      Sound(nc,io)%Rotor(i)%azimuth=degrad*atan2(e(2),-e(1))
!   end rotor
      endif; enddo
!
!   sum all rotors with same period
      do i=1,nrtr; if (Source(nc)%Rotor(i)%opsnd .ne. 0) then
      Sound(nc,io)%sound_loading(i)   =Sound(nc,io)%Rotor(i)%sound_loading
      Sound(nc,io)%sound_thickness(i) =Sound(nc,io)%Rotor(i)%sound_thickness
      Sound(nc,io)%sound_rotational(i)=Sound(nc,io)%Rotor(i)%sound_rotational
      endif; enddo
      do k=1,nrtr; if (Source(nc)%Rotor(k)%opsnd .ne. 0) then
      i=Sound(nc,io)%primary(k); if (i .ne. 0) then
            Sound(nc,io)%sound_loading(i)%press   =Sound(nc,io)%sound_loading(i)%press   +Sound(nc,io)%sound_loading(k)%press
            Sound(nc,io)%sound_thickness(i)%press =Sound(nc,io)%sound_thickness(i)%press +Sound(nc,io)%sound_thickness(k)%press
            Sound(nc,io)%sound_rotational(i)%press=Sound(nc,io)%sound_rotational(i)%press+Sound(nc,io)%sound_rotational(k)%press
            Sound(nc,io)%sound_loading(k)%press=0.
            Sound(nc,io)%sound_thickness(k)%press=0.
            Sound(nc,io)%sound_rotational(k)%press=0.
      endif
      endif; enddo
!   complete sound calculation
      do i=1,nrtr; if (Source(nc)%Rotor(i)%opsnd .ne. 0) then
      nbld =Source(nc)%Rotor(i)%nblade
      do m=1,nbld
      call CompleteSoundValue(Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading,  Sound(nc,io)%Rotor(i)%Blade(m)%freq2band)
      call CompleteSoundValue(Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness,Sound(nc,io)%Rotor(i)%Blade(m)%freq2band)
      enddo
      call CompleteSoundValue(Sound(nc,io)%Rotor(i)%sound_loading,   Sound(nc,io)%Rotor(i)%Blade(nbld)%freq2band)
      call CompleteSoundValue(Sound(nc,io)%Rotor(i)%sound_thickness, Sound(nc,io)%Rotor(i)%Blade(nbld)%freq2band)
      call CompleteSoundValue(Sound(nc,io)%Rotor(i)%sound_rotational,Sound(nc,io)%Rotor(i)%Blade(nbld)%freq2band)
      call CompleteSoundValue(Sound(nc,io)%sound_loading(i),   Sound(nc,io)%Rotor(i)%Blade(nbld)%freq2band)
      call CompleteSoundValue(Sound(nc,io)%sound_thickness(i), Sound(nc,io)%Rotor(i)%Blade(nbld)%freq2band)
      call CompleteSoundValue(Sound(nc,io)%sound_rotational(i),Sound(nc,io)%Rotor(i)%Blade(nbld)%freq2band)
      if (theInput%MODEL_broadband .le. 1) then
      call CompleteSoundValueF(Sound(nc,io)%Rotor(i)%sound_broadband,2)
      else
      call CompleteSoundValueF(Sound(nc,io)%Rotor(i)%sound_broadband,1)
      endif
      call CompleteSoundValueF(Sound(nc,io)%Rotor(i)%sound_bb_TBLTE,1)
      call CompleteSoundValueF(Sound(nc,io)%Rotor(i)%sound_bb_LBLVS,1)
      call CompleteSoundValueF(Sound(nc,io)%Rotor(i)%sound_bb_BLUNT,1)
      call CompleteSoundValueF(Sound(nc,io)%Rotor(i)%sound_bb_TIP,1)
      endif; enddo
!   add rotational and broadband
      do i=1,nrtr; if (Source(nc)%Rotor(i)%opsnd .ne. 0) then
      if (theInput%OMIT_broadband .eq. 0) then
      Sound(nc,io)%Rotor(i)%sound_total%spectrum_press=Sound(nc,io)%Rotor(i)%sound_rotational%spectrum_press &
                                                      +Sound(nc,io)%Rotor(i)%sound_broadband%spectrum_press
      else
      Sound(nc,io)%Rotor(i)%sound_total%spectrum_press=Sound(nc,io)%Rotor(i)%sound_rotational%spectrum_press
      endif
      call CompleteSoundValueF(Sound(nc,io)%Rotor(i)%sound_total,1)
      endif; enddo
!   add rotors
      Sound(nc,io)%sound_load_total%spectrum_press=0.
      Sound(nc,io)%sound_thick_total%spectrum_press=0.
      Sound(nc,io)%sound_rot_total%spectrum_press=0.
      Sound(nc,io)%sound_broadband%spectrum_press=0.
      Sound(nc,io)%sound_total%spectrum_press=0.
      do i=1,nrtr; if (Source(nc)%Rotor(i)%opsnd .ne. 0) then
      Sound(nc,io)%sound_load_total%spectrum_press=Sound(nc,io)%sound_load_total%spectrum_press &
                                                  +Sound(nc,io)%sound_loading(i)%spectrum_press
      Sound(nc,io)%sound_thick_total%spectrum_press=Sound(nc,io)%sound_thick_total%spectrum_press &
                                                   +Sound(nc,io)%sound_thickness(i)%spectrum_press
      Sound(nc,io)%sound_rot_total%spectrum_press=Sound(nc,io)%sound_rot_total%spectrum_press &
                                                 +Sound(nc,io)%sound_rotational(i)%spectrum_press
      Sound(nc,io)%sound_broadband%spectrum_press=Sound(nc,io)%sound_broadband%spectrum_press &
                                                 +Sound(nc,io)%Rotor(i)%sound_broadband%spectrum_press
      Sound(nc,io)%sound_total%spectrum_press=Sound(nc,io)%sound_total%spectrum_press &
                                             +Sound(nc,io)%sound_rotational(i)%spectrum_press
      if (theInput%OMIT_broadband .eq. 0) then
      Sound(nc,io)%sound_total%spectrum_press=Sound(nc,io)%sound_total%spectrum_press &
                                             +Sound(nc,io)%Rotor(i)%sound_broadband%spectrum_press
      endif
      endif; enddo
      call CompleteSoundValueF(Sound(nc,io)%sound_load_total,1)
      call CompleteSoundValueF(Sound(nc,io)%sound_thick_total,1)
      call CompleteSoundValueF(Sound(nc,io)%sound_rot_total,1)
      call CompleteSoundValueF(Sound(nc,io)%sound_broadband,1)
      call CompleteSoundValueF(Sound(nc,io)%sound_total,1)
!
      Sound(nc,io)%SEL=0.
      Sound(nc,io)%EPNL=0.
!   end observer
      enddo
!
!   sound exposure level
      call GetSEL(nc)
!   effective perceived noise level
      call GetEPNL(nc)
!   end case
      enddo
!
      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
!  Rotor Self Noise Calculation
!
!  Airfoil Self-Noise and Prediction (Appendix D, Program PREDICT)
!  T.F. Brooks, D.S. Pope, and M.A. Marcolini
!  NASA Reference Publication 1218
!  July 1989
!
!  Aircraft Noise Prediction Program Theoretical Manual. Rotorcraft System Noise Prediction System (ROTONET).
!  D.S. Weir, S.J. Jumper, C.L. Burley, and R.A. Golub
!  NASA TM 83199, Part 4, April 1995.
!  Rotor Broadband Noise (RBN) Module
!  C.L. Burley, D.S. Weir, and S.J. Jumper
!
!  Source:
!  a) NASA TP 1218 Program PREDICT
!  b) reconcilation of code and equations in TP 1218 ("original code")
!  c) ROTONET revisions, including directivity
!  d) single segment (tip vortex noise only for tip segment)
!  e) 1/3-octave frequencies = 1.25 to 40000
!  f) use module
!  g) construct subroutine
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      module broadband_structures
!
      integer, parameter:: maxfreq=46
!   1/3 octave center frequencies
      integer, parameter :: nfreq=46
      real, parameter :: frcen(46) = [1.2500,1.6000,2.0000,2.5000,3.1500,4.0000,5.0000,6.3000,8.0000,10.0000, &
                                      12.500,16.000,20.000,25.000,31.500,40.000,50.000,63.000,80.000,100.000, &
                                      125.00,160.00,200.00,250.00,315.00,400.00,500.00,630.00,800.00,1000.00, &
                                      1250.0,1600.0,2000.0,2500.0,3150.0,4000.0,5000.0,6300.0,8000.0,10000.0, &
                                      12500.,16000.,20000.,25000.,31500.,40000.]
!
      real c,L,r,rx,ry,rz,alpstar,u,h,psi,alphtip,alprat,c0,visc
      real M,Rc,Dbarl,Dbarh,deltap,dstrs,dstrp
      integer idirect,itrip,istip,isround
      real spllbl(maxfreq),spltbl(maxfreq),splp(maxfreq),spls(maxfreq),splalph(maxfreq),splblnt(maxfreq),spltip(maxfreq)
      real plblvs(maxfreq),ptblte(maxfreq),pblunt(maxfreq),ptip(maxfreq)
!
      common /broadbandcm/c,L,r,rx,ry,rz,alpstar,u,h,psi,alphtip,alprat,c0,visc, &
                          M,Rc,Dbarl,Dbarh,deltap,dstrs,dstrp, &
                          spllbl,spltbl,splp,spls,splalph,splblnt,spltip, &
                          plblvs,ptblte,pblunt,ptip, &
                          idirect,itrip,istip,isround
!
      endmodule broadband_structures
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine GetBroadband(icase,irtr,mbld,kpnl,tsrc,robs,Mr,ptblte_obs,plblvs_obs,pblunt_obs,ptip_obs)
      use CIIsound_structures
      use broadband_structures
      integer icase,irtr,mbld,kpnl
      real tsrc,robs(3),Mr,ptblte_obs(0:40),plblvs_obs(0:40),pblunt_obs(0:40),ptip_obs(0:40)
!
!   subroutine arguments
!   input
!      icase        integer    case number
!      irtr         integer    rotor number
!      mbld         integer    blade number
!      kpnl         integer    panel number
!      tsrc         real       source time
!      robs(3)      real       vector to observer
!      Mr           real       Mach number in direction of observer
!   output
!      p_obs(0:40)  real       pressure spectrum (TBLTE, LBLVS, BLUNT, TIP)
!
      real M_naca0012(11),cla_naca0012(11),cla_naca0012_rev
      data   M_naca0012/0.0000,0.2000,0.3000,0.4000,0.5000,0.6000,0.7000, 0.7500, 0.8000,0.9000,1.0000/            
      data cla_naca0012/6.0447,6.0447,6.3025,6.5317,7.0760,7.7922,8.9668,10.0268,11.3159,5.7296,5.7296/   
      data cla_naca0012_rev/5.9588/
      real cc(3),tt(3),nn(3)
!
      n=Source(icase)%Rotor(irtr)%Blade(mbld)%mpsi+1
      period=Source(icase)%Rotor(irtr)%Blade(mbld)%period
      call linear_interp(tsrc,n,Source(icase)%Rotor(irtr)%Blade(mbld)%time, &
                                Source(icase)%Rotor(irtr)%Blade(mbld)%alpha(0,kpnl),period,alpha)
      call linear_interp(tsrc,n,Source(icase)%Rotor(irtr)%Blade(mbld)%time, &
                                Source(icase)%Rotor(irtr)%Blade(mbld)%Umag(0,kpnl),period,Umag)
      call linear_interp(tsrc,n,Source(icase)%Rotor(irtr)%Blade(mbld)%time, &
                                Source(icase)%Rotor(irtr)%Blade(mbld)%cl(0,kpnl),period,cl)
      do l=1,3
      call linear_interp(tsrc,n,Source(icase)%Rotor(irtr)%Blade(mbld)%time, &
                                Source(icase)%Rotor(irtr)%Blade(mbld)%echord(0,l,kpnl),period,cc(l))
      call linear_interp(tsrc,n,Source(icase)%Rotor(irtr)%Blade(mbld)%time, &
                                Source(icase)%Rotor(irtr)%Blade(mbld)%etangent(0,l,kpnl),period,tt(l))
      call linear_interp(tsrc,n,Source(icase)%Rotor(irtr)%Blade(mbld)%time, &
                                Source(icase)%Rotor(irtr)%Blade(mbld)%enormal(0,l,kpnl),period,nn(l))
      enddo
!   angle of attack
      if (theInput%MODEL_alpha .le. 1) then
            alphazl=Source(icase)%Rotor(irtr)%alpha_zl(kpnl)
            if (abs(alpha) .le. 90.) then
                 alphastar=abs(alpha-alphazl)
            else
                 alphastar=abs((180.-abs(alpha))-alphazl)
            endif
      else
            if (abs(alpha) .le. 90.) then
                 M=Umag/Source(icase)%csound
                 call linear_interp(M,11,M_naca0012,cla_naca0012,0.,cla)
            else
                 cla=cla_naca0012_rev
            endif
            alphastar=degrad*abs(cl/cla)
      endif
!
      c=Source(icase)%Rotor(irtr)%chord(kpnl)
      L=CIIoutput(icase)%Rotor(irtr)%dra(kpnl)*CIIoutput(icase)%Rotor(irtr)%radius
      r=robs(1)**2+robs(2)**2+robs(3)**2; if (r .ne. 0.) r=sqrt(r)
      rx=robs(1)*cc(1)+robs(2)*cc(2)+robs(3)*cc(3)
      ry=robs(1)*tt(1)+robs(2)*tt(2)+robs(3)*tt(3)
      rz=robs(1)*nn(1)+robs(2)*nn(2)+robs(3)*nn(3)
      alpstar=alphastar
      u=Umag
      h=Source(icase)%Rotor(irtr)%hTE(kpnl)
      psi=Source(icase)%Rotor(irtr)%psiTE(kpnl)
      alphtip=alphastar
      istip=0; if (kpnl .eq. Source(icase)%Rotor(irtr)%npanel) istip=1
      idirect=theInput%MODEL_direct
      itrip=theInput%MODEL_trip
      isround=theInput%isRound(irtr)
      alprat=theInput%ftipalpha(irtr)
      c0=Source(icase)%csound
      visc=Source(icase)%kinvisc
!
      call GetBroadband_Brooks
!
!   scale with density, speed of sound
      scale=prefsq*((Source(icase)%density/rho)*(Source(icase)%csound/cs)**2)**2
      plblvs=plblvs*scale
      ptblte=ptblte*scale
      pblunt=pblunt*scale
      ptip  =ptip  *scale
!
!   source frequency (1:maxfreq) interpolated to observer frequency (0:40); Doppler shift = 1/(1-Mr)
      d=1./(1.-Mr)
      f=2.**.333333333; bigF=alog10(f)
      bigD=alog10(d)
      R=bigD/bigF
      intR=int(abs(R))
      if (bigD .gt. 0.) then; w1=R-intR; w2=1.-w1; else; w2=abs(R)-intR; w1=1.-w2; endif
      do i=0,40
      if (bigD .gt. 0.) then; i1=i-intR-1; i2=i1+1; else; i1=i+intR; i2=i1+1; endif
      if ((i1 .ge. 1) .and. (i1 .le. maxfreq)) then; p1=ptblte(i1); else; p1=0.; endif
      if ((i2 .ge. 1) .and. (i2 .le. maxfreq)) then; p2=ptblte(i2); else; p2=0.; endif
      ptblte_obs(i)=w1*p1+w2*p2
      if ((i1 .ge. 1) .and. (i1 .le. maxfreq)) then; p1=plblvs(i1); else; p1=0.; endif
      if ((i2 .ge. 1) .and. (i2 .le. maxfreq)) then; p2=plblvs(i2); else; p2=0.; endif
      plblvs_obs(i)=w1*p1+w2*p2
      if ((i1 .ge. 1) .and. (i1 .le. maxfreq)) then; p1=pblunt(i1); else; p1=0.; endif
      if ((i2 .ge. 1) .and. (i2 .le. maxfreq)) then; p2=pblunt(i2); else; p2=0.; endif
      pblunt_obs(i)=w1*p1+w2*p2
      if ((i1 .ge. 1) .and. (i1 .le. maxfreq)) then; p1=ptip(i1); else; p1=0.; endif
      if ((i2 .ge. 1) .and. (i2 .le. maxfreq)) then; p2=ptip(i2); else; p2=0.; endif
      ptip_obs(i)=w1*p1+w2*p2
      enddo
!
      return
      end
!***********************************************************************************************************************************
      subroutine GetBroadband_Brooks
      use broadband_structures
!
!     Variable Name       Definition                                                 Units
!     -------------       ----------                                                 -----
!   Input
!     c                   chord length                                               m
!     L                   span length                                                m
!     r                   observer distance                                          m
!     rx,ry,rz            position in chord/span/normal coordinates                  m
!     alpstar             angle of attack                                            deg
!     u                   freestream velocity                                        m/sec
!     h                   segment trailing edge thickness (NACA0012 = 0.00252c)      m
!     psi                 bluntness angle (NACA0012 = 16.)                           deg
!     alphtip             tip angle of attack                                        deg
!     istip               compute tip vortex formation noise (only tip segment)
!   Model
!     idirect             directivity (0 Brooks, 1 ROTONET)
!     itrip               trip boundary layer (0 = untripped; 1 = tripped; 2 = lightly tripped)
!     isround             rounded tip
!     alprat              tip lift curve slope correction (fixed=1.0)
!   Atmosphere
!     c0                  speed of sound                                             m/sec
!     visc                kinematic viscosity                                        m2/sec
!
!   Internal
!     frcen(nfreq)        1/3 octave centered  frequencies (100 to 40000 Hz)         Hz
!     nfreq               number of 1/3 octave frequencies (46)
!
!   Output
!     plblvs(nfreq)       laminar boundary layer, vortex shedding noise              N/m2
!     ptblte(nfreq)       turbulent boundary layer, trailing-edge noise              N/m2
!     pblunt(nfreq)       trailing-edge bluntness, vortex shedding noise             N/m2
!     ptip(nfreq)         tip vortex formation noise                                 N/m2

!   compute Reynolds number and Mach number
      M  = u / c0
      rc = u * c/visc
!   compute directivity function
      call directivity
!   compute boundary layer thicknesses
      call blthick

!   for each blade segment, make a noise prediction according to the mechanisms selected
!   tip noise is predicted for the last segment only
      call LBLVS
      call TBLTE
      call BLUNT
      if (istip .ne. 0) call TIPNOIS

!   mean-square pressure
      do i=1,nfreq
          plblvs(i) = 10.**(spllbl(i)/10.)
          ptblte(i) = 10.**(splp(i)/10.) + 10.**(spls(i)/10.) + 10.**(splalph(i)/10.)
          pblunt(i) = 10.**(splblnt(i)/10.)
          if (istip .ne. 0) then
               ptip(i) = 10.**(spltip(i)/10.)
          else
               ptip(i)= 0.
          endif
      enddo

      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine directivity
      use broadband_structures
      real  Mc

      Mc = 0.8 * M
!   cos(theta)=rx/r, tan(phi)=rz/ry
      cst=0.; if (r .ne. 0.) cst=rx/r
      snt=1.-cst**2; if (snt .ne. 0.) snt=sqrt(snt)
      snhtsq=abs(0.5*(1.-cst))
      snp=1.; den=ry**2+rz**2; if (den .ne. 0.) snp=rz/sqrt(den)

      if (idirect .ne. 0) then
!   high frequency directivity function for the input observer location
      Dbarh = 2.*snhtsq*(snp**2)/((1.+M*cst)*(1.+(M-Mc)*cst)**2) ! eq(B1)
!   low frequency directivity function for the input observer location
      Dbarl = (snt*snp)**2/((1.+M*cst)**4) ! eq(B2)
      if (Dbarl .le. 0.) Dbarl=0.000001
      if (Dbarh .le. 0.) Dbarh=0.000001

      else
!   ROTONET directivity function for the input observer location - eq(42)-eq(49)
      px=(M+cst)/(1.+M*cst)
     !py=(snt*csp)/(1.+M*cst)
      pz=(snt*snp)/(1.+M*cst)

      if (abs(pz) .le. 0.00001) then
        if (px .gt. 0.) then
           Dbarr = 0.
        else
           M1inv=1.; Msq1inv=1.; if (M .ne. 1.) then; M1inv=1./(1.-M); Msq1inv=1./(1.-M**2); endif
           f1=1.; den=1.+Mc*Msq1inv*(M-px); if (den .ne. 0.) f1=1./den**2
           f2=1.; den=1.+Mc*M1inv; if (den .ne. 0.) f2=(1.+Mc*Msq1inv*(M+abs(px)))/den
           f3=2.*abs(px)*M1inv
           Dbarr = (1. / (1.+M*cst)**2) * f1 * f2 * f3
        endif
      else
           M1inv=1.; Msq1inv=1.; if (M .ne. 1.) then; M1inv=1./(1.-M); Msq1inv=1./(1.-M**2); endif
           f1=1.; den=1.+Mc*Msq1inv*(M-px); if (den .ne. 0.) f1=1./den**2
           f2=1.; den=1.+Mc*M1inv; if (den .ne. 0.) f2=(1.+Mc*Msq1inv*(M+sqrt(px**2+(1-M**2)*pz**2)))/den
           f3=1.; den=px+sqrt(px**2+(1.-M**2)*pz**2); if (den .ne. 0.) f3=(M+1.)/den
           Dbarr = ((snt**2 * snp**2) / (1.+M*cst)**4) * f1 * f2 * f3
      endif
      if (Dbarr .le. 0.) Dbarr=0.000001
      Dbarl=Dbarr
      Dbarh=Dbarr
      endif

      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine LBLVS
      use broadband_structures ! original code did not pass itrip

!    --------------------------------
!    ***** variable Definitions *****
!    --------------------------------
!
!     Variable Name       Definition                                                 Units
!     -------------       ----------                                                 -----
!     alpstar             angle of attack                                            degrees
!     c                   chord length                                               meters
!     c0                  speed of sound                                             meters/sec
!     d                   Reynolds number ratio
!     Dbarh               high frequency directivity
!     deltap              pressure side boundary layer thickness                     meters
!     dstrp               pressure side boundary layer displacement thickness        meters
!     dstrs               suction side boundary layer displacement thickness         meters
!     e                   Strouhal number ratio
!     frcen               1/3 octave frequencies                                     Hertz
!     g1                  sound pressure level function                              dB
!     g2                  overall sound pressure level function                      dB
!     g3                  overall sound pressure level function                      dB
!     itrip               flag to trip boundary layer
!     L                   span                                                       meters
!     M                   Mach number
!     nfreq               number of frequencies
!     oaspl               overall sound pressure level                               dB
!     phi                 directivity angle                                          degrees
!     r                   observer distance from segment                             meters
!     rc                  Reynolds number based on chord
!     rc0                 reference Reynolds number
!     scale               geometric scaling term
!     spllam              sound pressure level due to laminar mechanism              dB
!     stprim              Strouhal number based on pressure side boundary layer thickness
!     st1prim             reference Strouhal number
!     stpkprm             peak Strouhal number
!     theta               directivity angle                                          degrees
!     u                   freestream velocity                                        meters/sec
!     visc                kinematic viscosity                                        m2/sec

!   compute reference Strouhal number - eq(55)
      if      (rc .le. 1.3e+05) then; st1prim = 0.18
      else if (rc .le. 4.0e+05) then; st1prim = 0.001756*(rc**0.3931)
      else;                           st1prim = 0.28
      endif

      stpkprm = (10.**(-0.04*alpstar)) * st1prim ! eq(56)

!   compute reference Reynolds number - eq(59)
      if (alpstar .le. 3.0) rc0=10.**(0.215*alpstar+4.978)
      if (alpstar .gt. 3.0) rc0=10.**(0.120*alpstar+5.263)

!   compute peak scaled spectrum level - G2(Rc/Rc0), eq(58)
      d = rc / rc0
      if      (d .le. 0.3237) then; g2 =   77.852*alog10(d) + 15.328
      else if (d .le. 0.5689) then; g2 =   65.188*alog10(d) +  9.125
      else if (d .le. 1.7579) then; g2 = -114.052*(alog10(d))**2
      else if (d .le. 3.0889) then; g2 =  -65.188*alog10(d) +  9.125
      else;                         g2 =  -77.852*alog10(d) + 15.328
      endif

      g3 = 171.04 - 3.03 * alpstar ! G3(astar), eq(60)

      scale = 10. * alog10(deltap*(M**5)*Dbarh*L/(r**2)) ! eq(53)

!   compute scaled sound pressure levels for each Strouhal number
      do  i=1,nfreq
        stprim = frcen(i) * deltap / u ! G1(St/Stpeak), eq(54)
        e = stprim / stpkprm
        if      (e .lt. 0.5974) then; g1 =  39.800*alog10(e) - 11.12
        else if (e .le. 0.8545) then; g1 =  98.409*alog10(e) +  2.00
        else if (e .lt. 1.1700) then; g1 =  -5.076+sqrt(2.484-506.25*((alog10(e))**2))
        else if (e .lt. 1.6740) then; g1 = -98.409*alog10(e) +  2.00
        else;                         g1 = -39.800*alog10(e) - 11.12
        endif
        spllbl(i)  =  g1  + g2  + g3  + scale ! eq(53)
      enddo

      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine TBLTE
      use broadband_structures

!    --------------------------------
!    ***** variable Definitions *****
!    --------------------------------
!
!     Variable Name       Definition                                                 Units
!     -------------       ----------                                                 -----
!     a                   Strouhal number ratio
!     a0                  function used in 'A' calculation
!     a02                 function used in 'A' calculation
!     aa                  'A' spectrum shape evaluated at Strouhal number ratio      dB
!     alpstar             angle of attack                                            degrees
!     amaxa               maximum 'A' curve evaluated at Strouhal number ratio       dB
!     amaxa0              maximum 'A' curve evaluated at a0                          dB
!     amaxa02             maximum 'A' curve evaluated at a02                         dB
!     amaxb               maximum 'A' curve evaluated at b                           dB
!     amina               minimum 'A' curve evaluated at Strouhal number ratio       dB
!     amina0              minimum 'A' curve evaluated at a0                          dB
!     amina02             minimum 'A' curve evaluated at a02                         dB
!     aminb               minimum 'A' curve evaluated at b                           dB
!     ara0                interpolation factor
!     ara02               interpolation factor
!     b                   Strouhal number ratio
!     b0                  function used in 'B' calculation
!     bb                  'B' spectrum shape evaluated at Strouhal number ratio      dB
!     beta                used in 'B' computation
!     beta0               used in 'B' computation
!     bmaxb               maximum 'B' evaluated at b                                 dB
!     bmaxb0              maximum 'B' evaluated at b0                                dB
!     bminb               minimum 'B' evaluated at b                                 dB
!     bminb0              minimum 'B' evaluated at b0                                dB
!     brb0                interpolation factor                                       dB
!     c                   chord length                                               meters
!     c0                  speed of sound                                             meters/sec
!     Dbarh               high frequency directivity
!     Dbarl               low frequency directivity
!     delk1               correction to amplitude function                           dB
!     deltap              pressure side boundary layer thickness                     meters
!     dstrp               pressure side displacement thickness                       meters
!     dstrs               suction side displacement thickness                        meters
!     frcen               array of centered frequencies                              Hertz
!     gamma               used in 'B' computation
!     gamma0              used in 'B' computation
!     itrip               trigger to trip boundary layer
!     k1                  amplitude function                                         dB
!     k2                  amplitude function                                         dB
!     L                   span                                                       meters
!     M                   Mach number
!     nfreq               number of centered frequencies
!     phi                 directivity angle                                          degrees
!     p1                  pressure side pressure                                     nt/m2
!     p2                  suction side pressure                                      nt/m2
!     p4                  pressure from angle of attack contribution                 nt/m2
!     r                   source to observer distance                                meters
!     rc                  Reynolds number based on chord
!     rdstrp              Reynolds number based on pressure side displacement thickness
!     rdstrs              Reynolds number based on suction side displacement thickness
!     splalph             sound pressure level due to angle of attack contribution   dB
!     splp                sound pressure level due to pressure side of airfoil       dB
!     spls                sound pressure level due to suction side of airfoil        dB
!     spltbl              total sound pressure level due to TBLTE mechanism          dB
!     stp                 pressure side Strouhal number
!     sts                 suction side Strouhal number
!     st1                 peak Strouhal number
!     st1prim             peak Strouhal number
!     st2                 peak Strouhal number
!     stpeak              peak Strouhal number
!     theta               directivity angle                                          degrees
!     u                   velocity                                                   meters/sec
!     visc                kinematic viscosity                                        m2/sec

      real k1,k2

!   calculate the Reynolds numbers based on pressure and suction displacement thickness
      rdstrs = dstrs * u / visc
      rdstrp = dstrp * u / visc

!   determine peak Strouhal numbers to be used for 'A' and 'B' curve calculations - eq(32)
      st1 = 0.02 * (M**(-0.6))
      if      (alpstar .lt.  1.33) then; st2 = st1 ! original code uses 1.333
      else if (alpstar .le. 12.50) then; st2 = st1*(10.**(0.0054*((alpstar-1.33)**2)))
      else;                              st2 = 4.72 * st1
      endif
      st1prim = (st1+st2)/2. ! eq(33)

      call a0comp(rc,a0)
      call a0comp(3.*rc,a02)

!   evaluate minimum and maximum 'A' curves at a0
      call aamin(a0,amina0)
      call aamax(a0,amaxa0)
      call aamin(a02,amina02)
      call aamax(a02,amaxa02)

!   compute 'A' max/min ratio - eq(39)
      ara0  = (20. + amina0) / (amina0  - amaxa0)
      ara02 = (20. + amina02)/ (amina02 - amaxa02)

!   compute b0 to be used in 'B' curve calculations - eq(44)
      if      (rc .lt. 9.52e+04) then; b0 = 0.30
      else if (rc .lt. 8.57e+05) then; b0 = (-4.48e-13)*((rc-8.57e+05)**2) + 0.56
      else;                            b0 = 0.56
      endif

!   evaluate minimum and maximum 'B' curves at b0
      call bbmin(b0,bminb0)
      call bbmax(b0,bmaxb0)

!   compute 'B' max/min ratio - eq(45)
      brb0 = (20. + bminb0) / (bminb0 - bmaxb0)

!   for each center frequency, compute an 'A' prediction for the pressure side
      stpeak = st1

      do i=1,nfreq
        stp = frcen(i) * dstrp / u ! eq(31)
        a = alog10(stp / stpeak)
        call aamin(a,amina)
        call aamax(a,amaxa)
        aa = amina + ara0 * (amaxa - amina) ! eq(40)

        if      (rc .lt. 2.47e+05) then; k1 = -4.31 * alog10(rc) + 156.3 ! eq(47)
        else if (rc .lt. 8.00e+05) then; k1 = -9.00 * alog10(rc) + 181.6
        else;                            k1 = 128.5
        endif
        if (rdstrp .le. 5000.) delk1 = -alpstar*(5.29-1.43*alog10(rdstrp)) ! eq(48)
        if (rdstrp .gt. 5000.) delk1 = 0.0

        splp(i)=aa+(k1-3.)+10.*alog10(dstrp*(M**5)*Dbarh*L/(r**2))+delk1 ! aa=A(Stp/St1), eq(25)

      gamma  = 27.094 * M +   3.31 ! eq(50)
      beta   = 72.650 * M +  10.74
      gamma0 = 23.430 * M +  4.651
      beta0  =-34.190 * M - 13.820

      if      (alpstar .le. (gamma0-gamma)) then; k2 = -1000.0 ! eq(49)
      else if (alpstar .le. (gamma0+gamma)) then; k2 = sqrt(beta**2-((beta/gamma)**2)*((alpstar-gamma0)**2))+beta0
      else;                                       k2 = -12.0
      endif
      k2 = k2 + k1

      sts = frcen(i) * dstrs / u ! eq(31)

!   check for 'A' computation for suction side
      if ((alpstar .ge. gamma0) .or. (alpstar .gt. 12.5)) then
!   the 'A' computation is dropped if 'switch' is true
!       spls(i) = 0.0 + 10.*alog10(dstrs*(M**5)*Dbarl*L/(r**2)) original code; RP1218 gives -infinity
!       splp(i) = 0.0 + 10.*alog10(dstrs*(M**5)*Dbarl*L/(r**2)) original code; RP1218 gives -infinity
        spls(i)=-100. ! eq(28)
        splp(i)=-100. ! eq(29)
        b = abs(alog10(sts / st2))
        call aamin(b,aminb)
        call aamax(b,amaxb)
        bb = aminb + ara02 * (amaxb-aminb) ! eq(40)
        splalph(i) = bb+k2+10.*alog10(dstrs*(M**5)*Dbarl*L/(r**2)) ! bb=A'(Sts/St2), eq(30)
      else
        a = alog10(sts / st1prim )
        call aamin(a,amina)
        call aamax(a,amaxa)
        aa = amina + ara0 * (amaxa -amina) ! eq(40)
        spls(i) = aa+k1-3.+10.*alog10(dstrs*(M**5)*Dbarh*L/(r**2)) ! aa=A(Sts/St1), eq(26)
!   'B' curve  computation
        b = abs(alog10(sts / st2))
        call bbmin(b,bminb)
        call bbmax(b,bmaxb)
        bb = bminb + brb0 * (bmaxb-bminb) ! eq(46)
        splalph(i) = bb+k2+10.*alog10(dstrs*(M**5)*Dbarh*L/(r**2)) ! bb=B(Sts/St2), eq(27)
      endif

!   sum all contributions from 'A' and 'B' on both pressure and suction side on a mean-square pressure basis
      if (splp(i) .lt. -100.) splp(i) = -100.
      if (spls(i) .lt. -100.) spls(i) = -100.
      if (splalph(i) .lt. -100.) splalph(i) = -100.

      pp1 = 10.**(splp(i) / 10.)
      pp2 = 10.**(spls(i) / 10.)
      pp4 = 10.**(splalph(i) / 10.)

      spltbl(i) = 10. * alog10(pp1 + pp2 + pp4) ! eq(24)

      enddo

      return
      end
!***********************************************************************************************************************************
      subroutine aamin(a,amina)
!   curve fit corresponding to the a-curve for the minimum allowed Reynolds number. - eq(35)
      x1 = abs(a)
      if      (x1 .le. 0.204) then; amina = sqrt(67.552-886.788*(x1**2))-8.219
      else if (x1 .le. 0.244) then; amina = -32.665*x1+3.981
      else;                         amina = -142.795*(x1**3)+103.656*(x1**2)-57.757*x1+6.006
      endif
      return
      end
!***********************************************************************************************************************************
      subroutine aamax(a,amaxa)
!   curve fit corresponding to the a-curve for the maximum allowed Reynolds number. - eq(36)
      x1 = abs(a)
      if      (x1 .le. 0.130) then; amaxa = sqrt(67.552-886.788*(x1**2))-8.219
      else if (x1 .le. 0.321) then; amaxa = -15.901*x1+1.098
      else;                         amaxa = -4.669*(x1**3)+3.491*(x1**2)-16.699*x1+1.149
      endif
      return
      end
!***********************************************************************************************************************************
      subroutine bbmin(b,bminb)
!   curve fit corresponding to the b-curve for the minimum allowed Reynolds number. - eq(41)
      x1 = abs(b)
      if      (x1 .le. 0.130) then; bminb = sqrt(16.888-886.788*(x1**2))-4.109
      else if (x1 .le. 0.145) then; bminb = -83.607*x1+8.138
      else;                         bminb = -817.81*(x1**3)+355.21*(x1**2)-135.024*x1+10.619
      endif
      return
      end
!***********************************************************************************************************************************
      subroutine bbmax(b,bmaxb)
!   curve fit corresponding to the b-curve for the maximum allowed Reynolds number. - 42)
      x1 = abs(b)
      if      (x1 .le. 0.100) then; bmaxb = sqrt(16.888-886.788*(x1**2))-4.109
      else if (x1 .le. 0.187) then; bmaxb = -31.330*x1+1.854 ! original code -31.313
      else;                         bmaxb = -80.541*(x1**3)+44.174*(x1**2)-39.381*x1+2.344
      endif
      return
      end
!***********************************************************************************************************************************
      subroutine a0comp(rc,a0)
!   determines where the a-curve takes on a value of -20 dB.
      if      (rc .lt. 9.52e+04) then; a0 = 0.57
      else if (rc .lt. 8.57e+05) then; a0 = (-9.57e-13)*((rc-8.57e+05)**2) + 1.13
      else;                            a0 = 1.13
      endif
      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine BLUNT
      use broadband_structures

!    --------------------------------
!    ***** variable Definitions *****
!    --------------------------------
!
!     Variable Name       Definition                                                 Units
!     -------------       ----------                                                 -----
!     alpstar             angle of attack                                            degrees
!     aterm               used to compute peak Strouhal no.
!     c                   chord length                                               meters
!     c0                  speed of sound                                             meters/sec
!     Dbarh               high frequency directivity
!     deltap              pressure side boundary layer thickness                     meters
!     dstarh              average displacement thickness over trailing edge bluntness
!     dstravg             average displacement thickness                             meters
!     dstrp               pressure side displacement thickness                       meters
!     dstrs               suction side displacement thickness                        meters
!     eta                 ratio of Strouhal numbers
!     frcen               array of 1/3 octave centered freq.                         Hertz
!     f4temp              g5 evaluated at minimum hdstarp                            dB
!     g4                  scaled spectrum level                                      dB
!     g5                  spectrum shape function                                    dB
!     g50                 g5 evaluated at psi=0.0                                    dB
!     g514                g5 evaluated at psi=14.0                                   dB
!     h                   trailing edge bluntness                                    meters
!     hdstar              bluntness over average displacement thickness
!     hdstarl             minimum allowed value of hdstar
!     hdstarp             modified value of hdstar
!     itrip               trigger for boundary layer tripping
!     L                   span                                                       meters
!     M                   Mach number
!     nfreq               number of centered frequencies
!     phi                 directivity angle                                          degrees
!     psi                 trailing edge angle                                        degrees
!     r                   source to observer distance                                meters
!     rc                  Reynolds number based on chord
!     scale               scaling factor
!     splblnt             sound pressure levels due to bluntness                     dB
!     stpeak              peak Strouhal number
!     stppp               Strouhal number
!     theta               directivity angle                                          degrees
!     u                   freestream velocity                                        meters/sec
!     visc                kinematic viscosity                                        m2/sec

!   ROTONET: SPL=-infinity for M>0.45, eq(66b)
      if (M .gt. 0.45) then
      do i=1,nfreq; splblnt(i)=-100.; enddo
      return
      endif

!   compute average displacement thickness - eq(73)
      dstravg = (dstrs + dstrp) / 2.
      hdstar = h / dstravg
      dstarh = 1. /hdstar

!   compute peak Strouhal number - eq(72)
      aterm = 0.212 - 0.0045 * psi
      if (hdstar .ge. .2) stpeak = aterm / (1.+0.235*dstarh-0.0132*dstarh**2)
      if (hdstar .lt. .2) stpeak = 0.1 * hdstar + 0.095 - 0.00243 * psi

!   compute scaled spectrum level - eq(74)
      if (hdstar .le. 5.) g4 = 17.5*alog10(hdstar)+157.5-1.114*psi
      if (hdstar .gt. 5.) g4 = 169.7 - 1.114 * psi

      scale = 10. * alog10((M**5.5)*h*Dbarh*L/(r**2)) ! eq(70)

!   for each frequency, compute spectrum shape referenced to 0 dB
      do i=1,nfreq
        stppp = frcen(i) * h / u ! eq(71)
        eta = alog10(stppp/stpeak) ! eq(77)

        hdstarl = hdstar
        call g5comp(hdstarl,eta,g514)

        hdstarp = 6.724*hdstar**2-4.019*hdstar+1.107 ! eq(82)
        call g5comp(hdstarp,eta,g50)

        g5 = g50 + 0.0714 * psi * (g514-g50) ! eq(75)
        if (g5 .gt. 0.) g5 = 0.
        call g5comp(0.25,eta,f4temp)
        if (g5 .gt. f4temp) g5 = f4temp

        splblnt(i) = g4 + g5 + scale ! eq(70)
      enddo

      return
      end
!***********************************************************************************************************************************
      subroutine g5comp(hdstar,eta,g5)

      real M,k,mu

      if      (hdstar .lt. 0.25) then; mu = 0.1211 ! eq(78)
      else if (hdstar .le. 0.62) then; mu = -0.2175*hdstar + 0.1755
      else if (hdstar .lt. 1.15) then; mu = -0.0308*hdstar + 0.0596
      else;                            mu = 0.0242
      endif

      if      (hdstar .le. 0.02) then; M = 0.0 ! eq(79)
      else if (hdstar .lt. 0.50) then; M =  68.724*hdstar - 1.35
      else if (hdstar .le. 0.62) then; M = 308.475*hdstar - 121.23
      else if (hdstar .le. 1.15) then; M = 224.811*hdstar - 69.354
      else if (hdstar .lt. 1.20) then; M = 1583.28*hdstar - 1631.592
      else;                            M = 268.344
      endif
      if (M .lt. 0.0) M = 0.0

      eta0 = -sqrt(((M**2)*(mu**4))/(6.25+(M**2)*(mu**2))) ! eq(80)

      k = 2.5*sqrt(1.-(eta0/mu)**2)-2.5-M*eta0

      if      (eta .lt.    eta0) then; g5 = M * eta + k ! eq(76)
      else if (eta .lt. 0.00000) then; g5 = 2.5*sqrt(1.-(eta/mu)**2)-2.5
      else if (eta .lt. 0.03616) then; g5 = sqrt(1.5625-1194.99*eta**2)-1.25
      else;                            g5 = -155.543 * eta + 4.375
      endif

      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine  TIPNOIS
      use broadband_structures

!    --------------------------------
!    ***** variable Definitions *****
!    --------------------------------
!
!     Variable Name       Definition                                                 Units
!     -------------       ----------                                                 -----

!     alphtip             tip angle of attack                                        degrees
!     alprat              tip lift curve slope correction
!     alptipp             corrected tip angle of attack                              degrees
!     c                   chord length                                               meters
!     c0                  speed of sound                                             meters/sec
!     Dbarh               directivity
!     frcen               centered frequencies                                       Hertz
!     ell                 characteristic length for tip                              meters
!     M                   Mach number
!     Mm                  maximum Mach number
!     nfreq               number of centered frequencies
!     phi                 directivity angle                                          degrees
!     r                   source to observer distance                                meters
!     isround             flag indicating tip is rounded
!     scale               scaling term
!     spltip              sound pressure level due to tip mechanism                  dB
!     stpp                Strouhal number
!     term                scaling term
!     theta               directivity angle                                          degrees
!     u                   freestream velocity                                        meters/sec
!     um                  maximum velocity                                           meters/sec
!     visc                kinematic viscosity                                        m2/sec

      real Mm

!   alprat = (dL/dy)/(dL/dy)_ref at tip, where reference is large AR, untwisted, uniform inflow
      alptipp = alphtip * alprat ! eq(66)

      if      (isround      .ne. 0 ) then; ell =           0.0080*alptipp  * c ! eq(63)
      else if (abs(alptipp) .le. 2.) then; ell = (0.0230 + 0.0169*alptipp) * c ! eq(67)
      else;                                ell = (0.0378 + 0.0095*alptipp) * c ! eq(67)
      endif

      Mm = (1. + .036*alptipp) * M ! eq(64)
      um = Mm * c0 ! eq(65)

      term = (M**2)*(Mm**3)*(ell**2)*Dbarh/(r**2)
      if (term .ne. 0.0) then; scale = 10.*alog10(term) ! eq(61)
      else;                    scale = 0.0
      endif

      do i=1,nfreq
        stpp = frcen(i) * ell / um
        spltip(i) = 126.-30.5*(alog10(stpp)+0.3)**2 + scale ! eq(61)
      enddo

      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine blthick
      use broadband_structures

!    --------------------------------
!    ***** variable Definitions *****
!    --------------------------------
!
!     Variable Name       Definition                                                 Units
!     -------------       ----------                                                 -----
!     alpstar             angle of attack                                            degrees
!     c                   chord length                                               meters
!     c0                  speed of sound                                             meters/sec
!     delta0              boundary layer thickness at zero angle of attack           meters
!     deltap              pressure side boundary layer thickness                     meters
!     dstr0               displacement thickness at zero angle of attack             meters
!     dstrp               pressure side displacement thickness                       meters
!     dstrs               suction side displacement thickness                        meters
!     itrip               trigger for boundary layer tripping
!     rc                  Reynolds number based on chord
!     u                   freestream velocity                                        meters/sec
!     visc                kinematic viscosity                                        m2/sec

!   itrip: 0 = untripped (natural transition); 1 = tripped; 2 = lightly tripped (not documented)

!   compute zero angle of attack boundary layer thickness (meters)
      delta0 = 10.**(1.6569-0.9045*alog10(rc)+0.0596*(alog10(rc))**2)*c ! untripped, eq(5)
      if (itrip .eq. 2) delta0 = 0.6 * delta0 ! lightly tripped
!   heavily tripped, eq(2); not in original code
!     if (itrip .eq. 1) delta0 = 10.**(1.892-0.9045*alog10(rc)+0.0596*(alog10(rc))**2)*c
      if (itrip .eq. 1) delta0 = 1.7183 * delta0

!   compute pressure side boundary layer thickness
      deltap = 10.**(-0.04175*alpstar+0.00106*alpstar**2)*delta0 ! eq(8)

!   compute zero angle of attack displacement thickness
     if ((itrip .eq. 1) .or. (itrip .eq. 2)) then ! tripped, eq(3)
       if (rc .le. .3e+06) dstr0 = 0.0601 * (rc**(-.114))*c
       if (rc .gt. .3e+06) dstr0 = 10.**(3.411-1.5397*alog10(rc)+0.1059*alog10(rc)**2)*c
       if (itrip .eq. 2) dstr0 = dstr0 * 0.6
     else
       dstr0=10.**(3.0187-1.5397*alog10(rc)+0.1059*((alog10(rc))**2))*c ! untripped, eq(6)
     endif

!   pressure side displacement thickness
      dstrp = 10.**(-0.0432*alpstar+0.00113*alpstar**2)*dstr0 ! eq(9)
!     if (itrip .eq. 3) dstrp = dstrp * 1.48 ! not documented

!   suction side displacement thickness
      if (itrip .eq. 1) then
        if      (alpstar .le.  5.0) then; dstrs =        10.**(0.0679*alpstar)*dstr0 ! tripped, eq(12)
        else if (alpstar .le. 12.5) then; dstrs =  0.381*10.**(0.1516*alpstar)*dstr0
        else;                             dstrs = 14.296*10.**(0.0258*alpstar)*dstr0
        endif
      else
        if      (alpstar .le.  7.5) then; dstrs =        10.**(0.0679*alpstar)*dstr0 ! untripped, eq(15)
        else if (alpstar .le. 12.5) then; dstrs = 0.0162*10.**(0.3066*alpstar)*dstr0
        else;                             dstrs =  52.42*10.**(0.0258*alpstar)*dstr0
        endif
      endif

      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
!***********************************************************************************************************************************
! Write Sound
!***********************************************************************************************************************************
      subroutine WriteSound
      use CIIsound_structures
!
      integer nobs(ncasemax),OUT_sound,allprimary
      character tname*100,t*1
      character*10 KIND_obs_str(0:2),KIND_traj_str(0:3),KIND_mic_str(0:3),opsnd_str(0:1)
      data  KIND_obs_str/'trajectory','input (AF)','input (IN)'/
      data KIND_traj_str/'general   ','takeoff   ','flyover   ','approach  '/
      data  KIND_mic_str/'general   ','center    ','starboard ','port      '/
      data     opsnd_str/'no        ','yes       '/
      character*40 lsel,lepnl,loaspl,loaspla,lpnl,lpnlt,lbvispl,loaspl_narrow,loaspl_press,lmsq_bvi,lmsq_narrow,lmsq_press
      character*40 lthirdmsp,lthirddb,lpress,lharm,lnarrowmsp,lnarrowdb,lcenter,ltime,lfreq
      data lsel/'   SEL (dB)                             '/
      data lepnl/'   EPNL (PNdB)                          '/
      data loaspl/'   OASPL (dB)                           '/
      data loaspla/'   OASPLA (dBA)                         '/
      data lpnl/'   PNL (PNdB)                           '/
      data lpnlt/'   PNLT (PNdB)                          '/
      data lbvispl/'   BVI SPL (dB)                         '/
      data loaspl_narrow/'   OASPL from harmonics (dB)            '/
      data loaspl_press/'   OASPL from pressure (dB)             '/
      data lmsq_bvi/'   BVI mean square pressure             '/
      data lmsq_narrow/'   mean square pressure (from harmonics)'/
      data lmsq_press/'   mean square pressure (from time)     '/
      data lthirdmsp/'   third-octave spectrum (msp)          '/
      data lthirddb/'   third-octave spectrum (dB)           '/
      data lpress/'   sound pressure (N/m2)                '/
      data lharm/'   sound pressure (harmonics)           '/
      data lnarrowmsp/'   narrow band spectrum (msp)           '/
      data lnarrowdb/'   narrow band spectrum (dB)            '/
      data lcenter/'      center frequency =                '/
      data ltime/'      time (sec)                        '/
      data lfreq/'      frequency (Hz)                    '/
   99 format ('CIIsound:  Calculation of Rotor Rotational, Thickness, and Broadband Noise from CAMRADII Output'///)
   98 format (/'case',a1,1000(a1,i4))
   97 format ('observer',a1,1000(a1,i4))
   96 format (a40,a1,i10)
   95 format (a40,a1,1000(a1,i10))
   94 format (a40,a1,1000(a1,g14.6))
   93 format (a40,a1,1000(a1,a10))
   92 format (a40,a1,g14.6,1000(a1,g14.6))
   80 format (a40)
   81 format ('rotor',i2)
   82 format ('Rotor',i2,1x,a40)
   83 format ('Rotor',i2,' Blade',i2,1x,a40)
      t=char(9)
!
      if (theInput%OUT_sound .eq. 0) return
!   export calculated sound
      nf=16
      call getenv('filesnd',tname)
      open (nf,file=tname,status='replace',recordsize=16384)
      nf2=16; if (theInput%OUT_soundfiles .ne. 0) nf2=17
!
      OUT_sound=theInput%OUT_sound
      nrtr=CIIoutput(1)%nrotor
      nobs=theInput%Nobserver
      Ntime=theInput%Ntime
      nfreq=(Ntime-1)/2
!
!   header
      write (nf,99)
      write (nf,96) 'number of cases                         ',t,ncase
      write (nf,96) 'Ntime, number of time steps             ',t,theInput%Ntime
      write (nf,96) 'Nspan, number of radial stations        ',t,theInput%Nspan
      write (nf,96) 'Ntraj, number of trajectory steps       ',t,theInput%Ntraj
      write (nf,98) t,(t,nc,nc=1,ncase)
      write (nf,80) 'operating condition                     '
      write (nf,94) '   aircraft speed (knots)               ',t,(t,CIIoutput(nc)%Condition%Vkts,nc=1,ncase)
      write (nf,94) '   aircraft speed (m/sec)               ',t,(t,Source(nc)%speed,nc=1,ncase)
      write (nf,94) '   air density (kg/m3)                  ',t,(t,Source(nc)%density,nc=1,ncase)
      write (nf,94) '   speed of sound (m/sec)               ',t,(t,Source(nc)%csound,nc=1,ncase)
      write (nf,94) '   roll angle (deg)                     ',t,(t,CIIoutput(nc)%Condition%roll,nc=1,ncase)
      write (nf,94) '   pitch angle (deg)                    ',t,(t,CIIoutput(nc)%Condition%pitch,nc=1,ncase)
      write (nf,94) '   yaw angle (deg)                      ',t,(t,CIIoutput(nc)%Condition%yaw,nc=1,ncase)
      write (nf,94) '   climb angle (deg)                    ',t,(t,CIIoutput(nc)%Condition%climb,nc=1,ncase)
      write (nf,94) '   sideslip  angle (deg)                ',t,(t,CIIoutput(nc)%Condition%side,nc=1,ncase)
      write (nf,95) 'number of rotors                        ',t,(t,CIIoutput(nc)%nrotor,nc=1,ncase)
      do i=1,nrtr
      write (nf,81) i
      write (nf,93) '   sound sensor                         ',t,(t,opsnd_str(Source(nc)%Rotor(i)%opsnd),nc=1,ncase)
      write (nf,94) '   blade radius (m)                     ',t,(t,Source(nc)%Rotor(i)%radius,nc=1,ncase)
      write (nf,94) '   solidity                             ',t,(t,CIIoutput(nc)%Rotor(i)%sigma,nc=1,ncase)
      write (nf,95) '   number of blades                     ',t,(t,Source(nc)%Rotor(i)%nblade,nc=1,ncase)
      write (nf,95) '   number of aerodynamic panels         ',t,(t,Source(nc)%Rotor(i)%npanel,nc=1,ncase)
      write (nf,94) '   advance ratio                        ',t,(t,CIIoutput(nc)%Condition%mu(i),nc=1,ncase)
      write (nf,94) '   axial advance ratio                  ',t,(t,CIIoutput(nc)%Condition%muz(i),nc=1,ncase)
      write (nf,94) '   advancing tip Mach number            ',t,(t,CIIoutput(nc)%Condition%Mat(i),nc=1,ncase)
      write (nf,94) '   tip Mach number                      ',t,(t,CIIoutput(nc)%Condition%Mtip(i),nc=1,ncase)
      write (nf,94) '   tip speed (m/sec)                    ',t,(t,Source(nc)%Rotor(i)%Vtip,nc=1,ncase)
      write (nf,94) '   rotational speed (rad/sec)           ',t,(t,CIIoutput(nc)%Condition%Omega(i),nc=1,ncase)
      write (nf,94) '   rotational speed (Hz)                ',t,(t,CIIoutput(nc)%Condition%Omega(i)/(2.*pi),nc=1,ncase)
      write (nf,94) '   period (sec)                         ',t,(t,(2.*pi)/CIIoutput(nc)%Condition%Omega(i),nc=1,ncase)
      write (nf,94) '   CT/s                                 ',t,(t,CIIoutput(nc)%Condition%CTs(i),nc=1,ncase)
      write (nf,94) '   hub force x (N)                      ',t,(t,Source(nc)%Rotor(i)%hub_force(1),nc=1,ncase)
      write (nf,94) '   hub force y (N)                      ',t,(t,Source(nc)%Rotor(i)%hub_force(2),nc=1,ncase)
      write (nf,94) '   hub force z (N)                      ',t,(t,Source(nc)%Rotor(i)%hub_force(3),nc=1,ncase)
      write (nf,94) '   hub position x (m)                   ',t,(t,Source(nc)%Rotor(i)%hub_disp(1),nc=1,ncase)
      write (nf,94) '   hub position y (m)                   ',t,(t,Source(nc)%Rotor(i)%hub_disp(2),nc=1,ncase)
      write (nf,94) '   hub position z (m)                   ',t,(t,Source(nc)%Rotor(i)%hub_disp(3),nc=1,ncase)
      write (nf,95) '   rotational sound, parent rotor       ',t,(t,Sound(nc,1)%primary(i),nc=1,ncase)
      enddo
      write (nf,80) 'observers                               '
      write (nf,95) '   number of observers                  ',t,(t,theInput%nObserver(nc),nc=1,ncase)
      write (nf,93) '   KIND_obs, observer definition        ',t,(t,KIND_obs_str(theInput%KIND_obs(nc)),nc=1,ncase)
      write (nf,80) 'trajectory                              '
      write (nf,93) '   KIND_traj, trajectory definition     ',t,(t,KIND_traj_str(theInput%KIND_traj(nc)),nc=1,ncase)
      write (nf,94) '   xbegin, beginning position (m)       ',t,(t,theInput%xbegin(nc),nc=1,ncase)
      write (nf,94) '   xend, ending position (m)            ',t,(t,theInput%xend(nc),nc=1,ncase)
      write (nf,94) '   xref, reference position (m)         ',t,(t,theInput%xref(nc),nc=1,ncase)
      write (nf,94) '   zref, altitude above ground level (m)',t,(t,theInput%zref(nc),nc=1,ncase)
      write (nf,80) 'microphone                              '
      write (nf,93) '   KIND_mic, microphone definition      ',t,(t,KIND_mic_str(theInput%KIND_mic(nc)),nc=1,ncase)
      write (nf,94) '   xmic x (m)                           ',t,(t,theInput%xmic(1,nc),nc=1,ncase)
      write (nf,94) '   xmic y (m)                           ',t,(t,theInput%xmic(2,nc),nc=1,ncase)
      write (nf,94) '   xmic z (m)                           ',t,(t,theInput%xmic(3,nc),nc=1,ncase)
!
      write (nf,98) t,((t,nc,io=1,nobs(nc)),nc=1,ncase)
      write (nf,97) t,((t,io,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) 'trajectory time (sec)                   ',t,((t,Sound(nc,io)%time_traj,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) 'observer position x (m)                 ',t,((t,Sound(nc,io)%position_obs(1),io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) 'observer position y (m)                 ',t,((t,Sound(nc,io)%position_obs(2),io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) 'observer position z (m)                 ',t,((t,Sound(nc,io)%position_obs(3),io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) 'trajectory position x (m)               ',t,((t,Sound(nc,io)%position_traj(1),io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) 'trajectory position y (m)               ',t,((t,Sound(nc,io)%position_traj(2),io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) 'trajectory position z (m)               ',t,((t,Sound(nc,io)%position_traj(3),io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) 'microphone position x (m)               ',t,((t,Sound(nc,io)%position_mic(1),io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) 'microphone position y (m)               ',t,((t,Sound(nc,io)%position_mic(2),io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) 'microphone position z (m)               ',t,((t,Sound(nc,io)%position_mic(3),io=1,nobs(nc)),nc=1,ncase)
      do i=1,nrtr
      write (nf,81) i
      write (nf,94) '   hub distance (m)                     ',t,((t,Sound(nc,io)%Rotor(i)%distance,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) '   elevation angle (deg)                ',t,((t,Sound(nc,io)%Rotor(i)%elevation,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) '   azimuth angle (deg)                  ',t,((t,Sound(nc,io)%Rotor(i)%azimuth,io=1,nobs(nc)),nc=1,ncase)
      write (nf,95) '   rotational sound, parent rotor       ',t,((t,Sound(nc,io)%primary(i),io=1,nobs(nc)),nc=1,ncase)
      enddo
!
!   trajectory
      write (nf,80) ' '
      write (nf,80) 'Aircraft Sound, Trajectory              '
      write (nf,94) lsel,t,((t,Sound(nc,io)%SEL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) lepnl,t,((t,Sound(nc,io)%EPNL,io=1,nobs(nc)),nc=1,ncase)
!
!   aircraft (sum all rotors)
!   (SoundValueF) Sound(nc,io)%sound_load_total           ! loading
!   (SoundValueF) Sound(nc,io)%sound_thick_total          ! thickness
!   (SoundValueF) Sound(nc,io)%sound_rot_total            ! rotational
!   (SoundValueF) Sound(nc,io)%sound_broadband            ! broadband
!   (SoundValueF) Sound(nc,io)%sound_total                ! total
      write (nf,80) 'Aircraft Sound, Total                   '
      write (nf,94) loaspl,t,((t,Sound(nc,io)%sound_total%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,t,((t,Sound(nc,io)%sound_total%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) lpnl,t,((t,Sound(nc,io)%sound_total%PNL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) lpnlt,t,((t,Sound(nc,io)%sound_total%PNLT,io=1,nobs(nc)),nc=1,ncase)
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%sound_total%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
!     write (nf,80) lthirdmsp
!        do k=0,40;
!        write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%sound_total%spectrum_press(k),io=1,nobs(nc)),nc=1,ncase)
!        enddo
      write (nf,80) 'Aircraft Sound, Total Broadband         '
      write (nf,94) loaspl,t,((t,Sound(nc,io)%sound_broadband%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,t,((t,Sound(nc,io)%sound_broadband%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%sound_broadband%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,80) 'Aircraft Sound, Total Rotational        '
      write (nf,94) loaspl,t,((t,Sound(nc,io)%sound_rot_total%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,t,((t,Sound(nc,io)%sound_rot_total%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%sound_rot_total%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,80) 'Aircraft Sound, Total Loading           '
      write (nf,94) loaspl,t,((t,Sound(nc,io)%sound_load_total%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,t,((t,Sound(nc,io)%sound_load_total%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%sound_load_total%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,80) 'Aircraft Sound, Total Thickness         '
      write (nf,94) loaspl,t,((t,Sound(nc,io)%sound_thick_total%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,t,((t,Sound(nc,io)%sound_thick_total%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%sound_thick_total%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
!
!   rotor (sum all rotors with same rotational speed)
      allprimary=1; do i=1,nrtr; if (Sound(1,1)%primary(i) .ne. 0) allprimary=0; enddo
      if ((nrtr .gt. 1) .and. (allprimary .eq. 0)) then
!   (SoundValue) Sound(nc,io)%sound_loading(i)      ! loading
!   (SoundValue) Sound(nc,io)%sound_thickness(i)    ! thickness
!   (SoundValue) Sound(nc,io)%sound_rotational(i)   ! total (sum loading and thickness)
      do i=1,nrtr; if (Sound(1,1)%primary(i) .eq. 0) then
      write (nf,82) i,'Sound (summed rotors), Rotational       '
      write (nf,94) loaspl,       t,((t,Sound(nc,io)%sound_rotational(i)%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,      t,((t,Sound(nc,io)%sound_rotational(i)%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) lbvispl,      t,((t,Sound(nc,io)%sound_rotational(i)%BVISPL,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_narrow,t,((t,Sound(nc,io)%sound_rotational(i)%OASPL_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_press, t,((t,Sound(nc,io)%sound_rotational(i)%OASPL_press,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_bvi,     t,((t,Sound(nc,io)%sound_rotational(i)%msq_BVI,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_narrow,  t,((t,Sound(nc,io)%sound_rotational(i)%msq_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_press,   t,((t,Sound(nc,io)%sound_rotational(i)%msq_press,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
       write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%sound_rotational(i)%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
!     write (nf,80) lthirdmsp
!        do k=0,40
!        write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%sound_rotational(i)%spectrum_press(k),io=1,nobs(nc)),nc=1,ncase)
!        enddo
      endif
      if (OUT_sound .ge. 3) then
      call WriteSoundFile(nf2,tname,i,0,'sum-rotational-freq')
      write (nf2,80) lnarrowdb; dfreq=1./Sound(1,1)%sound_rotational(i)%period
         do k=1,nfreq; freq=float(k)*dfreq
         write (nf2,92) lfreq,t,freq,((t,Sound(nc,io)%sound_rotational(i)%SPL_narrow(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
!     write (nf2,80) lnarrowmsp; dfreq=1./Sound(1,1)%sound_rotational(i)%period
!        do k=1,nfreq; freq=float(k)*dfreq
!        write (nf2,92) lfreq,t,freq,((t,Sound(nc,io)%sound_rotational(i)%narrow(k),io=1,nobs(nc)),nc=1,ncase)
!        enddo
!     write (nf2,80) lharm; dfreq=1./Sound(1,1)%sound_rotational(i)%period
!        do k=1,Ntime; kfreq=k/2; freq=float(kfreq)*dfreq
!        write (nf2,92) lfreq,t,freq,((t,Sound(nc,io)%sound_rotational(i)%harm(k),io=1,nobs(nc)),nc=1,ncase)
!        enddo
      call WriteSoundFile(nf2,tname,i,0,'sum-rotational-time')
      write (nf2,80) lpress; dtime=Sound(1,1)%sound_rotational(i)%period/float(Ntime)
         do k=0,Ntime; time=float(k)*dtime
         write (nf2,92) ltime,t,time,((t,Sound(nc,io)%sound_rotational(i)%press(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,82) i,'Sound (summed rotors), Loading          '
      write (nf,94) loaspl,       t,((t,Sound(nc,io)%sound_loading(i)%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,      t,((t,Sound(nc,io)%sound_loading(i)%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) lbvispl,      t,((t,Sound(nc,io)%sound_loading(i)%BVISPL,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_narrow,t,((t,Sound(nc,io)%sound_loading(i)%OASPL_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_press, t,((t,Sound(nc,io)%sound_loading(i)%OASPL_press,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_bvi,     t,((t,Sound(nc,io)%sound_loading(i)%msq_BVI,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_narrow,  t,((t,Sound(nc,io)%sound_loading(i)%msq_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_press,   t,((t,Sound(nc,io)%sound_loading(i)%msq_press,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%sound_loading(i)%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      if (OUT_sound .ge. 3) then
      call WriteSoundFile(nf2,tname,i,0,'sum-loading-freq')
      write (nf2,80) lnarrowdb; dfreq=1./Sound(1,1)%sound_loading(i)%period
         do k=1,nfreq; freq=float(k)*dfreq
         write (nf2,92) lfreq,t,freq,((t,Sound(nc,io)%sound_loading(i)%SPL_narrow(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      call WriteSoundFile(nf2,tname,i,0,'sum-loading-time')
      write (nf2,80) lpress; dtime=Sound(1,1)%sound_loading(i)%period/float(Ntime)
         do k=0,Ntime; time=float(k)*dtime
         write (nf2,92) ltime,t,time,((t,Sound(nc,io)%sound_loading(i)%press(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,82) i,'Sound (summed rotors), Thickness        '
      write (nf,94) loaspl,       t,((t,Sound(nc,io)%sound_thickness(i)%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,      t,((t,Sound(nc,io)%sound_thickness(i)%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) lbvispl,      t,((t,Sound(nc,io)%sound_thickness(i)%BVISPL,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_narrow,t,((t,Sound(nc,io)%sound_thickness(i)%OASPL_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_press, t,((t,Sound(nc,io)%sound_thickness(i)%OASPL_press,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_bvi,     t,((t,Sound(nc,io)%sound_thickness(i)%msq_BVI,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_narrow,  t,((t,Sound(nc,io)%sound_thickness(i)%msq_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_press,   t,((t,Sound(nc,io)%sound_thickness(i)%msq_press,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%sound_thickness(i)%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      if (OUT_sound .ge. 3) then
      call WriteSoundFile(nf2,tname,i,0,'sum-thickness-freq')
      write (nf2,80) lnarrowdb; dfreq=1./Sound(1,1)%sound_thickness(i)%period
         do k=1,nfreq; freq=float(k)*dfreq
         write (nf2,92) lfreq,t,freq,((t,Sound(nc,io)%sound_thickness(i)%SPL_narrow(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      call WriteSoundFile(nf2,tname,i,0,'sum-thickness-time')
      write (nf2,80) lpress; dtime=Sound(1,1)%sound_thickness(i)%period/float(Ntime)
         do k=0,Ntime; time=float(k)*dtime
         write (nf2,92) ltime,t,time,((t,Sound(nc,io)%sound_thickness(i)%press(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      endif; enddo
      endif
!
!   rotor
!   (SoundValue) Sound(nc,io)%Rotor(i)%sound_loading       ! loading (sum all blades)
!   (SoundValue) Sound(nc,io)%Rotor(i)%sound_thickness     ! thickness (sum all blades)
!   (SoundValue) Sound(nc,io)%Rotor(i)%sound_rotational    ! total (sum loading and thickness)
!   (SoundValueF) Sound(nc,io)%Rotor(i)%sound_broadband    ! broadband
!   (SoundValueF) Sound(nc,io)%Rotor(i)%sound_total        ! total (sum loading, thickness, and broadband)
      do i=1,nrtr; if (Sound(1,1)%Rotor(i)%opsnd .ne. 0) then
      write (nf,82) i,'Sound, Total                            '
      write (nf,94) loaspl,t,((t,Sound(nc,io)%Rotor(i)%sound_total%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,t,((t,Sound(nc,io)%Rotor(i)%sound_total%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%Rotor(i)%sound_total%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,82) i,'Sound, Broadband                        '
      write (nf,94) loaspl,t,((t,Sound(nc,io)%Rotor(i)%sound_broadband%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,t,((t,Sound(nc,io)%Rotor(i)%sound_broadband%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%Rotor(i)%sound_broadband%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,82) i,'Sound, Rotational                       '
      write (nf,94) loaspl,       t,((t,Sound(nc,io)%Rotor(i)%sound_rotational%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,      t,((t,Sound(nc,io)%Rotor(i)%sound_rotational%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) lbvispl,      t,((t,Sound(nc,io)%Rotor(i)%sound_rotational%BVISPL,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_narrow,t,((t,Sound(nc,io)%Rotor(i)%sound_rotational%OASPL_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_press, t,((t,Sound(nc,io)%Rotor(i)%sound_rotational%OASPL_press,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_bvi,     t,((t,Sound(nc,io)%Rotor(i)%sound_rotational%msq_BVI,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_narrow,  t,((t,Sound(nc,io)%Rotor(i)%sound_rotational%msq_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_press,   t,((t,Sound(nc,io)%Rotor(i)%sound_rotational%msq_press,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%Rotor(i)%sound_rotational%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      if (OUT_sound .ge. 3) then
      call WriteSoundFile(nf2,tname,i,0,'rotational-freq')
      write (nf2,80) lnarrowdb; dfreq=1./Sound(1,1)%Rotor(i)%sound_rotational%period
         do k=1,nfreq; freq=float(k)*dfreq
         write (nf2,92) lfreq,t,freq,((t,Sound(nc,io)%Rotor(i)%sound_rotational%SPL_narrow(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      call WriteSoundFile(nf2,tname,i,0,'rotational-time')
      write (nf2,80) lpress; dtime=Sound(1,1)%Rotor(i)%sound_rotational%period/float(Ntime)
         do k=0,Ntime; time=float(k)*dtime
         write (nf2,92) ltime,t,time,((t,Sound(nc,io)%Rotor(i)%sound_rotational%press(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,82) i,'Sound, Loading                          '
      write (nf,94) loaspl,       t,((t,Sound(nc,io)%Rotor(i)%sound_loading%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,      t,((t,Sound(nc,io)%Rotor(i)%sound_loading%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) lbvispl,      t,((t,Sound(nc,io)%Rotor(i)%sound_loading%BVISPL,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_narrow,t,((t,Sound(nc,io)%Rotor(i)%sound_loading%OASPL_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_press, t,((t,Sound(nc,io)%Rotor(i)%sound_loading%OASPL_press,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_bvi,     t,((t,Sound(nc,io)%Rotor(i)%sound_loading%msq_BVI,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_narrow,  t,((t,Sound(nc,io)%Rotor(i)%sound_loading%msq_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_press,   t,((t,Sound(nc,io)%Rotor(i)%sound_loading%msq_press,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%Rotor(i)%sound_loading%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      if (OUT_sound .ge. 3) then
      call WriteSoundFile(nf2,tname,i,0,'loading-freq')
      write (nf2,80) lnarrowdb; dfreq=1./Sound(1,1)%Rotor(i)%sound_loading%period
         do k=1,nfreq; freq=float(k)*dfreq
         write (nf2,92) lfreq,t,freq,((t,Sound(nc,io)%Rotor(i)%sound_loading%SPL_narrow(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      call WriteSoundFile(nf2,tname,i,0,'loading-time')
      write (nf2,80) lpress; dtime=Sound(1,1)%Rotor(i)%sound_loading%period/float(Ntime)
         do k=0,Ntime; time=float(k)*dtime
         write (nf2,92) ltime,t,time,((t,Sound(nc,io)%Rotor(i)%sound_loading%press(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,82) i,'Sound, Thickness                        '
      write (nf,94) loaspl,       t,((t,Sound(nc,io)%Rotor(i)%sound_thickness%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,      t,((t,Sound(nc,io)%Rotor(i)%sound_thickness%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) lbvispl,      t,((t,Sound(nc,io)%Rotor(i)%sound_thickness%BVISPL,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_narrow,t,((t,Sound(nc,io)%Rotor(i)%sound_thickness%OASPL_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_press, t,((t,Sound(nc,io)%Rotor(i)%sound_thickness%OASPL_press,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_bvi,     t,((t,Sound(nc,io)%Rotor(i)%sound_thickness%msq_BVI,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_narrow,  t,((t,Sound(nc,io)%Rotor(i)%sound_thickness%msq_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_press,   t,((t,Sound(nc,io)%Rotor(i)%sound_thickness%msq_press,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%Rotor(i)%sound_thickness%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      if (OUT_sound .ge. 3) then
      call WriteSoundFile(nf2,tname,i,0,'thickness-freq')
      write (nf2,80) lnarrowdb; dfreq=1./Sound(1,1)%Rotor(i)%sound_thickness%period
         do k=1,nfreq; freq=float(k)*dfreq
         write (nf2,92) lfreq,t,freq,((t,Sound(nc,io)%Rotor(i)%sound_thickness%SPL_narrow(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      call WriteSoundFile(nf2,tname,i,0,'thickness-time')
      write (nf2,80) lpress; dtime=Sound(1,1)%Rotor(i)%sound_thickness%period/float(Ntime)
         do k=0,Ntime; time=float(k)*dtime
         write (nf2,92) ltime,t,time,((t,Sound(nc,io)%Rotor(i)%sound_thickness%press(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      endif; enddo
!
!   blade
      if (OUT_sound .ge. 4) then
!   (SoundValue) Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading       ! loading
!   (SoundValue) Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness     ! thickness
      do i=1,nrtr; if (Sound(1,1)%Rotor(i)%opsnd .ne. 0) then
      nbld=Source(1)%Rotor(i)%nblade
      do m=1,nbld
      write (nf,83) i,m,'Sound, Loading                          '
      write (nf,94) loaspl,       t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,      t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) lbvispl,      t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%BVISPL,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_narrow,t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%OASPL_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_press, t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%OASPL_press,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_bvi,     t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%msq_BVI,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_narrow,  t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%msq_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_press,   t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%msq_press,io=1,nobs(nc)),nc=1,ncase)
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      call WriteSoundFile(nf2,tname,i,m,'loading-freq')
      write (nf2,80) lnarrowdb; dfreq=1./Sound(1,1)%Rotor(i)%Blade(m)%sound_loading%period
         do k=1,nfreq; freq=float(k)*dfreq
         write (nf2,92) lfreq,t,freq,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%SPL_narrow(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      call WriteSoundFile(nf2,tname,i,m,'loading-time')
      write (nf2,80) lpress; dtime=Sound(1,1)%Rotor(i)%Blade(m)%sound_loading%period/float(Ntime)
         do k=0,Ntime; time=float(k)*dtime
         write (nf2,92) ltime,t,time,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_loading%press(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      write (nf,83) i,m,'Sound, Thickness                        '
      write (nf,94) loaspl,       t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,      t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) lbvispl,      t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%BVISPL,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_narrow,t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%OASPL_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) loaspl_press, t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%OASPL_press,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_bvi,     t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%msq_BVI,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_narrow,  t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%msq_narrow,io=1,nobs(nc)),nc=1,ncase)
!     write (nf,94) lmsq_press,   t,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%msq_press,io=1,nobs(nc)),nc=1,ncase)
      write (nf,80) lthirddb
       do k=0,40
       write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
       enddo
      call WriteSoundFile(nf2,tname,i,m,'thickness-freq')
      write (nf2,80) lnarrowdb; dfreq=1./Sound(1,1)%Rotor(i)%Blade(m)%sound_thickness%period
       do k=1,nfreq; freq=float(k)*dfreq
       write (nf2,92) lfreq,t,freq,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%SPL_narrow(k),io=1,nobs(nc)),nc=1,ncase)
       enddo
      call WriteSoundFile(nf2,tname,i,m,'thickness-time')
      write (nf2,80) lpress; dtime=Sound(1,1)%Rotor(i)%Blade(m)%sound_thickness%period/float(Ntime)
       do k=0,Ntime; time=float(k)*dtime
       write (nf2,92) ltime,t,time,((t,Sound(nc,io)%Rotor(i)%Blade(m)%sound_thickness%press(k),io=1,nobs(nc)),nc=1,ncase)
       enddo
      enddo
      endif; enddo
      endif
!
!   rotor broadband components
!   (SoundValueF) Sound(nc,io)%Rotor(i)%sound_bb_TBLTE    ! broadband TBLTE
!   (SoundValueF) Sound(nc,io)%Rotor(i)%sound_bb_LBLVS    ! broadband LBLVS
!   (SoundValueF) Sound(nc,io)%Rotor(i)%sound_bb_BLUNT    ! broadband BLUNT
!   (SoundValueF) Sound(nc,io)%Rotor(i)%sound_bb_TIP      ! broadband TIP
      if (theInput%MODEL_broadband .ge. 2) then
      do i=1,nrtr; if (Sound(1,1)%Rotor(i)%opsnd .ne. 0) then
      write (nf,82) i,'Sound, Broadband TBLTE                  '
      write (nf,94) loaspl,t,((t,Sound(nc,io)%Rotor(i)%sound_bb_TBLTE%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,t,((t,Sound(nc,io)%Rotor(i)%sound_bb_TBLTE%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%Rotor(i)%sound_bb_TBLTE%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,82) i,'Sound, Broadband LBLVS                  '
      write (nf,94) loaspl,t,((t,Sound(nc,io)%Rotor(i)%sound_bb_LBLVS%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,t,((t,Sound(nc,io)%Rotor(i)%sound_bb_LBLVS%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%Rotor(i)%sound_bb_LBLVS%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,82) i,'Sound, Broadband BLUNT                  '
      write (nf,94) loaspl,t,((t,Sound(nc,io)%Rotor(i)%sound_bb_BLUNT%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,t,((t,Sound(nc,io)%Rotor(i)%sound_bb_BLUNT%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%Rotor(i)%sound_bb_BLUNT%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      write (nf,82) i,'Sound, Broadband TIP                    '
      write (nf,94) loaspl,t,((t,Sound(nc,io)%Rotor(i)%sound_bb_TIP%OASPL,io=1,nobs(nc)),nc=1,ncase)
      write (nf,94) loaspla,t,((t,Sound(nc,io)%Rotor(i)%sound_bb_TIP%OASPLA,io=1,nobs(nc)),nc=1,ncase)
      if (OUT_sound .ge. 2) then
      write (nf,80) lthirddb
         do k=0,40
         write (nf,92) lcenter,t,f3center(k),((t,Sound(nc,io)%Rotor(i)%sound_bb_TIP%spectrum(k),io=1,nobs(nc)),nc=1,ncase)
         enddo
      endif
      endif; enddo
      endif
!
      close(nfile)
!
      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine WriteSoundFile(nf2,tname,iRotor,mBlade,title)
      use CIIsound_structures
      integer nf2,iRotor,mBlade
      character(*) tname,title
!
      character fname*512,xname*100,name*100,ext*100,rtr*8,bld*8,shortfname*120,t*1
      integer nobs(ncasemax)
   99 format ('CIIsound:  Calculation of Rotor Rotational, Thickness, and Broadband Noise from CAMRADII Output'/a120)
   98 format (/'case',a1,1000(a1,i4))
   97 format ('observer',a1,1000(a1,i4))
   96 format ('Rotor',i2)
   95 format ('Rotor',i2,' Blade',i2)
   90 format (a5,i3)
      t=char(9)
!
      if (theInput%OUT_soundfiles .eq. 0) return
!
      xname=tname; call removeblank(xname,l); n=index(xname,'.')
      name=xname; ext=' '; if (n .ne. 0) then; name=xname(1:n-1); ext=xname(n+1:l); endif
      if ((iRotor .eq. 0) .and. (mBlade .eq. 0)) then
           fname=name//'-'//title//'.'//ext
      else if (mBlade .eq. 0) then
           write (rtr,90) 'Rotor',iRotor
           fname=name//'-'//title//'-'//rtr//'.'//ext
      else
           write (rtr,90) 'Rotor',iRotor
           write (bld,90) 'Blade',mBlade
           fname=name//'-'//title//'-'//rtr//bld//'.'//ext
      endif
      call removeblank(fname,l)
      shortfname=fname(1:l)
!
      close (nf2,iostat=ierror,err=10)
   10 continue
      open (nf2,file=fname(1:l),status='replace',recordsize=16384)
!
      write (nf2,99) shortfname
      if ((iRotor .gt. 0) .and. (mBlade .eq. 0)) write (nf2,96) iRotor
      if ((iRotor .gt. 0) .and. (mBlade .gt. 0)) write (nf2,95) iRotor,mBlade
      nobs=theInput%Nobserver
      write (nf2,98) t,((t,nc,io=1,nobs(nc)),nc=1,ncase)
      write (nf2,97) t,((t,io,io=1,nobs(nc)),nc=1,ncase)
!
      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
! Write Data Extracted from CAMRADII Output File
!***********************************************************************************************************************************
      subroutine WriteExtractedData
      use CIIsound_structures
!
      character tname*100,tnamen*100,number*10,t*1
      character*9 Ulen,Uare,Uvel,Ufrc,Uden,Udeg,Uang,Utim,Unon,Ukin
  999 format ('Extracted Output')
  998 format (/'Rotor',i2)
  997 format (/'Rotor',i2,' Blade',i2)
  996 format (a14,a1,a9/a1,'azimuth (deg)',a1,'time (sec)',a1,'ra/R=',100(a1,f14.6))
  995 format ('English Units'/)
  994 format ('SI Units'/)
  989 format (a10,a1,a1,i10)
  988 format (a10,a1,a9,100(a1,g14.6))
  987 format (a1,f14.6,a1,f14.6,a1,100(a1,g14.6))
   99 format (i10)
      t=char(9)
!
      if (theInput%OUT_sensor .le. 0) return
!
      Udeg='deg'; Uang='rad/s'; Utim='sec'; Unon=' '
!   English output
      if (theInput%OUT_sensor .eq. 1) then
      Ulen='ft'; Uare='ft^2'; Uvel='ft/sec'; Ufrc='lb'; Uden='slug/ft^3'; Ukin='ft^2/sec'
      flen=1.; fare=1.; fvel=1.; ffrc=1.; fden=1.; fkin=1.
      if (CIIoutput(1)%opunit .ge. 2) then ! SI to English
      flen=1./m_ft; fare=1./m_ft**2; fvel=1./m_ft; ffrc=1./N_lb; fden=m_ft**4/N_lb; fkin=1./m_ft**2
      endif
!   SI output
      else
      Ulen='m'; Uare='m^2'; Uvel='m/sec'; Ufrc='N'; Uden='kg/m^3'; Ukin='m^2/sec'
      flen=1.; fare=1.; fvel=1.; ffrc=1.; fden=1.; fkin=1.
      if (CIIoutput(1)%opunit .le. 1) then ! English to SI
      flen=m_ft; fare=m_ft**2; fvel=m_ft; ffrc=N_lb; fden=N_lb/m_ft**4; fkin=m_ft**2
      endif
      endif
!
      nfile=13
      call getenv('filesen',tname)
!
      do n=1,ncase
      if (ncase .gt. 1) then
      write (number,99) n; l=len_trim(tname); istxt=0; if (l .ge. 4) then; if (tname(l-3:l) .eq. '.txt') istxt=1; endif
      if (istxt .eq. 0) then
           if (n .le. 9) then; tnamen=tname(1:l)//number(10:10); else; tnamen=tname(1:l)//number(9:10); endif
      else
           if (n .le. 9) then; tnamen=tname(1:l-4)//number(10:10)//'.txt'; else; tnamen=tname(1:l-4)//number(9:10)//'.txt'; endif
      endif
      open (nfile,file=tnamen,status='replace',recordsize=2048)
      else
      open (nfile,file=tname,status='replace',recordsize=2048)
      endif
!
      nrtr=CIIoutput(n)%nrotor
      write (nfile,999)
      if (theInput%OUT_sensor .le. 1) write (nfile,995)
      if (theInput%OUT_sensor .ge. 2) write (nfile,994)
      write (nfile,989) 'opunit    ',t,t,CIIoutput(n)%opunit
      write (nfile,989) 'nrotor    ',t,t,CIIoutput(n)%nrotor
      write (nfile,989) 'opfree    ',t,t,CIIoutput(n)%opfree
!
      write (nfile,988) 'density   ',t,Uden,t,CIIoutput(n)%Condition%density*fden
      write (nfile,988) 'csound    ',t,Uvel,t,CIIoutput(n)%Condition%csound*fvel
      write (nfile,988) 'kin visc  ',t,Ukin,t,CIIoutput(n)%Condition%kinvisc*fkin
      write (nfile,988) 'speed     ',t,Uvel,t,CIIoutput(n)%Condition%speed*fvel
      write (nfile,988) 'roll      ',t,Udeg,t,CIIoutput(n)%Condition%roll
      write (nfile,988) 'pitch     ',t,Udeg,t,CIIoutput(n)%Condition%pitch
      write (nfile,988) 'yaw       ',t,Udeg,t,CIIoutput(n)%Condition%yaw
      write (nfile,988) 'climb     ',t,Udeg,t,CIIoutput(n)%Condition%climb
      write (nfile,988) 'side      ',t,Udeg,t,CIIoutput(n)%Condition%side
      write (nfile,988) 'Vtip      ',t,Uvel,(t,CIIoutput(n)%Condition%Vtip(i)*fvel,i=1,nrtr)
      write (nfile,988) 'Omega     ',t,Uang,(t,CIIoutput(n)%Condition%Omega(i),i=1,nrtr)
      write (nfile,988) 'mu        ',t,Unon,(t,CIIoutput(n)%Condition%mu(i),i=1,nrtr)
      write (nfile,988) 'muz       ',t,Unon,(t,CIIoutput(n)%Condition%muz(i),i=1,nrtr)
      write (nfile,988) 'Mtip      ',t,Unon,(t,CIIoutput(n)%Condition%Mtip(i),i=1,nrtr)
      write (nfile,988) 'Mat       ',t,Unon,(t,CIIoutput(n)%Condition%Mat(i),i=1,nrtr)
      write (nfile,988) 'CTs       ',t,Unon,(t,CIIoutput(n)%Condition%CTs(i),i=1,nrtr)
      write (nfile,988) 'CFI       ',t,Unon,(t,CIIoutput(n)%Condition%CFI(1,i),i=1,3)
      write (nfile,988) 'CFI       ',t,Unon,(t,CIIoutput(n)%Condition%CFI(2,i),i=1,3)
      write (nfile,988) 'CFI       ',t,Unon,(t,CIIoutput(n)%Condition%CFI(3,i),i=1,3)
!
      do i=1,nrtr
      write (nfile,998) i
      npnl=CIIoutput(n)%Rotor(i)%npanel
      nprp=CIIoutput(n)%Rotor(i)%nprop
      write (nfile,989) 'opsnd     ',t,     t,CIIoutput(n)%Rotor(i)%opsnd
      write (nfile,989) 'oppart    ',t,     t,CIIoutput(n)%Rotor(i)%oppart
      write (nfile,989) 'oprpm     ',t,     t,CIIoutput(n)%Rotor(i)%oprpm
      write (nfile,988) 'rgear     ',t,Unon,t,CIIoutput(n)%Rotor(i)%rgear
      write (nfile,988) 'azref     ',t,Udeg,t,CIIoutput(n)%Rotor(i)%azref
      write (nfile,989) 'nblade    ',t,     t,CIIoutput(n)%Rotor(i)%nblade
      write (nfile,989) 'rotate    ',t,     t,CIIoutput(n)%Rotor(i)%rotate
      write (nfile,988) 'radius    ',t,Ulen,t,CIIoutput(n)%Rotor(i)%radius*flen
      write (nfile,988) 'sigma     ',t,Unon,t,CIIoutput(n)%Rotor(i)%sigma
      write (nfile,988) 'area_rotor',t,Uare,t,CIIoutput(n)%Rotor(i)%area_rotor*fare
      write (nfile,988) 'area_blade',t,Uare,t,CIIoutput(n)%Rotor(i)%area_blade*fare
      write (nfile,989) 'npanel    ',t,     t,CIIoutput(n)%Rotor(i)%npanel
      write (nfile,988) 'ra        ',t,Unon,(t,CIIoutput(n)%Rotor(i)%ra(j),j=1,npnl)
      write (nfile,988) 'Dra       ',t,Unon,(t,CIIoutput(n)%Rotor(i)%dra(j),j=1,npnl)
      write (nfile,989) 'nprop     ',t,     t,CIIoutput(n)%Rotor(i)%nprop
      write (nfile,988) 'rprop     ',t,Unon,(t,CIIoutput(n)%Rotor(i)%rprop(j),j=1,nprp)
      write (nfile,988) 'chord     ',t,Ulen,(t,CIIoutput(n)%Rotor(i)%chord(j)*flen,j=1,nprp)
      write (nfile,988) 'hub force ',t,Ufrc,(t,CIIoutput(n)%Rotor(i)%hub_force(j)*ffrc,j=1,3)
      write (nfile,988) 'hub disp  ',t,Ulen,(t,CIIoutput(n)%Rotor(i)%hub_disp(j)*flen,j=1,3)
      enddo
!
      do i=1,nrtr
      nbld=CIIoutput(n)%Rotor(i)%nblade
      npnl=CIIoutput(n)%Rotor(i)%npanel
      do m=1,nbld
      write (nfile,997) i,m
      mpsi=CIIoutput(n)%Rotor(i)%Blade(m)%mpsi
      if (mpsi .gt. 0) then
      write (nfile,989) 'mpsi      ',t,     t,CIIoutput(n)%Rotor(i)%Blade(m)%mpsi
      write (nfile,988) 'azref     ',t,Udeg,t,CIIoutput(n)%Rotor(i)%Blade(m)%azref
      write (nfile,988) 'Omega     ',t,Uang,t,CIIoutput(n)%Rotor(i)%Blade(m)%Omega
      write (nfile,988) 'period    ',t,Utim,t,CIIoutput(n)%Rotor(i)%Blade(m)%period
      write (nfile,996) 'force x       ',t,Ufrc,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%force(1,k,j)*ffrc,k=1,npnl)
      enddo
      write (nfile,996) 'force y       ',t,Ufrc,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%force(2,k,j)*ffrc,k=1,npnl)
      enddo
      write (nfile,996) 'force z       ',t,Ufrc,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%force(3,k,j)*ffrc,k=1,npnl)
      enddo
      write (nfile,996) 'displacement x',t,Ulen,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%disp(1,k,j)*flen,k=1,npnl)
      enddo
      write (nfile,996) 'displacement y',t,Ulen,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%disp(2,k,j)*flen,k=1,npnl)
      enddo
      write (nfile,996) 'displacement z',t,Ulen,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%disp(3,k,j)*flen,k=1,npnl)
      enddo
      write (nfile,996) 'velocity x    ',t,Uvel,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%vel(1,k,j)*fvel,k=1,npnl)
      enddo
      write (nfile,996) 'velocity y    ',t,Uvel,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%vel(2,k,j)*fvel,k=1,npnl)
      enddo
      write (nfile,996) 'velocity z    ',t,Uvel,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%vel(3,k,j)*fvel,k=1,npnl)
      enddo
      write (nfile,996) 'disp 3qc x    ',t,Ulen,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%disp3(1,k,j)*flen,k=1,npnl)
      enddo
      write (nfile,996) 'disp 3qc y    ',t,Ulen,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%disp3(2,k,j)*flen,k=1,npnl)
      enddo
      write (nfile,996) 'disp 3qc z    ',t,Ulen,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%disp3(3,k,j)*flen,k=1,npnl)
      enddo
      if (theInput%READ_aero .ne. 0) then
      write (nfile,996) 'section U     ',t,Uvel,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%Umag(k,j)*fvel,k=1,npnl)
      enddo
      write (nfile,996) 'section alpha ',t,Udeg,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%alpha(k,j),k=1,npnl)
      enddo
      write (nfile,996) 'section cl    ',t,Unon,t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=CIIoutput(n)%Rotor(i)%Blade(m)%psi(j); time=CIIoutput(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,CIIoutput(n)%Rotor(i)%Blade(m)%cl(k,j),k=1,npnl)
      enddo
      endif
      endif
      enddo
!
      enddo
!
      close(nfile)
      enddo
!
      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
! Write Acoustic Source Data
!***********************************************************************************************************************************
      subroutine WriteSourceData
      use CIIsound_structures
!
      character tname*100,tnamen*100,number*10,t*1
  999 format ('Acoustic Source Data'/'case number',i4)
  998 format (/'Rotor',i2)
  997 format (/'Rotor',i2,' Blade',i2)
  996 format (a16/a1,'psi',a1,'time',a1,'ra=',100(a1,f14.6))
  995 format (a16/a1,'harm',a1,a1,'ra=',100(a1,f14.6))
  994 format (' ')
  989 format (a16,a1,100(a1,i10))
  988 format (a16,a1,100(a1,g14.6))
  987 format (a1,f14.6,a1,f14.6,a1,100(a1,g14.6))
  986 format (a1,i10,a1,a1,100(a1,g14.6))
   99 format (i10)
      t=char(9)
!
      if (theInput%OUT_source .eq. 0) return
!
      nfile=13
      call getenv('filesrc',tname)
!
      do n=1,ncase
      if (ncase .gt. 1) then
      write (number,99) n; l=len_trim(tname); istxt=0; if (l .ge. 4) then; if (tname(l-3:l) .eq. '.txt') istxt=1; endif
      if (istxt .eq. 0) then
           if (n .le. 9) then; tnamen=tname(1:l)//number(10:10); else; tnamen=tname(1:l)//number(9:10); endif
      else
           if (n .le. 9) then; tnamen=tname(1:l-4)//number(10:10)//'.txt'; else; tnamen=tname(1:l-4)//number(9:10)//'.txt'; endif
      endif
      open (nfile,file=tnamen,status='replace',recordsize=2048)
      else
      open (nfile,file=tname,status='replace',recordsize=2048)
      endif
!
      nrtr=Source(n)%nrotor
      write (nfile,999) n
      write (nfile,989) 'KIND_obs        ',t,t,theInput%KIND_obs(n)
      write (nfile,989) 'KIND_traj       ',t,t,theInput%KIND_traj(n)
      write (nfile,989) 'KIND_mic        ',t,t,theInput%KIND_mic(n)
      write (nfile,989) 'Ntraj           ',t,t,theInput%Ntraj
      write (nfile,988) 'xbegain         ',t,t,theInput%xbegin(n)
      write (nfile,988) 'xend            ',t,t,theInput%xend(n)
      write (nfile,988) 'xref            ',t,t,theInput%xref(n)
      write (nfile,988) 'zref            ',t,t,theInput%zref(n)
      write (nfile,988) 'xmic            ',t,(t,theInput%xmic(imic,n),imic=1,3)
      write (nfile,989) 'nObserver       ',t,t,theInput%nObserver(n)
      if (theInput%KIND_obs(n) .ge. 1) then
      write (nfile,988) 'position_obs1   ',t,(t,theInput%position_obs(1,iobs,n),iobs=1,theInput%nObserver(n))
      write (nfile,988) 'position_obs2   ',t,(t,theInput%position_obs(2,iobs,n),iobs=1,theInput%nObserver(n))
      write (nfile,988) 'position_obs3   ',t,(t,theInput%position_obs(3,iobs,n),iobs=1,theInput%nObserver(n))
      endif
      write (nfile,994)
      write (nfile,989) 'Ntime           ',t,t,theInput%Ntime
      write (nfile,989) 'Nspan           ',t,t,theInput%Nspan
      write (nfile,989) 'interp          ',t,t,theInput%interp
      write (nfile,989) 'NfreqBVI        ',t,t,theInput%NfreqBVI(1),t,theInput%NfreqBVI(2)
      write (nfile,989) 'nprop           ',t,t,theInput%nprop
      write (nfile,989) 'MODEL_broadband ',t,t,theInput%MODEL_broadband
      write (nfile,989) 'OMIT_broadband  ',t,t,theInput%OMIT_broadband
      if (theInput%MODEL_broadband .ge. 2) then
      write (nfile,989) 'READ_aero       ',t,t,theInput%READ_aero
      write (nfile,989) 'OMIT_TBLTE      ',t,t,theInput%OMIT_TBLTE
      write (nfile,989) 'OMIT_LBLVS      ',t,t,theInput%OMIT_LBLVS
      write (nfile,989) 'OMIT_BLUNT      ',t,t,theInput%OMIT_BLUNT
      write (nfile,989) 'OMIT_TIP        ',t,t,theInput%OMIT_TIP
      write (nfile,989) 'MODEL_direct    ',t,t,theInput%MODEL_direct
      write (nfile,989) 'MODEL_alpha     ',t,t,theInput%MODEL_alpha
      write (nfile,989) 'MODEL_trip      ',t,t,theInput%MODEL_trip
      write (nfile,989) 'Nbbave          ',t,t,theInput%Nbbave
      endif
      write (nfile,989) 'OUT_perf        ',t,t,theInput%OUT_perf
      write (nfile,989) 'OUT_sensor      ',t,t,theInput%OUT_sensor
      write (nfile,989) 'OUT_source      ',t,t,theInput%OUT_source
      write (nfile,989) 'OUT_sound       ',t,t,theInput%OUT_sound
      write (nfile,994)
      write (nfile,989) 'nrotor          ',t,t,Source(n)%nrotor
      write (nfile,988) 'density         ',t,t,Source(n)%density
      write (nfile,988) 'csound          ',t,t,Source(n)%csound
      write (nfile,988) 'kinvisc         ',t,t,Source(n)%kinvisc
      write (nfile,988) 'speed           ',t,t,Source(n)%speed
!
      do i=1,nrtr
      write (nfile,998) i
      write (nfile,989) 'opsnd           ',t,t,Source(n)%Rotor(i)%opsnd
      write (nfile,989) 'nblade          ',t,t,Source(n)%Rotor(i)%nblade
      write (nfile,989) 'npanel          ',t,t,Source(n)%Rotor(i)%npanel
      write (nfile,989) 'rotate          ',t,t,Source(n)%Rotor(i)%rotate
      write (nfile,988) 'radius          ',t,t,Source(n)%Rotor(i)%radius
      write (nfile,988) 'Vtip            ',t,t,Source(n)%Rotor(i)%Vtip
      write (nfile,988) 'area_rotor      ',t,t,Source(n)%Rotor(i)%area_rotor
      write (nfile,988) 'area_blade      ',t,t,Source(n)%Rotor(i)%area_blade
      write (nfile,988) 'hub force       ',t,(t,Source(n)%Rotor(i)%hub_force(j),j=1,3)
      write (nfile,988) 'hub disp        ',t,(t,Source(n)%Rotor(i)%hub_disp(j),j=1,3)
      write (nfile,988) 'rroot           ',t,t,Source(n)%Rotor(i)%rroot
      write (nfile,988) 'rtip            ',t,t,Source(n)%Rotor(i)%rtip
      write (nfile,988) 'dr_thick        ',t,t,Source(n)%Rotor(i)%dr_thick
      rroot=Source(n)%Rotor(i)%rroot; dr=Source(n)%Rotor(i)%dr_thick
      write (nfile,988) 'r/R             ',t,(t,rroot+float(ispan-1)*dr+0.5*dr,ispan=1,theInput%Nspan)
      write (nfile,988) 'Axs             ',t,(t,Source(n)%Rotor(i)%Axs(ispan),ispan=1,theInput%Nspan)
      if (theInput%nprop .gt. 0) then
      write (nfile,988) 'rprop           ',t,(t,theInput%rprop(iprop,i),iprop=1,theInput%nprop)
      write (nfile,988) 'thick           ',t,(t,theInput%thick(iprop,i),iprop=1,theInput%nprop)
      endif
      if (theInput%MODEL_broadband .ge. 2) then
      if (theInput%nprop .gt. 0) then
      write (nfile,988) 'alpha_zl        ',t,(t,theInput%alpha_zl(iprop,i),iprop=1,theInput%nprop)
      write (nfile,988) 'hTE/c           ',t,(t,theInput%hTE(iprop,i),iprop=1,theInput%nprop)
      write (nfile,988) 'psiTE           ',t,(t,theInput%psiTE(iprop,i),iprop=1,theInput%nprop)
      endif
      write (nfile,989) 'isRound         ',t,t,theInput%isRound(i)
      write (nfile,988) 'ftipalpha       ',t,t,theInput%ftipalpha(i)
      npnl=Source(n)%Rotor(i)%npanel
      write (nfile,988) 'ra/R            ',t,(t,CIIoutput(n)%Rotor(i)%ra(ipnl),ipnl=1,npnl)
      write (nfile,988) 'chord           ',t,(t,Source(n)%Rotor(i)%chord(ipnl),ipnl=1,npnl)
      write (nfile,988) 'alpha_zl        ',t,(t,Source(n)%Rotor(i)%alpha_zl(ipnl),ipnl=1,npnl)
      write (nfile,988) 'hTE             ',t,(t,Source(n)%Rotor(i)%hTE(ipnl),ipnl=1,npnl)
      write (nfile,988) 'psiTE           ',t,(t,Source(n)%Rotor(i)%psiTE(ipnl),ipnl=1,npnl)
      endif
      enddo
!
      do i=1,nrtr
      nbld=Source(n)%Rotor(i)%nblade
      npnl=Source(n)%Rotor(i)%npanel
      do m=1,nbld
      write (nfile,997) i,m
      mpsi=Source(n)%Rotor(i)%Blade(m)%mpsi
      if (mpsi .gt. 0) then
      write (nfile,989) 'mpsi            ',t,t,Source(n)%Rotor(i)%Blade(m)%mpsi
      write (nfile,988) 'Omega           ',t,t,Source(n)%Rotor(i)%Blade(m)%Omega
      write (nfile,988) 'period          ',t,t,Source(n)%Rotor(i)%Blade(m)%period
      write (nfile,996) 'force x         ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%force(j,1,k),k=1,npnl)
      enddo
      write (nfile,996) 'force y         ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%force(j,2,k),k=1,npnl)
      enddo
      write (nfile,996) 'force z         ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%force(j,3,k),k=1,npnl)
      enddo
      write (nfile,996) 'displacement x  ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%disp(j,1,k),k=1,npnl)
      enddo
      write (nfile,996) 'displacement y  ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%disp(j,2,k),k=1,npnl)
      enddo
      write (nfile,996) 'displacement z  ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%disp(j,3,k),k=1,npnl)
      enddo
      write (nfile,996) 'velocity x      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%vel(j,1,k),k=1,npnl)
      enddo
      write (nfile,996) 'velocity y      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%vel(j,2,k),k=1,npnl)
      enddo
      write (nfile,996) 'velocity z      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%vel(j,3,k),k=1,npnl)
      enddo
      write (nfile,996) 'disp 3qc x      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%disp3(j,1,k),k=1,npnl)
      enddo
      write (nfile,996) 'disp 3qc y      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%disp3(j,2,k),k=1,npnl)
      enddo
      write (nfile,996) 'disp 3qc z      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%disp3(j,3,k),k=1,npnl)
      enddo
      write (nfile,996) 'force_deriv x   ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time_deriv(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%force_deriv(j,1,k),k=1,npnl)
      enddo
      write (nfile,996) 'force_deriv y   ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time_deriv(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%force_deriv(j,2,k),k=1,npnl)
      enddo
      write (nfile,996) 'force_deriv z   ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time_deriv(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%force_deriv(j,3,k),k=1,npnl)
      enddo
      write (nfile,995) 'force_harm x    ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=1,mpsi; write (nfile,986) t,j,t,t,(t,Source(n)%Rotor(i)%Blade(m)%force_harm(j,1,k),k=1,npnl); enddo
      write (nfile,995) 'force_harm y    ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=1,mpsi; write (nfile,986) t,j,t,t,(t,Source(n)%Rotor(i)%Blade(m)%force_harm(j,2,k),k=1,npnl); enddo
      write (nfile,995) 'force_harm z    ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=1,mpsi; write (nfile,986) t,j,t,t,(t,Source(n)%Rotor(i)%Blade(m)%force_harm(j,3,k),k=1,npnl); enddo
      write (nfile,995) 'disp_harm x     ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=1,mpsi; write (nfile,986) t,j,t,t,(t,Source(n)%Rotor(i)%Blade(m)%disp_harm(j,1,k),k=1,npnl); enddo
      write (nfile,995) 'disp_harm y     ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=1,mpsi; write (nfile,986) t,j,t,t,(t,Source(n)%Rotor(i)%Blade(m)%disp_harm(j,2,k),k=1,npnl); enddo
      write (nfile,995) 'disp_harm z     ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=1,mpsi; write (nfile,986) t,j,t,t,(t,Source(n)%Rotor(i)%Blade(m)%disp_harm(j,3,k),k=1,npnl); enddo
      write (nfile,995) 'vel_harm x      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=1,mpsi; write (nfile,986) t,j,t,t,(t,Source(n)%Rotor(i)%Blade(m)%vel_harm(j,1,k),k=1,npnl); enddo
      write (nfile,995) 'vel_harm y      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=1,mpsi; write (nfile,986) t,j,t,t,(t,Source(n)%Rotor(i)%Blade(m)%vel_harm(j,2,k),k=1,npnl); enddo
      write (nfile,995) 'vel_harm z      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=1,mpsi; write (nfile,986) t,j,t,t,(t,Source(n)%Rotor(i)%Blade(m)%vel_harm(j,3,k),k=1,npnl); enddo
      if (theInput%READ_aero .ne. 0) then
      write (nfile,996) 'section U       ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%Umag(j,k),k=1,npnl)
      enddo
      write (nfile,996) 'section alpha   ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%alpha(j,k),k=1,npnl)
      enddo
      write (nfile,996) 'section cl      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%cl(j,k),k=1,npnl)
      enddo
      endif
      if (theInput%MODEL_broadband .ge. 2) then
      write (nfile,996) 'echord x        ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%echord(j,1,k),k=1,npnl)
      enddo
      write (nfile,996) 'echord y        ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%echord(j,2,k),k=1,npnl)
      enddo
      write (nfile,996) 'echord z        ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%echord(j,3,k),k=1,npnl)
      enddo
      write (nfile,996) 'etangent x      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%etangent(j,1,k),k=1,npnl)
      enddo
      write (nfile,996) 'etangent y      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%etangent(j,2,k),k=1,npnl)
      enddo
      write (nfile,996) 'etangent z      ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%etangent(j,3,k),k=1,npnl)
      enddo
      write (nfile,996) 'enormal x       ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%enormal(j,1,k),k=1,npnl)
      enddo
      write (nfile,996) 'enormal y        ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%enormal(j,2,k),k=1,npnl)
      enddo
      write (nfile,996) 'enormal z        ',t,t,t,(t,CIIoutput(n)%Rotor(i)%ra(k),k=1,npnl)
      do j=0,mpsi; psi=Source(n)%Rotor(i)%Blade(m)%psi(j); time=Source(n)%Rotor(i)%Blade(m)%time(j)
           write (nfile,987) t,psi,t,time,t,(t,Source(n)%Rotor(i)%Blade(m)%enormal(j,3,k),k=1,npnl)
      enddo
      endif
      endif
      enddo
!
      enddo
!
      close(nfile)
      enddo
!
      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
! Write Interpolated Source Data
!***********************************************************************************************************************************
      subroutine WriteInterpolatedData
      use CIIsound_structures
!
      real flin(3),dflin(3),fharm(3),dfharm(3),y(3),v(3),dv(3),dum(3),yy(3,6),vv(3,6),dvv(3,6),span(6)
      character tname*100,t*1
  999 format ('Interpolate Loading and Motion')
  997 format (/'Rotor',i2,' Blade',i2)
  996 format ('i=',i2,'  time',a1,'forig',a1,'flin',a1,'fharm',a1,'time_deriv',a1,'dforig',a1,'dflin',a1,'dfharm', &
                         a1,'yorig',a1,'y',a1,'vorig',a1,'v',a1,'dv')
  995 format ('i=',i2,'  time',a1,'flin',a1,'fharm',a1,'dflin',a1,'dfharm',a1,'y',a1,'v',a1,'dv')
  994 format ('i=',i2,'  time',a1,'ypanel',a1,a1,'yspan',a1,a1,a1,a1,'vpanel',a1,a1,'vspan',a1,a1,a1,a1, &
              'dvpanel',a1,a1,'dvspan',a1,a1,a1)
  989 format (g14.6,100(a1,g14.6))
      t=char(9)
!
      if (theInput%OUT_source .eq. 0) return
!
      nfile=16
      call getenv('fileint',tname)
      open (nfile,file=tname,status='replace',recordsize=2048)
!
      icase=1; irtr=1; kpnl=2; nbld=Source(1)%Rotor(1)%nblade
      write (nfile,999)
!
      do ibld=1,2; mbld=1; if (ibld .eq. 2) mbldl=nbld
      write (nfile,997) irtr,mbld
      mpsi=Source(icase)%Rotor(irtr)%Blade(mbld)%mpsi
      do l=1,3
      write (nfile,996) l,t,t,t,t,t,t,t,t,t,t,t,t
      do j=0,mpsi
      theInput%interp=1
      time=Source(icase)%Rotor(irtr)%Blade(mbld)%time(j)
      call GetLoading(icase,irtr,mbld,kpnl,time,flin,dum)
      time_deriv=Source(icase)%Rotor(irtr)%Blade(mbld)%time_deriv(j)
      call GetLoading(icase,irtr,mbld,kpnl,time_deriv,dum,dflin)
      theInput%interp=3
      call GetLoading(icase,irtr,mbld,kpnl,time,fharm,dfharm)
      call GetMotion_panel(icase,irtr,mbld,kpnl,time,y,v,dv)
      forig=Source(icase)%Rotor(irtr)%Blade(mbld)%force(j,l,kpnl)
      dforig=Source(icase)%Rotor(irtr)%Blade(mbld)%force_deriv(j,l,kpnl)
      yorig=Source(icase)%Rotor(irtr)%Blade(mbld)%disp(j,l,kpnl)
      vorig=Source(icase)%Rotor(irtr)%Blade(mbld)%vel(j,l,kpnl)
      write (nfile,989) time,t,forig,t,flin(l),t,fharm(l),t,time_deriv,t,dforig,t,dflin(l),t,dfharm(l), &
                        t,yorig,t,y(l),t,vorig,t,v(l),t,dv(l)
      enddo
      enddo
      enddo
!
      ntime=256
      do ibld=1,2; mbld=1; if (ibld .eq. 2) mbldl=nbld
      write (nfile,997) irtr,mbld
      period=Source(icase)%Rotor(irtr)%Blade(mbld)%period
      dt=period/float(ntime)
      do l=1,3
      write (nfile,995) l,t,t,t,t,t,t,t
      do j=0,ntime
      time=float(j)*dt
      theInput%interp=1
      call GetLoading(icase,irtr,mbld,kpnl,time,flin,dflin)
      theInput%interp=3
      call GetLoading(icase,irtr,mbld,kpnl,time,fharm,dfharm)
      call GetMotion_panel(icase,irtr,mbld,kpnl,time,y,v,dv)
      write (nfile,989) time,t,flin(l),t,fharm(l),t,dflin(l),t,dfharm(l),t,y(l),t,v(l),t,dv(l)
      enddo
      enddo
      enddo
!
      ntime=24
      mbld=nbld
      npnl=CIIoutput(icase)%Rotor(irtr)%npanel
      write (nfile,997) irtr,mbld
      period=Source(icase)%Rotor(irtr)%Blade(mbld)%period
      dt=period/float(ntime)
      rroot=Source(icase)%Rotor(irtr)%rroot
      rtip=Source(icase)%Rotor(irtr)%rtip
      dr=Source(icase)%Rotor(irtr)%dr_thick
      span(1)=CIIoutput(icase)%Rotor(irtr)%ra(1); span(2)=CIIoutput(icase)%Rotor(irtr)%ra(npnl)
      span(3)=rroot+0.5*dr; span(4)=rroot+1.5*dr; span(5)=rtip-1.5*dr; span(6)=rtip-0.5*dr
      do l=1,3
      write (nfile,994) l,t,t,t,t,t,t,t,t,t,t,t,t,t,t,t,t,t,t
      do j=0,ntime
      time=float(j)*dt
      call GetMotion_panel(icase,irtr,mbld,1,   time,yy(1,1),vv(1,1),dvv(1,1))
      call GetMotion_panel(icase,irtr,mbld,npnl,time,yy(1,2),vv(1,2),dvv(1,2))
      call GetMotion_span(icase,irtr,mbld,span(3),time,yy(1,3),vv(1,3),dvv(1,3))
      call GetMotion_span(icase,irtr,mbld,span(4),time,yy(1,4),vv(1,4),dvv(1,4))
      call GetMotion_span(icase,irtr,mbld,span(5),time,yy(1,5),vv(1,5),dvv(1,5))
      call GetMotion_span(icase,irtr,mbld,span(6),time,yy(1,6),vv(1,6),dvv(1,6))
      write (nfile,989) time,(t,yy(l,k),k=1,6),(t,vv(l,k),k=1,6),(t,dvv(l,k),k=1,6)
      enddo
      enddo
!
      close(nfile)
!
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
! Interpolate Loading and Motion
!***********************************************************************************************************************************
      subroutine GetLoading(icase,irtr,mbld,kpnl,t,f,df)
      use CIIsound_structures
      integer icase,irtr,mbld,kpnl
      real t,f(3),df(3)
!
!   subroutine arguments
!   input
!      icase  integer  case number
!      irtr   integer  rotor number
!      mbld   integer  blade number
!      kpnl   integer  panel number
!      t      real     time
!   output
!      f(3)   real     force
!      df(3)  real     d(force)/dt
!
!   linear interpolation
      if (theInput%interp .le. 1) then
      n     =Source(icase)%Rotor(irtr)%Blade(mbld)%mpsi+1
      period=Source(icase)%Rotor(irtr)%Blade(mbld)%period
      do l=1,3
      call linear_interp(t,n,Source(icase)%Rotor(irtr)%Blade(mbld)%time, &
                             Source(icase)%Rotor(irtr)%Blade(mbld)%force(0,l,kpnl),period,f(l))
      call linear_interp(t,n,Source(icase)%Rotor(irtr)%Blade(mbld)%time_deriv, &
                             Source(icase)%Rotor(irtr)%Blade(mbld)%force_deriv(0,l,kpnl),period,df(l))
      enddo
!   harmonic interpolation
      else if (theInput%interp .eq. 2) then
      n     =Source(icase)%Rotor(irtr)%Blade(mbld)%mpsi
      period=Source(icase)%Rotor(irtr)%Blade(mbld)%period
      tzero =Source(icase)%Rotor(irtr)%Blade(mbld)%time(0)
      do l=1,3
      call harm_interp_frc(t,n,Source(icase)%Rotor(irtr)%Blade(mbld)%force_harm(1,l,kpnl),tzero,period,f(l),df(l))
      enddo
!   linear-harmonic interpolation
      else
      n     =Source(icase)%Rotor(irtr)%Blade(mbld)%mpsi
      period=Source(icase)%Rotor(irtr)%Blade(mbld)%period
      tzero =Source(icase)%Rotor(irtr)%Blade(mbld)%time(0)
      do l=1,3
      call linharm_interp_frc(t,n,Source(icase)%Rotor(irtr)%Blade(mbld)%force_harm(1,l,kpnl),tzero,period,f(l),df(l))
      enddo
      endif
!
      return
      end
!***********************************************************************************************************************************
      subroutine GetMotion_panel(icase,irtr,mbld,kpnl,t,y,v,dv)
      use CIIsound_structures
      integer icase,irtr,mbld,kpnl
      real t,y(3),v(3),dv(3)
!
!   subroutine arguments
!   input
!      icase  integer  case number
!      irtr   integer  rotor number
!      mbld   integer  blade number
!      kpnl   integer  panel number
!      t      real     time
!   output
!      y(3)   real     displacement
!      v(3)   real     velocity
!      dv(3)  real     d(velocity)/dt
!
      n     =Source(icase)%Rotor(irtr)%Blade(mbld)%mpsi
      period=Source(icase)%Rotor(irtr)%Blade(mbld)%period
      tzero =Source(icase)%Rotor(irtr)%Blade(mbld)%time(0)
      do l=1,3
      call harm_interp_mtn(t,n,Source(icase)%Rotor(irtr)%Blade(mbld)%disp_harm(1,l,kpnl), &
                               Source(icase)%Rotor(irtr)%Blade(mbld)%vel_harm(1,l,kpnl), &
                               tzero,period,y(l),v(l),dv(l))
      enddo
!
      return
      end
!***********************************************************************************************************************************
      subroutine GetMotion_span(icase,irtr,mbld,r,t,y,v,dv)
      use CIIsound_structures
      integer icase,irtr,mbld
      real r,t,y(3),v(3),dv(3)
!
!   subroutine arguments
!   input
!      icase  integer  case number
!      irtr   integer  rotor number
!      mbld   integer  blade number
!      r      real     radial station
!      t      real     time
!   output
!      y(3)   real     displacement
!      v(3)   real     velocity
!      dv(3)  real     d(velocity)/dt
!
      n     =Source(icase)%Rotor(irtr)%Blade(mbld)%mpsi
      period=Source(icase)%Rotor(irtr)%Blade(mbld)%period
      tzero =Source(icase)%Rotor(irtr)%Blade(mbld)%time(0)
      npnl=CIIoutput(icase)%Rotor(irtr)%npanel
!   search
      if (r .le. CIIoutput(icase)%Rotor(irtr)%ra(1)) then
           k=1
           dx=CIIoutput(icase)%Rotor(irtr)%ra(2)-CIIoutput(icase)%Rotor(irtr)%ra(1)
           w=(CIIoutput(icase)%Rotor(irtr)%ra(2)-r)/dx
      else if (r .ge. CIIoutput(icase)%Rotor(irtr)%ra(npnl)) then
           k=npnl-1
           dx=CIIoutput(icase)%Rotor(irtr)%ra(npnl)-CIIoutput(icase)%Rotor(irtr)%ra(npnl-1)
           w=(CIIoutput(icase)%Rotor(irtr)%ra(npnl)-r)/dx
      else
           k=0
    1      k=k+1
           if (r .gt. CIIoutput(icase)%Rotor(irtr)%ra(k+1)) go to 1
           dx=CIIoutput(icase)%Rotor(irtr)%ra(k+1)-CIIoutput(icase)%Rotor(irtr)%ra(k)
           w=(CIIoutput(icase)%Rotor(irtr)%ra(k+1)-r)/dx
      endif
!   interpolate
      do l=1,3
      call harm_interp_mtn(t,n,Source(icase)%Rotor(irtr)%Blade(mbld)%disp_harm(1,l,k), &
                               Source(icase)%Rotor(irtr)%Blade(mbld)%vel_harm(1,l,k), &
                               tzero,period,yk,vk,dvk)
      call harm_interp_mtn(t,n,Source(icase)%Rotor(irtr)%Blade(mbld)%disp_harm(1,l,k+1), &
                               Source(icase)%Rotor(irtr)%Blade(mbld)%vel_harm(1,l,k+1), &
                               tzero,period,yk1,vk1,dvk1)
      y(l) =w*yk +(1.-w)*yk1
      v(l) =w*vk +(1.-w)*vk1
      dv(l)=w*dvk+(1.-w)*dvk1
      enddo
!
      return
      end
!***********************************************************************************************************************************
      subroutine linharm_interp_frc(t,n,h,tzero,period,fout,dfout)
      real t,h(*),tzero,period,fout,dfout
      integer n
!
!   subroutine arguments
!   input
!      t      real      input t
!      n      integer   number of harmonic values
!      h(n)   real      harmonics of f
!      tzero  real      reference t
!      period real      period
!   output
!      fout   real      interpolated f(t)
!      dfout  real      interpolated df(t)
!
      real, parameter:: pi=3.141592653589793238462643383280
!
      time=t-tzero
      freq=2.*pi/period
      nharm=(n-1)/2
!
      fout=h(1)
      dfout=0.
      do i=1,nharm
      f=float(i)*freq; cs=cos(f*time); sn=sin(f*time)
      arg=float(i)*pi/float(n); factor=(sin(arg)/arg)**2
      fout =fout +  ( h(2*i)*cs+h(2*i+1)*sn)*factor
      dfout=dfout+f*(-h(2*i)*sn+h(2*i+1)*cs)*factor
      enddo
!
      return
      end
!***********************************************************************************************************************************
      subroutine harm_interp_frc(t,n,h,tzero,period,fout,dfout)
      real t,h(*),tzero,period,fout,dfout
      integer n
!
!   subroutine arguments
!   input
!      t      real      input t
!      n      integer   number of harmonic values
!      h(n)   real      harmonics of f
!      tzero  real      reference t
!      period real      period
!   output
!      fout   real      interpolated f(t)
!      dfout  real      interpolated df(t)
!
      real, parameter:: pi=3.141592653589793238462643383280
!
      time=t-tzero
      freq=2.*pi/period
      nharm=(n-1)/2
!
      fout=h(1)
      dfout=0.
      do i=1,nharm
      f=float(i)*freq; cs=cos(f*time); sn=sin(f*time)
      fout =fout +    h(2*i)*cs+h(2*i+1)*sn
      dfout=dfout+f*(-h(2*i)*sn+h(2*i+1)*cs)
      enddo
!
      return
      end
!***********************************************************************************************************************************
      subroutine harm_interp_mtn(t,n,h,dh,tzero,period,xout,vout,dvout)
      real t,h(*),dh(*),tzero,period,xout,vout,dvout
      integer n
!
!   subroutine arguments
!   input
!      t      real      input t
!      n      integer   number of harmonic values
!      h(n)   real      harmonics of x
!      dh(n)  real      harmonics of v
!      tzero  real      reference t
!      period real      period
!   output
!      xout   real      interpolated x(t)
!      vout   real      interpolated v(t)
!      dvout  real      interpolated dv(t)
!
      real, parameter:: pi=3.141592653589793238462643383280
!
      time=t-tzero
      freq=2.*pi/period
      nharm=(n-1)/2
!
      xout=h(1)
      vout=dh(1)
      dvout=0.
      do i=1,nharm
      f=float(i)*freq; cs=cos(f*time); sn=sin(f*time)
      xout =xout +     h(2*i)*cs+ h(2*i+1)*sn
      vout =vout +    dh(2*i)*cs+dh(2*i+1)*sn
      dvout=dvout+f*(-dh(2*i)*sn+dh(2*i+1)*cs)
      enddo
!
      return
      end
!***********************************************************************************************************************************
      subroutine linear_interp(xin,n,x,z,period,zout)
      real xin,x(*),z(*),period,zout
      integer n
!
!   subroutine arguments
!   input
!      xin    real      input x
!      n      integer   number of table values
!      x(n)   real      array of independent variables x
!      z(n)   real      array of dependent variables z
!      period real      period (0. not periodic)
!   output
!      zout   real      interpolated z(xin)
!
!   no array
!     if (n .le. 0) then
!           zout=0.
!     else if (n .eq. 1) then
!           zout=z(1)
!     else
      xin2=xin
!   periodic
      if (period .ne. 0.) then
   10 if (xin2 .gt. x(n)) then
           xin2=xin2-period
           go to 10
      endif
   11 if (xin2 .lt. x(1)) then
           xin2=xin2+period
           go to 11
      endif
      endif
!   search
      if (xin2 .le. x(1)) then
           j=1
           w=1.
      else if (xin2 .ge. x(n)) then
           j=n-1
           w=0.
      else
           j=0
    1      j=j+1
           if (xin2 .gt. x(j+1)) go to 1
           dx=x(j+1)-x(j)
           if (dx .ne. 0.) dx=1./dx
           w=(x(j+1)-xin2)*dx
      endif
!   interpolate
      zout=w*z(j)+(1.-w)*z(j+1)
!     endif
!
      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine removeblank(c,last)
      character(*) c
!
! remove blank from c; left justify; last = new length
!
      l=len(c)
      n0=0
      do 1 n=1,l
      if (c(n:n) .ne. ' ') then
           n0=n0+1
           c(n0:n0)=c(n:n)
      endif
    1 continue
      last=n0
      if (n0 .gt. 0) c=c(1:n0)
      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
! Sound Analysis Operations
!***********************************************************************************************************************************
      subroutine GetThirdOctaveBand(freq,kband)
      use CIIsound_structures
      real freq
      integer kband
!
!   subroutine arguments
!   input
!      frequency   real      freq (Hz)
!   output
!      kband       integer   1/3 octave band
!
      if (freq .le. f3upper(0)) then
           kband=0
      else if (freq .ge. f3lower(40)) then
           kband=40
      else
           kband=0
   1       kband=kband+1
           if ((f3lower(kband) .gt. freq) .or. (freq .gt. f3upper(kband))) go to 1
      endif
!
      return
      end
!***********************************************************************************************************************************
      subroutine GetFreq2Band(Ntime,period,freq2band)
      use CIIsound_structures
      integer Ntime,freq2band(*)
      real period
!
!   subroutine arguments
!   input
!      Ntime         integer   number of time steps
!      period        real      period (freq=n/period Hz)
!   output
!      freq2band(*)  integer   third-octave band number from frequency
!
      nfreq=(Ntime-1)/2
      if ((nfreq .le. 0) .or. (period .le. 0.)) return
      dfreq=1./period
!
      kband=0
      do i=1,nfreq
      freq=float(i)*dfreq
      if (freq .le. f3upper(0)) then
           freq2band(i)=0
      else if (freq .ge. f3lower(40)) then
           freq2band(i)=40
      else
   1       continue
           if ((f3lower(kband) .gt. freq) .or. (freq .gt. f3upper(kband))) then
                kband=kband+1
                go to 1
           endif
           freq2band(i)=kband
      endif
      enddo
!
      return
      end
!***********************************************************************************************************************************
      subroutine InitSoundValue(Ntime,Omega,period,nblade,s)
      use CIIsound_structures
      type (SoundValue) s
      integer Ntime,nblade
      real Omega,period
!
!   subroutine arguments
!   input
!      Ntime    integer    number of time steps
!      Omega    real       rotational speed (rad/sec)
!      period   real       period=2*pi/Omega (sec)
!      nblade   real       number of blades
!
      s%Ntime=Ntime
      s%Omega=Omega
      s%period=period
      s%nblade=nblade
      s%press=0.
!
      return
      end
!***********************************************************************************************************************************
      subroutine CompleteSoundValue(s,freq2band)
      use CIIsound_structures
      type (SoundValue) s
      integer freq2band(*)
!
      real work(ntimemax),harm(ntimemax)
!
      Ntime=s%Ntime
      nblade=s%nblade
      nfreq=(Ntime-1)/2
      s%OASPL_press=-101.
      s%SPL_narrow=-101.
      s%OASPL_narrow=-101.
      s%BVISPL=-101.
      s%spectrum=-101.
      s%OASPL=-101.
      s%OASPLA=-101.
!
      pmean=0.; psqmean=0.
      do j=1,Ntime
      pmean=pmean+s%press(j); psqmean=psqmean+s%press(j)**2
      enddo
      pmean=pmean/float(Ntime); psqmean=psqmean/float(Ntime)-pmean**2
      s%msq_press=psqmean
      if (psqmean .gt. 0.) s%OASPL_press=10.*log10(psqmean/prefsq)
!
      do j=1,Ntime; harm(j)=s%press(j-1); enddo
      call rfft1f (Ntime,1,harm,ntimemax,fftwork,lfftwork,work,ntimemax,ierror )
      if (ierror .ne. 0) write (6,*) '  sound: forward fft error',ierror
      do j=1,Ntime; s%harm(j)=harm(j); enddo
      psqmean=0.; psqbvi=0.
      s%spectrum_press=0.
      do i=1,nfreq
      psq=0.5*(harm(2*i)**2+harm(2*i+1)**2); psqmean=psqmean+psq
      if ((i .ge. theInput%NfreqBVI(1)*nblade) .and. (i .le. theInput%NfreqBVI(2)*nblade)) psqbvi=psqbvi+psq
      s%narrow(i)=psq
      if (psq .gt. 0.) s%SPL_narrow(i)=10.*log10(psq/prefsq)
      s%spectrum_press(freq2band(i))=s%spectrum_press(freq2band(i))+psq
      enddo
      s%msq_narrow=psqmean
      s%msq_BVI=psqbvi
      if (psqmean .gt. 0.) s%OASPL_narrow=10.*log10(psqmean/prefsq)
      if (psqbvi  .gt. 0.) s%BVISPL      =10.*log10(psqbvi/prefsq)
!
      OASPL=0.; OASPLA=0.
      do i=0,40
      psq=s%spectrum_press(i)
      spec=-101.; specA=-101.; if (psq .gt. 0.) then; spec=10.*log10(psq/prefsq); specA=spec+dBAcorr(i); endif
      OASPL=OASPL+psq/prefsq; OASPLA=OASPLA+10.**(0.1*specA)
      s%spectrum(i)=spec
      enddo
      if (OASPL .gt. 0.) s%OASPL=10.*log10(OASPL); if (OASPLA .gt. 0.) s%OASPLA=10.*log10(OASPLA)
!
      return
      end
!***********************************************************************************************************************************
      subroutine CompleteSoundValueF(s,option)
      use CIIsound_structures
      type (SoundValueF) s
      integer option
!
!   subroutine arguments
!   input
!      option   integer   input (1 spectrum_press, 2 spectrum)
!
      real spectrum(24)
!
!   pressure
      if (option .le. 1) then
      OASPL=0.; OASPLA=0.
      do i=0,40
      psq=s%spectrum_press(i)
      spec=-101.; if (psq .gt. 0.) spec=10.*log10(psq/prefsq); specA=spec+dBAcorr(i)
      OASPL=OASPL+psq/prefsq; OASPLA=OASPLA+10.**(0.1*specA)
      s%spectrum(i)=spec
      enddo
      if (OASPL .gt. 0.) s%OASPL=10.*log10(OASPL); if (OASPLA .gt. 0.) s%OASPLA=10.*log10(OASPLA)
!
!   spectrum
      else
      OASPL=0.; OASPLA=0.
      do i=0,40
      spec=s%spectrum(i); specA=spec+dBAcorr(i)
      psq=10.**(0.1*spec); psqA=10.**(0.1*specA)
      OASPL=OASPL+psq; OASPLA=OASPLA+psqA
      s%spectrum_press(i)=psq*prefsq
      enddo
      if (OASPL .gt. 0.) s%OASPL=10.*log10(OASPL); if (OASPLA .gt. 0.) s%OASPLA=10.*log10(OASPLA)
      endif
!
!   PNL and PNLT (from third-octave spectrum, 50-10000 Hz)
      do i=1,24; spectrum(i)=s%spectrum(i+16); enddo
      call GetPNL(spectrum,PNL,PNLT)
      s%PNL=PNL
      s%PNLT=PNLT
!
      return
      end
!***********************************************************************************************************************************
      subroutine GetPNL(s,PNL,PNLT)
      real s(24),PNL,PNLT
!
      real N(24),Nmax,Ntot
      real slope(24),dslope(24),sp(24),slopep(25),slopea(24),spp(24),f(24),C(24)
      integer slope_circled(24),spl_circled(24)
      real SPLa(24),SPLb(24),SPLc(24),SPLd(24),SPLe(24),Mb(24),Mc(24),Md(24),Me(24)
      data SPLa/91.0,85.9,87.3,79.9,79.8,76.0,73.0,74.9,94.6,13*1000.0,44.3,50.7/
      data SPLb/64.,60.,56.,53.,51.,48.,46.,44.,42.,40.,40.,40., &
                40.,40.,38.,34.,32.,30.,29.,29.,30.,31.,37.,41./
      data SPLc/52.,51.,49.,47.,46.,45.,43.,42.,41.,40.,40.,40., &
                40.,40.,38.,34.,32.,30.,29.,29.,30.,31.,34.,37./
      data SPLd/49.,44.,39.,34.,30.,27.,24.,21.,18.,16.,16.,16., &
                16.,16.,15.,12., 9., 5., 4., 5., 6.,10.,17.,21./
      data SPLe/55.,51.,46.,42.,39.,36.,33.,30.,27.,25.,25.,25., &
                25.,25.,23.,21.,18.,15.,14.,14.,15.,17.,23.,29./
      data Mb/0.043478,0.040570,0.036831,0.036831,0.035336,0.033333,0.033333,0.032051, &
              0.030675,0.030103,0.030103,0.030103,0.030103,0.030103,0.030103,0.029960, &
              0.029960,0.029960,0.029960,0.029960,0.029960,0.029960,0.042285,0.042285/
      data Mc/0.030103,0.030103,0.030103,0.030103,0.030103,0.030103,0.030103,0.030103, &
              0.030103,1.000000,1.000000,1.000000,1.000000,1.000000,1.000000,1.000000, &
              1.000000,1.000000,1.000000,1.000000,1.000000,1.000000,0.029960,0.029960/
      data Md/0.079520,0.068160,0.068160,0.059640,0.053013,0.053013,0.053013,0.053013, &
              0.053013,0.053013,0.053013,0.053013,0.053013,0.053013,0.059640,0.053013, &
              0.053013,0.047712,0.047712,0.053013,0.053013,0.068160,0.079520,0.059640/
      data Me/0.058098,0.058098,0.052288,0.047534,0.043573,0.043573,0.040221,0.037349, &
              0.034859,0.034859,0.034859,0.034859,0.034859,0.034859,0.034859,0.040221, &
              0.037349,0.034859,0.034859,0.034859,0.034859,0.037349,0.037349,0.043573/
!
!   perceived noisiness
      Nmax=0.; Ntot=0.
      do i=1,24
           SPL=s(i)
           if ((i .le. 9) .and. (SPL .ge. SPLa(i))) then;       N(i)=10.**(Mc(i)*(SPL-SPLc(i)))
           else if ((i .ge. 23) .and. (SPL .ge. SPLa(i))) then; N(i)=10.**(Mc(i)*(SPL-SPLc(i)))
           else if (SPL .ge. SPLb(i)) then;                     N(i)=10.**(Mb(i)*(SPL-SPLb(i)))
           else if (SPL .ge. SPLe(i)) then;                N(i)=0.3*(10.**(Me(i)*(SPL-SPLe(i))))
           else if (SPL .ge. SPLd(i)) then;                N(i)=0.1*(10.**(Md(i)*(SPL-SPLd(i))))
           else; N(i)=0.
           endif
           Nmax=max(Nmax,N(i))
           Ntot=Ntot+N(i)
      enddo
      Ntot=0.85*Nmax+0.15*Ntot
      PNL=-101.; if (Ntot .gt. 0.) PNL=40.+(10.*log10(Ntot))/(log10(2.))
!
!   tone correction
      slope=0.; dslope=0.; slope_circled=0
      do i=4,24
           slope(i)=s(i)-s(i-1)
           dslope(i)=slope(i)-slope(i-1)
           if (abs(dslope(i)) .gt. 5.) slope_circled(i)=1
      enddo
      spl_circled=0
      do i=4,24
           if (slope_circled(i) .ne. 0) then
               if ((slope(i) .gt. 0.) .and. (slope(i) .gt. slope(i-1))) spl_circled(i)=1
               if ((slope(i) .le. 0.) .and. (slope(i-1) .gt. 0.)) spl_circled(i-1)=1
           endif
      enddo
      sp=s
      do i=3,23
            if (spl_circled(i) .ne. 0) sp(i)=0.5*(s(i-1)+s(i+1))
      enddo
      if (spl_circled(24) .ne. 0) sp(24)=s(23)+slope(23)
      slopep=0.
      do i=4,24
            slopep(i)=sp(i)-sp(i-1)
      enddo
      slopep(3)=slopep(4); slopep(25)=slopep(24)
      slopea=0.
      do i=3,23
            slopea(i)=(slopep(i)+slopep(i+1)+slopep(i+2))/3.
      enddo
      spp=s
      do i=4,24
            spp(i)=spp(i-1)+slopea(i-1)
      enddo
      do i=1,24
            f(i)=s(i)-spp(i)
            if (f(i) .lt. 1.5) f(i)=0.
      enddo
      C=0.
      do i=3,24
        if ((i .le. 10) .or. (i .ge. 22)) then
            if (f(i) .lt. 1.5) then; C(i)=0.
            else if (f(i) .lt. 3.) then; C(i)=f(i)/3.-0.5
            else if (f(i) .lt. 20.) then; C(i)=f(i)/6.
            else; C(i)=10./3.
            endif
        else
            if (f(i) .lt. 1.5) then; C(i)=0.
            else if (f(i) .lt. 3.) then; C(i)=2.*f(i)/3.-1.
            else if (f(i) .lt. 20.) then; C(i)=f(i)/3.
            else; C(i)=20./3.
            endif
        endif
      enddo
      Cmax=0.
      do i=3,24
           Cmax=max(Cmax,C(i))
      enddo
      PNLT=PNL+Cmax
!
!     write (8,98) 'band','SPL','circled','slope','circled','dslope','SPL-p','slope-p','slope-a','SPL-pp','F','C'
!     do i=1,24
!     write (8,99) i,s(i),spl_circled(i),slope(i),slope_circled(i),dslope(i),sp(i),slopep(i),slopea(i),spp(i),f(i),c(i)
!     enddo
!  98 format (12a10)
!  99 format (i10,f10.2,i10,f10.2,i10,3f10.2,4f10.4)
!
      return
      end
!***********************************************************************************************************************************
      subroutine GetSEL(nc)
      use CIIsound_structures
      integer nc
!
!   subroutine arguments
!   input
!      nc    integer    case number
!
      real psq(nobsmax),time(nobsmax)
!
      if (theInput%KIND_obs(nc) .gt. 0) return
      Tref=1.
      Ntraj=theInput%Nobserver(nc)
!   pressure time history
      do i=1,Ntraj
           psq(i)=10.**(0.1*Sound(nc,i)%sound_total%OASPLA)
           time(i)=Sound(nc,i)%time_traj
      enddo
!   integrate
      call IntegrateSPL(Ntraj,time,psq,psqint)
!   SEL
      psqint=psqint/Tref
      Sound(nc,1)%SEL=10.*log10(psqint)
!
      return
      end
!***********************************************************************************************************************************
      subroutine GetEPNL(nc)
      use CIIsound_structures
      integer nc
!
!   subroutine arguments
!   input
!      nc    integer    case number
!
      real psq(nobsmax),time(nobsmax)
!
      if (theInput%KIND_obs(nc) .gt. 0) return
      Tref=10.
      Ntraj=theInput%Nobserver(nc)
!   pressure time history
      do i=1,Ntraj
           psq(i)=10.**(0.1*Sound(nc,i)%sound_total%PNLT)
           time(i)=Sound(nc,i)%time_traj
      enddo
!   integrate
      call IntegrateSPL(Ntraj,time,psq,psqint)
!   EPNL
      psqint=psqint/Tref
      Sound(nc,1)%EPNL=10.*log10(psqint)
!
      return
      end
!***********************************************************************************************************************************
!***********************************************************************************************************************************
      subroutine IntegrateSPL(N,time,psq,psqint)
      integer N
      real time(*),psq(*),psqint
!
!   peak
      ipeak=1; psqpeak=psq(1)
      do i=2,N
           if (psq(i) .gt. psqpeak) then; ipeak=i; psqpeak=psq(i); endif
      enddo
      pmin=0.1*psqpeak
!   first and last points above peak-10dB
      ifirst=0; ilast=0
      do i=1,N
           if (psq(i) .gt. pmin) then; ilast=i; if (ifirst .eq. 0) ifirst=i; endif
      enddo
!
!   integrate
      psqint=0.
!   ifirst-1 to ifirst
      if (ifirst .gt. 1) then
      t1=time(ifirst-1); p1=psq(ifirst-1); t2=time(ifirst); p2=psq(ifirst); dpinv=p2-p1; if (dpinv .ne. 0.) dpinv=1./dpinv
      d=0.5*((p2**2-pmin**2)*dpinv)*abs(t2-t1)
      psqint=psqint+d
      endif
!   ilast to ilast+1
      if (ilast .lt. N) then
      t1=time(ilast); p1=psq(ilast); t2=time(ilast+1); p2=psq(ilast+1); dpinv=p1-p2; if (dpinv .ne. 0.) dpinv=1./dpinv
      d=0.5*((p1**2-pmin**2)*dpinv)*abs(t2-t1)
      psqint=psqint+d
      endif
!   ifirst to ilast
      if (ilast .gt. ifirst) then
      do i=ifirst,ilast-1
      t1=time(i); p1=psq(i); t2=time(i+1); p2=psq(i+1)
      d=0.5*(p1+p2)*abs(t2-t1)
      psqint=psqint+d
      enddo
      endif
!
      return
      end
!***********************************************************************************************************************************
!###############################################################################
!###############################################################################
!  FFTPACK 5.1 (Fortran 90)
!
!   time series: r(i), i=1,n, t=(i-1)*dt
!   harmonics (forward transform): r=mean,cos1,sin1,cos2,sin2,...,diff
!
!   initialize
!     call rfft1i ( n, wsave, lensav, ier )
!   forward transform
!     call rfft1f ( n, inc, r, lenr, wsave, lensav, work, lenwrk, ier )
!   parameters
!   input
!     N              integer      length of the sequence to be transformed
!                                 transform  most efficient when N is  product of small primes
!     LENSAV         integer      dimension of the WSAVE array; at least N + INT(LOG(REAL(N))/LOG(2)) + 4.
!     WSAVE(LENSAV)  real         prime factors of N and trigonometric values
!     LENWRK         integer      dimension of WORK; at least N
!     WORK(LENWRK)   real         workspace
!     INC            integer      increment between two consecutive elements in R
!     LENR           integer      dimension of  R; at least INC*(N-1) + 1.
!   input/output
!     R(LENR)        real         sequence to be transformed; transformed data
!
!###############################################################################
subroutine rfft1i ( n, wsave, lensav, ier )
!*****************************************************************************80
!
!! RFFT1I: initialization for RFFT1B and RFFT1F.
!
!  Discussion:
!
!    RFFT1I initializes array WSAVE for use in its companion routines 
!    RFFT1B and RFFT1F.  The prime factorization of N together with a
!    tabulation of the trigonometric functions are computed and stored
!    in array WSAVE.  Separate WSAVE arrays are required for different
!    values of N. 
!
!  License:
!
!    Licensed under the GNU General Public License (GPL).
!    Copyright (C) 1995-2004, Scientific Computing Division,
!    University Corporation for Atmospheric Research
!
!  Modified:
!
!    31 July 2011
!
!  Author:
!
!    Original FORTRAN77 version by Paul Swarztrauber, Richard Valent.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Paul Swarztrauber,
!    Vectorizing the Fast Fourier Transforms,
!    in Parallel Computations,
!    edited by G. Rodrigue,
!    Academic Press, 1982.
!
!    Paul Swarztrauber,
!    Fast Fourier Transform Algorithms for Vector Computers,
!    Parallel Computing, pages 45-63, 1984.
!
!  Parameters:
!
!    Input, integer  N, the length of the sequence to be
!    transformed.  The transform is most efficient when N is a product of 
!    small primes.
!
!    Output, real  WSAVE(LENSAV), containing the prime factors of
!    N and also containing certain trigonometric values which will be used in
!    routines RFFT1B or RFFT1F.
!
!    Input, integer  LENSAV, the dimension of the WSAVE array.
!    LENSAV must be at least N + INT(LOG(REAL(N))) + 4.
!
!    Output, integer  IER, error flag.
!    0, successful exit;
!    2, input parameter LENSAV not big enough.
!
  implicit none

  integer  lensav

  integer  ier
  integer  n
  real  wsave(lensav)

  ier = 0

  if (lensav < n + int(log( real ( n ) )/log( 2.0E+00 )) +4) then
    ier = 2
!   call xerfft ('rfft1i ', 3)
  end if

  if (n == 1) then
    return
  end if

  call rffti1 (n,wsave(1),wsave(n+1))

  return
end
!###############################################################################
subroutine rfft1f ( n, inc, r, lenr, wsave, lensav, work, lenwrk, ier )
!*****************************************************************************80
!
!! RFFT1F: real single precision forward fast Fourier transform, 1D.
!
!  Discussion:
!
!    RFFT1F computes the one-dimensional Fourier transform of a periodic
!    sequence within a real array.  This is referred to as the forward 
!    transform or Fourier analysis, transforming the sequence from physical
!    to spectral space.  This transform is normalized since a call to 
!    RFFT1F followed by a call to RFFT1B (or vice-versa) reproduces the 
!    original array within roundoff error.
!
!  License:
!
!    Licensed under the GNU General Public License (GPL).
!    Copyright (C) 1995-2004, Scientific Computing Division,
!    University Corporation for Atmospheric Research
!
!  Modified:
!
!    31 July 2011
!
!  Author:
!
!    Original FORTRAN77 version by Paul Swarztrauber, Richard Valent.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Paul Swarztrauber,
!    Vectorizing the Fast Fourier Transforms,
!    in Parallel Computations,
!    edited by G. Rodrigue,
!    Academic Press, 1982.
!
!    Paul Swarztrauber,
!    Fast Fourier Transform Algorithms for Vector Computers,
!    Parallel Computing, pages 45-63, 1984.
!
!  Parameters:
!
!    Input, integer  N, the length of the sequence to be
!    transformed.  The transform is most efficient when N is a product of 
!    small primes.
!
!    Input, integer  INC, the increment between the locations, in
!    array R, of two consecutive elements within the sequence. 
!
!    Input/output, real  R(LENR), on input, contains the sequence
!    to be transformed, and on output, the transformed data.
!
!    Input, integer  LENR, the dimension of the R array.
!    LENR must be at least INC*(N-1) + 1.
!
!    Input, real  WSAVE(LENSAV).  WSAVE's contents must be
!    initialized with a call to RFFT1I before the first call to routine RFFT1F
!    or RFFT1B for a given transform length N. 
!
!    Input, integer  LENSAV, the dimension of the WSAVE array.
!    LENSAV must be at least N + INT(LOG(REAL(N))) + 4.
!
!    Workspace, real  WORK(LENWRK).
!
!    Input, integer  LENWRK, the dimension of the WORK array.
!    LENWRK must be at least N. 
!
!    Output, integer  IER, error flag.
!    0, successful exit;
!    1, input parameter LENR not big enough:
!    2, input parameter LENSAV not big enough;
!    3, input parameter LENWRK not big enough.
!
  implicit none

  integer  lenr
  integer  lensav
  integer  lenwrk

  integer  ier
  integer  inc
  integer  n
  real  work(lenwrk)
  real  wsave(lensav)
  real  r(lenr)

  ier = 0

  if (lenr < inc*(n-1) + 1) then
    ier = 1
!   call xerfft ('rfft1f ', 6)
  else if (lensav < n + int(log( real ( n ) )/log( 2.0E+00 )) +4) then
    ier = 2
!   call xerfft ('rfft1f ', 8)
  else if (lenwrk < n) then
    ier = 3
!   call xerfft ('rfft1f ', 10)
  end if

  if (n == 1) then
    return
  end if

  call rfftf1 (n,inc,r,work,wsave,wsave(n+1))

  return
end
!###############################################################################
!###############################################################################
subroutine rffti1 ( n, wa, fac )
!*****************************************************************************80
!
!! RFFTI1 is an FFTPACK5.1 auxiliary routine.
!
!  License:
!
!    Licensed under the GNU General Public License (GPL).
!    Copyright (C) 1995-2004, Scientific Computing Division,
!    University Corporation for Atmospheric Research
!
!  Modified:
!
!    31 July 2011
!
!  Author:
!
!    Original FORTRAN77 version by Paul Swarztrauber, Richard Valent.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Paul Swarztrauber,
!    Vectorizing the Fast Fourier Transforms,
!    in Parallel Computations,
!    edited by G. Rodrigue,
!    Academic Press, 1982.
!
!    Paul Swarztrauber,
!    Fast Fourier Transform Algorithms for Vector Computers,
!    Parallel Computing, pages 45-63, 1984.
!
!  Parameters:
!
!    Input, integer  N, the number for which factorization
!    and other information is needed.
!
!    Output, real  WA(N), trigonometric information.
!
!    Output, real  FAC(15), factorization information.
!    FAC(1) is N, FAC(2) is NF, the number of factors, and FAC(3:NF+2) are the 
!    factors.
!
  implicit none

  integer  n

  real ( kind = 8 ) arg
  real ( kind = 8 ) argh
  real ( kind = 8 ) argld
  real  fac(15)
  real  fi
  integer  i
  integer  ib
  integer  ido
  integer  ii
  integer  ip
  integer  ipm
  integer  is
  integer  j
  integer  k1
  integer  l1
  integer  l2
  integer  ld
  integer  nf
  integer  nfm1
  integer  nl
  integer  nq
  integer  nr
  integer  ntry
  integer  ntryh(4)
  real ( kind = 8 ) tpi
  real  wa(n)

  save ntryh

  data ntryh / 4, 2, 3, 5 /

      nl = n
      nf = 0
      j = 0
  101 j = j+1
      if (j-4) 102,102,103
  102 ntry = ntryh(j)
      go to 104
  103 ntry = ntry+2
  104 nq = nl/ntry
      nr = nl-ntry*nq
      if (nr) 101,105,101
  105 nf = nf+1
      fac(nf+2) = ntry
      nl = nq
      if (ntry /= 2) go to 107
      if (nf == 1) go to 107
      do 106 i=2,nf
         ib = nf-i+2
         fac(ib+2) = fac(ib+1)
  106 continue
      fac(3) = 2
  107 if (nl /= 1) go to 104
      fac(1) = n
      fac(2) = nf
      tpi = 8.0D+00 * atan ( 1.0D+00 )
      argh = tpi / real ( n, kind = 8 )
      is = 0
      nfm1 = nf-1
      l1 = 1
      if (nfm1 == 0) return
      do 110 k1=1,nfm1
         ip = fac(k1+2)
         ld = 0
         l2 = l1*ip
         ido = n/l2
         ipm = ip-1
         do 109 j=1,ipm
            ld = ld+l1
            i = is
            argld = real ( ld, kind = 8 ) * argh
            fi = 0.0E+00
            do 108 ii=3,ido,2
               i = i+2
               fi = fi + 1.0E+00
               arg = fi*argld
	       wa(i-1) = cos ( arg )
	       wa(i) = sin ( arg )
  108       continue
            is = is+ido
  109    continue
         l1 = l2
  110 continue

  return
end
!###############################################################################
subroutine rfftf1 ( n, in, c, ch, wa, fac )
!*****************************************************************************80
!
!! RFFTF1 is an FFTPACK5.1 auxiliary routine.
!
!  License:
!
!    Licensed under the GNU General Public License (GPL).
!    Copyright (C) 1995-2004, Scientific Computing Division,
!    University Corporation for Atmospheric Research
!
!  Modified:
!
!    31 July 2011
!
!  Author:
!
!    Original FORTRAN77 version by Paul Swarztrauber, Richard Valent.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Paul Swarztrauber,
!    Vectorizing the Fast Fourier Transforms,
!    in Parallel Computations,
!    edited by G. Rodrigue,
!    Academic Press, 1982.
!
!    Paul Swarztrauber,
!    Fast Fourier Transform Algorithms for Vector Computers,
!    Parallel Computing, pages 45-63, 1984.
!
!  Parameters:
!
  implicit none

  integer  in
  integer  n

  real  c(in,*)
  real  ch(*)
  real  fac(15)
  integer  idl1
  integer  ido
  integer  ip
  integer  iw
  integer  ix2
  integer  ix3
  integer  ix4
  integer  j
  integer  k1
  integer  kh
  integer  l1
  integer  l2
  integer  modn
  integer  na
  integer  nf
  integer  nl
  real  sn
  real  tsn
  real  tsnm
  real  wa(n)

      nf = int ( fac(2) )
      na = 1
      l2 = n
      iw = n

      do 111 k1=1,nf
         kh = nf-k1
         ip = fac(kh+3)
         l1 = l2/ip
         ido = n/l2
         idl1 = ido*l1
         iw = iw-(ip-1)*ido
         na = 1-na
         if (ip /= 4) go to 102
         ix2 = iw+ido
         ix3 = ix2+ido
         if (na /= 0) go to 101
	     call r1f4kf (ido,l1,c,in,ch,1,wa(iw),wa(ix2),wa(ix3))
         go to 110
  101    call r1f4kf (ido,l1,ch,1,c,in,wa(iw),wa(ix2),wa(ix3))
         go to 110
  102    if (ip /= 2) go to 104
         if (na /= 0) go to 103
	 call r1f2kf (ido,l1,c,in,ch,1,wa(iw))
         go to 110
  103    call r1f2kf (ido,l1,ch,1,c,in,wa(iw))
         go to 110
  104    if (ip /= 3) go to 106
         ix2 = iw+ido
         if (na /= 0) go to 105
	 call r1f3kf (ido,l1,c,in,ch,1,wa(iw),wa(ix2))
         go to 110
  105    call r1f3kf (ido,l1,ch,1,c,in,wa(iw),wa(ix2))
         go to 110
  106    if (ip /= 5) go to 108
         ix2 = iw+ido
         ix3 = ix2+ido
         ix4 = ix3+ido
         if (na /= 0) go to 107
         call r1f5kf (ido,l1,c,in,ch,1,wa(iw),wa(ix2),wa(ix3),wa(ix4))
         go to 110
  107    call r1f5kf (ido,l1,ch,1,c,in,wa(iw),wa(ix2),wa(ix3),wa(ix4))
         go to 110
  108    if (ido == 1) na = 1-na
         if (na /= 0) go to 109
	 call r1fgkf (ido,ip,l1,idl1,c,c,c,in,ch,ch,1,wa(iw))
         na = 1
         go to 110
  109    call r1fgkf (ido,ip,l1,idl1,ch,ch,ch,1,c,c,in,wa(iw))
         na = 0
  110    l2 = l1
  111 continue

      sn = 1.0E+00 / real ( n )
      tsn =  2.0E+00  / real ( n )
      tsnm = -tsn
      modn = mod(n,2)
      nl = n-2
      if(modn /= 0) nl = n-1
      if (na /= 0) go to 120
      c(1,1) = sn*ch(1)
      do 118 j=2,nl,2
	 c(1,j) = tsn*ch(j)
	 c(1,j+1) = tsnm*ch(j+1)
  118 continue
      if(modn /= 0) return
      c(1,n) = sn*ch(n)
      return
  120 c(1,1) = sn*c(1,1)
      do 122 j=2,nl,2
	 c(1,j) = tsn*c(1,j)
	 c(1,j+1) = tsnm*c(1,j+1)
  122 continue
      if(modn /= 0) return
      c(1,n) = sn*c(1,n)

  return
end
!###############################################################################
!###############################################################################
subroutine r1f2kf (ido,l1,cc,in1,ch,in2,wa1)
!*****************************************************************************80
!
!! R1F1KF is an FFTPACK5.1 auxilliary function.
!
!  License:
!
!    Licensed under the GNU General Public License (GPL).
!    Copyright (C) 1995-2004, Scientific Computing Division,
!    University Corporation for Atmospheric Research
!
!  Modified:
!
!    31 July 2011
!
!  Author:
!
!    Original FORTRAN77 version by Paul Swarztrauber, Richard Valent.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Paul Swarztrauber,
!    Vectorizing the Fast Fourier Transforms,
!    in Parallel Computations,
!    edited by G. Rodrigue,
!    Academic Press, 1982.
!
!    Paul Swarztrauber,
!    Fast Fourier Transform Algorithms for Vector Computers,
!    Parallel Computing, pages 45-63, 1984.
!
!  Parameters:
!
  implicit none

  integer  ido
  integer  in1
  integer  in2
  integer  l1

  real  ch(in2,ido,2,l1)
  real  cc(in1,ido,l1,2)
  integer  i
  integer  ic
  integer  idp2
  integer  k
  real  wa1(ido)

  do k=1,l1
    ch(1,1,1,k) = cc(1,1,k,1)+cc(1,1,k,2)
    ch(1,ido,2,k) = cc(1,1,k,1)-cc(1,1,k,2)
  end do

      if (ido-2) 107,105,102
  102 idp2 = ido+2
      do 104 k=1,l1
         do i=3,ido,2
            ic = idp2-i
            ch(1,i,1,k) = cc(1,i,k,1)+(wa1(i-2)*cc(1,i,k,2) &
              -wa1(i-1)*cc(1,i-1,k,2))
            ch(1,ic,2,k) = (wa1(i-2)*cc(1,i,k,2) &
              -wa1(i-1)*cc(1,i-1,k,2))-cc(1,i,k,1)
            ch(1,i-1,1,k) = cc(1,i-1,k,1)+(wa1(i-2)*cc(1,i-1,k,2) &
              +wa1(i-1)*cc(1,i,k,2))
            ch(1,ic-1,2,k) = cc(1,i-1,k,1)-(wa1(i-2)*cc(1,i-1,k,2) &
              +wa1(i-1)*cc(1,i,k,2))
          end do
  104 continue
      if (mod(ido,2) == 1) return
  105 do 106 k=1,l1
         ch(1,1,2,k) = -cc(1,ido,k,2)
         ch(1,ido,1,k) = cc(1,ido,k,1)
  106 continue
  107 continue

  return
end
!###############################################################################
subroutine r1f3kf (ido,l1,cc,in1,ch,in2,wa1,wa2)
!*****************************************************************************80
!
!! R1F3KF is an FFTPACK5.1 auxilliary function.
!
!  License:
!
!    Licensed under the GNU General Public License (GPL).
!    Copyright (C) 1995-2004, Scientific Computing Division,
!    University Corporation for Atmospheric Research
!
!  Modified:
!
!    31 July 2011
!
!  Author:
!
!    Original FORTRAN77 version by Paul Swarztrauber, Richard Valent.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Paul Swarztrauber,
!    Vectorizing the Fast Fourier Transforms,
!    in Parallel Computations,
!    edited by G. Rodrigue,
!    Academic Press, 1982.
!
!    Paul Swarztrauber,
!    Fast Fourier Transform Algorithms for Vector Computers,
!    Parallel Computing, pages 45-63, 1984.
!
!  Parameters:
!
  implicit none

  integer  ido
  integer  in1
  integer  in2
  integer  l1

  real  arg
  real  cc(in1,ido,l1,3)
  real  ch(in2,ido,3,l1)
  integer  i
  integer  ic
  integer  idp2
  integer  k
  real  taui
  real  taur
  real  wa1(ido)
  real  wa2(ido)

  arg= 2.0E+00 * 4.0E+00 * atan( 1.0E+00 )/ 3.0E+00 
  taur=cos(arg)
  taui=sin(arg)

  do k=1,l1
    ch(1,1,1,k) = cc(1,1,k,1)+(cc(1,1,k,2)+cc(1,1,k,3))
    ch(1,1,3,k) = taui*(cc(1,1,k,3)-cc(1,1,k,2))
    ch(1,ido,2,k) = cc(1,1,k,1)+taur*(cc(1,1,k,2)+cc(1,1,k,3))
  end do

  if (ido == 1) then
    return
  end if

      idp2 = ido+2
      do 103 k=1,l1
         do 102 i=3,ido,2
            ic = idp2-i

            ch(1,i-1,1,k) = cc(1,i-1,k,1)+((wa1(i-2)*cc(1,i-1,k,2)+ &
             wa1(i-1)*cc(1,i,k,2))+(wa2(i-2)*cc(1,i-1,k,3)+wa2(i-1)* &
             cc(1,i,k,3)))

            ch(1,i,1,k) = cc(1,i,k,1)+((wa1(i-2)*cc(1,i,k,2)- &
             wa1(i-1)*cc(1,i-1,k,2))+(wa2(i-2)*cc(1,i,k,3)-wa2(i-1)* &
             cc(1,i-1,k,3)))

            ch(1,i-1,3,k) = (cc(1,i-1,k,1)+taur*((wa1(i-2)* &
             cc(1,i-1,k,2)+wa1(i-1)*cc(1,i,k,2))+(wa2(i-2)* &
             cc(1,i-1,k,3)+wa2(i-1)*cc(1,i,k,3))))+(taui*((wa1(i-2)* &
             cc(1,i,k,2)-wa1(i-1)*cc(1,i-1,k,2))-(wa2(i-2)* &
             cc(1,i,k,3)-wa2(i-1)*cc(1,i-1,k,3))))

            ch(1,ic-1,2,k) = (cc(1,i-1,k,1)+taur*((wa1(i-2)* &
             cc(1,i-1,k,2)+wa1(i-1)*cc(1,i,k,2))+(wa2(i-2)* &
             cc(1,i-1,k,3)+wa2(i-1)*cc(1,i,k,3))))-(taui*((wa1(i-2)* &
             cc(1,i,k,2)-wa1(i-1)*cc(1,i-1,k,2))-(wa2(i-2)* &
             cc(1,i,k,3)-wa2(i-1)*cc(1,i-1,k,3))))

            ch(1,i,3,k) = (cc(1,i,k,1)+taur*((wa1(i-2)*cc(1,i,k,2)- &
             wa1(i-1)*cc(1,i-1,k,2))+(wa2(i-2)*cc(1,i,k,3)-wa2(i-1)* &
             cc(1,i-1,k,3))))+(taui*((wa2(i-2)*cc(1,i-1,k,3)+wa2(i-1)* &
             cc(1,i,k,3))-(wa1(i-2)*cc(1,i-1,k,2)+wa1(i-1)* &
             cc(1,i,k,2))))

            ch(1,ic,2,k) = (taui*((wa2(i-2)*cc(1,i-1,k,3)+wa2(i-1)* &
             cc(1,i,k,3))-(wa1(i-2)*cc(1,i-1,k,2)+wa1(i-1)* &
             cc(1,i,k,2))))-(cc(1,i,k,1)+taur*((wa1(i-2)*cc(1,i,k,2)- &
             wa1(i-1)*cc(1,i-1,k,2))+(wa2(i-2)*cc(1,i,k,3)-wa2(i-1)* &
             cc(1,i-1,k,3))))
  102    continue
  103 continue

  return
end
!###############################################################################
subroutine r1f4kf (ido,l1,cc,in1,ch,in2,wa1,wa2,wa3)
!*****************************************************************************80
!
!! R1F4KF is an FFTPACK5.1 auxilliary function.
!
!  License:
!
!    Licensed under the GNU General Public License (GPL).
!    Copyright (C) 1995-2004, Scientific Computing Division,
!    University Corporation for Atmospheric Research
!
!  Modified:
!
!    31 July 2011
!
!  Author:
!
!    Original FORTRAN77 version by Paul Swarztrauber, Richard Valent.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Paul Swarztrauber,
!    Vectorizing the Fast Fourier Transforms,
!    in Parallel Computations,
!    edited by G. Rodrigue,
!    Academic Press, 1982.
!
!    Paul Swarztrauber,
!    Fast Fourier Transform Algorithms for Vector Computers,
!    Parallel Computing, pages 45-63, 1984.
!
!  Parameters:
!
  implicit none

  integer  ido
  integer  in1
  integer  in2
  integer  l1

  real  cc(in1,ido,l1,4)
  real  ch(in2,ido,4,l1)
  real  hsqt2
  integer  i
  integer  ic
  integer  idp2
  integer  k
  real  wa1(ido)
  real  wa2(ido)
  real  wa3(ido)

  hsqt2=sqrt( 2.0E+00 )/ 2.0E+00 

  do k=1,l1
    ch(1,1,1,k) = (cc(1,1,k,2)+cc(1,1,k,4))+(cc(1,1,k,1)+cc(1,1,k,3))
    ch(1,ido,4,k) = (cc(1,1,k,1)+cc(1,1,k,3))-(cc(1,1,k,2)+cc(1,1,k,4))
    ch(1,ido,2,k) = cc(1,1,k,1)-cc(1,1,k,3)
    ch(1,1,3,k) = cc(1,1,k,4)-cc(1,1,k,2)
  end do

      if (ido-2) 107,105,102
  102 idp2 = ido+2
      do 104 k=1,l1
         do 103 i=3,ido,2
            ic = idp2-i
            ch(1,i-1,1,k) = ((wa1(i-2)*cc(1,i-1,k,2)+wa1(i-1)* &
             cc(1,i,k,2))+(wa3(i-2)*cc(1,i-1,k,4)+wa3(i-1)* &
             cc(1,i,k,4)))+(cc(1,i-1,k,1)+(wa2(i-2)*cc(1,i-1,k,3)+ &
             wa2(i-1)*cc(1,i,k,3)))
            ch(1,ic-1,4,k) = (cc(1,i-1,k,1)+(wa2(i-2)*cc(1,i-1,k,3)+ &
             wa2(i-1)*cc(1,i,k,3)))-((wa1(i-2)*cc(1,i-1,k,2)+ &
             wa1(i-1)*cc(1,i,k,2))+(wa3(i-2)*cc(1,i-1,k,4)+ &
             wa3(i-1)*cc(1,i,k,4)))
            ch(1,i,1,k) = ((wa1(i-2)*cc(1,i,k,2)-wa1(i-1)* &
             cc(1,i-1,k,2))+(wa3(i-2)*cc(1,i,k,4)-wa3(i-1)* &
             cc(1,i-1,k,4)))+(cc(1,i,k,1)+(wa2(i-2)*cc(1,i,k,3)- &
             wa2(i-1)*cc(1,i-1,k,3)))
            ch(1,ic,4,k) = ((wa1(i-2)*cc(1,i,k,2)-wa1(i-1)* &
             cc(1,i-1,k,2))+(wa3(i-2)*cc(1,i,k,4)-wa3(i-1)* &
             cc(1,i-1,k,4)))-(cc(1,i,k,1)+(wa2(i-2)*cc(1,i,k,3)- &
             wa2(i-1)*cc(1,i-1,k,3)))
            ch(1,i-1,3,k) = ((wa1(i-2)*cc(1,i,k,2)-wa1(i-1)* &
             cc(1,i-1,k,2))-(wa3(i-2)*cc(1,i,k,4)-wa3(i-1)* &
             cc(1,i-1,k,4)))+(cc(1,i-1,k,1)-(wa2(i-2)*cc(1,i-1,k,3)+ &
             wa2(i-1)*cc(1,i,k,3)))
            ch(1,ic-1,2,k) = (cc(1,i-1,k,1)-(wa2(i-2)*cc(1,i-1,k,3)+ &
             wa2(i-1)*cc(1,i,k,3)))-((wa1(i-2)*cc(1,i,k,2)-wa1(i-1)* &
             cc(1,i-1,k,2))-(wa3(i-2)*cc(1,i,k,4)-wa3(i-1)* &
             cc(1,i-1,k,4)))
            ch(1,i,3,k) = ((wa3(i-2)*cc(1,i-1,k,4)+wa3(i-1)* &
             cc(1,i,k,4))-(wa1(i-2)*cc(1,i-1,k,2)+wa1(i-1)* &
             cc(1,i,k,2)))+(cc(1,i,k,1)-(wa2(i-2)*cc(1,i,k,3)- &
             wa2(i-1)*cc(1,i-1,k,3)))
            ch(1,ic,2,k) = ((wa3(i-2)*cc(1,i-1,k,4)+wa3(i-1)* &
             cc(1,i,k,4))-(wa1(i-2)*cc(1,i-1,k,2)+wa1(i-1)* &
             cc(1,i,k,2)))-(cc(1,i,k,1)-(wa2(i-2)*cc(1,i,k,3)- &
             wa2(i-1)*cc(1,i-1,k,3)))
  103    continue
  104 continue
      if (mod(ido,2) == 1) return
  105 continue
      do 106 k=1,l1
            ch(1,ido,1,k) = (hsqt2*(cc(1,ido,k,2)-cc(1,ido,k,4)))+cc(1,ido,k,1)
            ch(1,ido,3,k) = cc(1,ido,k,1)-(hsqt2*(cc(1,ido,k,2)-cc(1,ido,k,4)))
            ch(1,1,2,k) = (-hsqt2*(cc(1,ido,k,2)+cc(1,ido,k,4)))-cc(1,ido,k,3)
            ch(1,1,4,k) = (-hsqt2*(cc(1,ido,k,2)+cc(1,ido,k,4)))+cc(1,ido,k,3)
  106 continue
  107 continue

  return
end
!###############################################################################
subroutine r1f5kf (ido,l1,cc,in1,ch,in2,wa1,wa2,wa3,wa4)
!*****************************************************************************80
!
!! R1F5KF is an FFTPACK5.1 auxilliary function.
!
!  License:
!
!    Licensed under the GNU General Public License (GPL).
!    Copyright (C) 1995-2004, Scientific Computing Division,
!    University Corporation for Atmospheric Research
!
!  Modified:
!
!    31 July 2011
!
!  Author:
!
!    Original FORTRAN77 version by Paul Swarztrauber, Richard Valent.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Paul Swarztrauber,
!    Vectorizing the Fast Fourier Transforms,
!    in Parallel Computations,
!    edited by G. Rodrigue,
!    Academic Press, 1982.
!
!    Paul Swarztrauber,
!    Fast Fourier Transform Algorithms for Vector Computers,
!    Parallel Computing, pages 45-63, 1984.
!
!  Parameters:
!
  implicit none

  integer  ido
  integer  in1
  integer  in2
  integer  l1

  real  arg
  real  cc(in1,ido,l1,5)
  real  ch(in2,ido,5,l1)
  integer  i
  integer  ic
  integer  idp2
  integer  k
  real  ti11
  real  ti12
  real  tr11
  real  tr12
  real  wa1(ido)
  real  wa2(ido)
  real  wa3(ido)
  real  wa4(ido)

  arg= 2.0E+00 * 4.0E+00 * atan( 1.0E+00 ) / 5.0E+00
  tr11=cos(arg)
  ti11=sin(arg)
  tr12=cos( 2.0E+00 *arg)
  ti12=sin( 2.0E+00 *arg)

  do k=1,l1
    ch(1,1,1,k) = cc(1,1,k,1)+(cc(1,1,k,5)+cc(1,1,k,2))+ &
      (cc(1,1,k,4)+cc(1,1,k,3))
    ch(1,ido,2,k) = cc(1,1,k,1)+tr11*(cc(1,1,k,5)+cc(1,1,k,2))+ &
      tr12*(cc(1,1,k,4)+cc(1,1,k,3))
    ch(1,1,3,k) = ti11*(cc(1,1,k,5)-cc(1,1,k,2))+ti12* &
      (cc(1,1,k,4)-cc(1,1,k,3))
    ch(1,ido,4,k) = cc(1,1,k,1)+tr12*(cc(1,1,k,5)+cc(1,1,k,2))+ &
      tr11*(cc(1,1,k,4)+cc(1,1,k,3))
    ch(1,1,5,k) = ti12*(cc(1,1,k,5)-cc(1,1,k,2))-ti11* &
      (cc(1,1,k,4)-cc(1,1,k,3))
  end do

  if (ido == 1) return

      idp2 = ido+2
      do 103 k=1,l1
         do 102 i=3,ido,2
            ic = idp2-i

            ch(1,i-1,1,k) = cc(1,i-1,k,1)+((wa1(i-2)*cc(1,i-1,k,2)+ &
            wa1(i-1)*cc(1,i,k,2))+(wa4(i-2)*cc(1,i-1,k,5)+wa4(i-1)* &
            cc(1,i,k,5)))+((wa2(i-2)*cc(1,i-1,k,3)+wa2(i-1)* &
            cc(1,i,k,3))+(wa3(i-2)*cc(1,i-1,k,4)+ &
            wa3(i-1)*cc(1,i,k,4))) 

            ch(1,i,1,k) = cc(1,i,k,1)+((wa1(i-2)*cc(1,i,k,2)- &
             wa1(i-1)*cc(1,i-1,k,2))+(wa4(i-2)*cc(1,i,k,5)-wa4(i-1)* &
             cc(1,i-1,k,5)))+((wa2(i-2)*cc(1,i,k,3)-wa2(i-1)* &
             cc(1,i-1,k,3))+(wa3(i-2)*cc(1,i,k,4)-wa3(i-1)* &
             cc(1,i-1,k,4)))

            ch(1,i-1,3,k) = cc(1,i-1,k,1)+tr11* &
            ( wa1(i-2)*cc(1,i-1,k,2)+wa1(i-1)*cc(1,i,k,2) &
             +wa4(i-2)*cc(1,i-1,k,5)+wa4(i-1)*cc(1,i,k,5))+tr12* &
            ( wa2(i-2)*cc(1,i-1,k,3)+wa2(i-1)*cc(1,i,k,3) &
             +wa3(i-2)*cc(1,i-1,k,4)+wa3(i-1)*cc(1,i,k,4))+ti11* &
            ( wa1(i-2)*cc(1,i,k,2)-wa1(i-1)*cc(1,i-1,k,2) &
             -(wa4(i-2)*cc(1,i,k,5)-wa4(i-1)*cc(1,i-1,k,5)))+ti12* &
            ( wa2(i-2)*cc(1,i,k,3)-wa2(i-1)*cc(1,i-1,k,3) &
             -(wa3(i-2)*cc(1,i,k,4)-wa3(i-1)*cc(1,i-1,k,4)))

            ch(1,ic-1,2,k) = cc(1,i-1,k,1)+tr11* &
            ( wa1(i-2)*cc(1,i-1,k,2)+wa1(i-1)*cc(1,i,k,2) &
             +wa4(i-2)*cc(1,i-1,k,5)+wa4(i-1)*cc(1,i,k,5))+tr12* &
           ( wa2(i-2)*cc(1,i-1,k,3)+wa2(i-1)*cc(1,i,k,3) &
            +wa3(i-2)*cc(1,i-1,k,4)+wa3(i-1)*cc(1,i,k,4))-(ti11* &
            ( wa1(i-2)*cc(1,i,k,2)-wa1(i-1)*cc(1,i-1,k,2) &
             -(wa4(i-2)*cc(1,i,k,5)-wa4(i-1)*cc(1,i-1,k,5)))+ti12* &
            ( wa2(i-2)*cc(1,i,k,3)-wa2(i-1)*cc(1,i-1,k,3) &
             -(wa3(i-2)*cc(1,i,k,4)-wa3(i-1)*cc(1,i-1,k,4))))

            ch(1,i,3,k) = (cc(1,i,k,1)+tr11*((wa1(i-2)*cc(1,i,k,2)- &
             wa1(i-1)*cc(1,i-1,k,2))+(wa4(i-2)*cc(1,i,k,5)-wa4(i-1)* &
             cc(1,i-1,k,5)))+tr12*((wa2(i-2)*cc(1,i,k,3)-wa2(i-1)* &
             cc(1,i-1,k,3))+(wa3(i-2)*cc(1,i,k,4)-wa3(i-1)* &
             cc(1,i-1,k,4))))+(ti11*((wa4(i-2)*cc(1,i-1,k,5)+ &
             wa4(i-1)*cc(1,i,k,5))-(wa1(i-2)*cc(1,i-1,k,2)+wa1(i-1)* &
             cc(1,i,k,2)))+ti12*((wa3(i-2)*cc(1,i-1,k,4)+wa3(i-1)* &
             cc(1,i,k,4))-(wa2(i-2)*cc(1,i-1,k,3)+wa2(i-1)* &
             cc(1,i,k,3))))

            ch(1,ic,2,k) = (ti11*((wa4(i-2)*cc(1,i-1,k,5)+wa4(i-1)* &
             cc(1,i,k,5))-(wa1(i-2)*cc(1,i-1,k,2)+wa1(i-1)* &
             cc(1,i,k,2)))+ti12*((wa3(i-2)*cc(1,i-1,k,4)+wa3(i-1)* &
             cc(1,i,k,4))-(wa2(i-2)*cc(1,i-1,k,3)+wa2(i-1)* &
             cc(1,i,k,3))))-(cc(1,i,k,1)+tr11*((wa1(i-2)*cc(1,i,k,2)- &
             wa1(i-1)*cc(1,i-1,k,2))+(wa4(i-2)*cc(1,i,k,5)-wa4(i-1)* &
             cc(1,i-1,k,5)))+tr12*((wa2(i-2)*cc(1,i,k,3)-wa2(i-1)* &
             cc(1,i-1,k,3))+(wa3(i-2)*cc(1,i,k,4)-wa3(i-1)* &
             cc(1,i-1,k,4))))

            ch(1,i-1,5,k) = (cc(1,i-1,k,1)+tr12*((wa1(i-2)* &
             cc(1,i-1,k,2)+wa1(i-1)*cc(1,i,k,2))+(wa4(i-2)* &
             cc(1,i-1,k,5)+wa4(i-1)*cc(1,i,k,5)))+tr11*((wa2(i-2)* &
             cc(1,i-1,k,3)+wa2(i-1)*cc(1,i,k,3))+(wa3(i-2)* &
             cc(1,i-1,k,4)+wa3(i-1)*cc(1,i,k,4))))+(ti12*((wa1(i-2)* &
             cc(1,i,k,2)-wa1(i-1)*cc(1,i-1,k,2))-(wa4(i-2)* &
             cc(1,i,k,5)-wa4(i-1)*cc(1,i-1,k,5)))-ti11*((wa2(i-2)* &
             cc(1,i,k,3)-wa2(i-1)*cc(1,i-1,k,3))-(wa3(i-2)* &
             cc(1,i,k,4)-wa3(i-1)*cc(1,i-1,k,4))))

            ch(1,ic-1,4,k) = (cc(1,i-1,k,1)+tr12*((wa1(i-2)* &
             cc(1,i-1,k,2)+wa1(i-1)*cc(1,i,k,2))+(wa4(i-2)* &
             cc(1,i-1,k,5)+wa4(i-1)*cc(1,i,k,5)))+tr11*((wa2(i-2)* &
             cc(1,i-1,k,3)+wa2(i-1)*cc(1,i,k,3))+(wa3(i-2)* &
             cc(1,i-1,k,4)+wa3(i-1)*cc(1,i,k,4))))-(ti12*((wa1(i-2)* &
             cc(1,i,k,2)-wa1(i-1)*cc(1,i-1,k,2))-(wa4(i-2)* &
             cc(1,i,k,5)-wa4(i-1)*cc(1,i-1,k,5)))-ti11*((wa2(i-2)* &
             cc(1,i,k,3)-wa2(i-1)*cc(1,i-1,k,3))-(wa3(i-2)* &
             cc(1,i,k,4)-wa3(i-1)*cc(1,i-1,k,4))))

            ch(1,i,5,k) = (cc(1,i,k,1)+tr12*((wa1(i-2)*cc(1,i,k,2)- &
             wa1(i-1)*cc(1,i-1,k,2))+(wa4(i-2)*cc(1,i,k,5)-wa4(i-1)* &
             cc(1,i-1,k,5)))+tr11*((wa2(i-2)*cc(1,i,k,3)-wa2(i-1)* &
             cc(1,i-1,k,3))+(wa3(i-2)*cc(1,i,k,4)-wa3(i-1)* &
             cc(1,i-1,k,4))))+(ti12*((wa4(i-2)*cc(1,i-1,k,5)+ &
             wa4(i-1)*cc(1,i,k,5))-(wa1(i-2)*cc(1,i-1,k,2)+wa1(i-1)* &
             cc(1,i,k,2)))-ti11*((wa3(i-2)*cc(1,i-1,k,4)+wa3(i-1)* &
             cc(1,i,k,4))-(wa2(i-2)*cc(1,i-1,k,3)+wa2(i-1)* &
             cc(1,i,k,3))))

            ch(1,ic,4,k) = (ti12*((wa4(i-2)*cc(1,i-1,k,5)+wa4(i-1)* &
             cc(1,i,k,5))-(wa1(i-2)*cc(1,i-1,k,2)+wa1(i-1)* &
             cc(1,i,k,2)))-ti11*((wa3(i-2)*cc(1,i-1,k,4)+wa3(i-1)* &
             cc(1,i,k,4))-(wa2(i-2)*cc(1,i-1,k,3)+wa2(i-1)* &
             cc(1,i,k,3))))-(cc(1,i,k,1)+tr12*((wa1(i-2)*cc(1,i,k,2)- &
             wa1(i-1)*cc(1,i-1,k,2))+(wa4(i-2)*cc(1,i,k,5)-wa4(i-1)* &
             cc(1,i-1,k,5)))+tr11*((wa2(i-2)*cc(1,i,k,3)-wa2(i-1)* &
             cc(1,i-1,k,3))+(wa3(i-2)*cc(1,i,k,4)-wa3(i-1)* &
             cc(1,i-1,k,4))))

  102    continue
  103 continue

  return
end
!###############################################################################
subroutine r1fgkf (ido,ip,l1,idl1,cc,c1,c2,in1,ch,ch2,in2,wa)
!*****************************************************************************80
!
!! R1FGKF is an FFTPACK5.1 auxilliary function.
!
!  License:
!
!    Licensed under the GNU General Public License (GPL).
!    Copyright (C) 1995-2004, Scientific Computing Division,
!    University Corporation for Atmospheric Research
!
!  Modified:
!
!    31 July 2011
!
!  Author:
!
!    Original FORTRAN77 version by Paul Swarztrauber, Richard Valent.
!    FORTRAN90 version by John Burkardt.
!
!  Reference:
!
!    Paul Swarztrauber,
!    Vectorizing the Fast Fourier Transforms,
!    in Parallel Computations,
!    edited by G. Rodrigue,
!    Academic Press, 1982.
!
!    Paul Swarztrauber,
!    Fast Fourier Transform Algorithms for Vector Computers,
!    Parallel Computing, pages 45-63, 1984.
!
!  Parameters:
!
  implicit none

  integer  idl1
  integer  ido
  integer  in1
  integer  in2
  integer  ip
  integer  l1

  real  ai1
  real  ai2
  real  ar1
  real  ar1h
  real  ar2
  real  ar2h
  real  arg
  real  c1(in1,ido,l1,ip)
  real  c2(in1,idl1,ip)
  real  cc(in1,ido,ip,l1)
  real  ch(in2,ido,l1,ip)
  real  ch2(in2,idl1,ip)
  real  dc2
  real  dcp
  real  ds2
  real  dsp
  integer  i
  integer  ic
  integer  idij
  integer  idp2
  integer  ik
  integer  ipp2
  integer  ipph
  integer  is
  integer  j
  integer  j2
  integer  jc
  integer  k
  integer  l
  integer  lc
  integer  nbd
  real  tpi
  real  wa(ido)

  tpi= 2.0E+00 * 4.0E+00 * atan( 1.0E+00 )
  arg = tpi/real ( ip )
  dcp = cos(arg)
  dsp = sin(arg)
  ipph = (ip+1)/2
  ipp2 = ip+2
  idp2 = ido+2
  nbd = (ido-1)/2

      if (ido == 1) go to 119

      do ik=1,idl1
         ch2(1,ik,1) = c2(1,ik,1)
      end do

      do j=2,ip
         do k=1,l1
            ch(1,1,k,j) = c1(1,1,k,j)
         end do
      end do

      if ( l1 < nbd ) go to 107
      is = -ido
      do 106 j=2,ip
         is = is+ido
         idij = is
         do 105 i=3,ido,2
            idij = idij+2
            do 104 k=1,l1
               ch(1,i-1,k,j) = wa(idij-1)*c1(1,i-1,k,j)+wa(idij)*c1(1,i,k,j)
               ch(1,i,k,j) = wa(idij-1)*c1(1,i,k,j)-wa(idij)*c1(1,i-1,k,j)
  104       continue
  105    continue
  106 continue
      go to 111
  107 is = -ido
      do 110 j=2,ip
         is = is+ido
         do 109 k=1,l1
            idij = is
            do 108 i=3,ido,2
               idij = idij+2
               ch(1,i-1,k,j) = wa(idij-1)*c1(1,i-1,k,j)+wa(idij)*c1(1,i,k,j)
               ch(1,i,k,j) = wa(idij-1)*c1(1,i,k,j)-wa(idij)*c1(1,i-1,k,j)
  108       continue
  109    continue
  110 continue
  111 if (nbd < l1) go to 115
      do 114 j=2,ipph
         jc = ipp2-j
         do 113 k=1,l1
            do 112 i=3,ido,2
               c1(1,i-1,k,j) = ch(1,i-1,k,j)+ch(1,i-1,k,jc)
               c1(1,i-1,k,jc) = ch(1,i,k,j)-ch(1,i,k,jc)
               c1(1,i,k,j) = ch(1,i,k,j)+ch(1,i,k,jc)
               c1(1,i,k,jc) = ch(1,i-1,k,jc)-ch(1,i-1,k,j)
  112       continue
  113    continue
  114 continue
      go to 121
  115 do 118 j=2,ipph
         jc = ipp2-j
         do 117 i=3,ido,2
            do 116 k=1,l1
               c1(1,i-1,k,j) = ch(1,i-1,k,j)+ch(1,i-1,k,jc)
               c1(1,i-1,k,jc) = ch(1,i,k,j)-ch(1,i,k,jc)
               c1(1,i,k,j) = ch(1,i,k,j)+ch(1,i,k,jc)
               c1(1,i,k,jc) = ch(1,i-1,k,jc)-ch(1,i-1,k,j)
  116       continue
  117    continue
  118 continue
      go to 121
  119 do 120 ik=1,idl1
         c2(1,ik,1) = ch2(1,ik,1)
  120 continue
  121 do 123 j=2,ipph
         jc = ipp2-j
         do 122 k=1,l1
            c1(1,1,k,j) = ch(1,1,k,j)+ch(1,1,k,jc)
            c1(1,1,k,jc) = ch(1,1,k,jc)-ch(1,1,k,j)
  122    continue
  123 continue

      ar1 = 1.0E+00
      ai1 = 0.0E+00
      do 127 l=2,ipph
         lc = ipp2-l
         ar1h = dcp*ar1-dsp*ai1
         ai1 = dcp*ai1+dsp*ar1
         ar1 = ar1h
         do 124 ik=1,idl1
            ch2(1,ik,l) = c2(1,ik,1)+ar1*c2(1,ik,2)
            ch2(1,ik,lc) = ai1*c2(1,ik,ip)
  124    continue
         dc2 = ar1
         ds2 = ai1
         ar2 = ar1
         ai2 = ai1
         do 126 j=3,ipph
            jc = ipp2-j
            ar2h = dc2*ar2-ds2*ai2
            ai2 = dc2*ai2+ds2*ar2
            ar2 = ar2h
            do 125 ik=1,idl1
               ch2(1,ik,l) = ch2(1,ik,l)+ar2*c2(1,ik,j)
               ch2(1,ik,lc) = ch2(1,ik,lc)+ai2*c2(1,ik,jc)
  125       continue
  126    continue
  127 continue
      do 129 j=2,ipph
         do 128 ik=1,idl1
            ch2(1,ik,1) = ch2(1,ik,1)+c2(1,ik,j)
  128    continue
  129 continue

      if (ido < l1) go to 132
      do 131 k=1,l1
         do 130 i=1,ido
            cc(1,i,1,k) = ch(1,i,k,1)
  130    continue
  131 continue
      go to 135
  132 do 134 i=1,ido
         do 133 k=1,l1
            cc(1,i,1,k) = ch(1,i,k,1)
  133    continue
  134 continue
  135 do 137 j=2,ipph
         jc = ipp2-j
         j2 = j+j
         do 136 k=1,l1
            cc(1,ido,j2-2,k) = ch(1,1,k,j)
            cc(1,1,j2-1,k) = ch(1,1,k,jc)
  136    continue
  137 continue
      if (ido == 1) return
      if (nbd < l1) go to 141
      do 140 j=2,ipph
         jc = ipp2-j
         j2 = j+j
         do 139 k=1,l1
            do 138 i=3,ido,2
               ic = idp2-i
               cc(1,i-1,j2-1,k) = ch(1,i-1,k,j)+ch(1,i-1,k,jc)
               cc(1,ic-1,j2-2,k) = ch(1,i-1,k,j)-ch(1,i-1,k,jc)
               cc(1,i,j2-1,k) = ch(1,i,k,j)+ch(1,i,k,jc)
               cc(1,ic,j2-2,k) = ch(1,i,k,jc)-ch(1,i,k,j)
  138       continue
  139    continue
  140 continue
      return
  141 do 144 j=2,ipph
         jc = ipp2-j
         j2 = j+j
         do 143 i=3,ido,2
            ic = idp2-i
            do 142 k=1,l1
               cc(1,i-1,j2-1,k) = ch(1,i-1,k,j)+ch(1,i-1,k,jc)
               cc(1,ic-1,j2-2,k) = ch(1,i-1,k,j)-ch(1,i-1,k,jc)
               cc(1,i,j2-1,k) = ch(1,i,k,j)+ch(1,i,k,jc)
               cc(1,ic,j2-2,k) = ch(1,i,k,jc)-ch(1,i,k,j)
  142       continue
  143    continue
  144 continue

  return
end
!###############################################################################
!###############################################################################
